// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.jar.rb_common.exception;


import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;

import java.util.ArrayList;

/**
 * @author vic.fan
 * @version V1.0
 * @ClassName AbstractRuntimeException
 * @Description Exception 抽象类
 * @date 2013-8-19
 */
public abstract class AbstractRuntimeException extends RuntimeException {

    /**
     * @Fields serialVersionUID
     */
    private static final long serialVersionUID = -7569397491617300988L;

    /**
     * 错误消息列表
     */
    private final ArrayList<ErrorMessage> errorMessages = new ArrayList<ErrorMessage>();

    /**
     * <p>Title: AbstractRuntimeException </p>
     * <p>Description: 构造函数 </p>
     *
     * @param errorCode
     */
    public AbstractRuntimeException(String errorCode) {
        super(errorCode);
        addExceptionMessage(errorCode);
    }

    /**
     * <p>Title: AbstractRuntimeException </p>
     * <p>Description: 构造函数 </p>
     *
     * @param errorCode
     * @param args
     */
    public AbstractRuntimeException(String errorCode, String[] args) {
        super(errorCode);
        addExceptionMessage(errorCode, args);
    }

    /**
     * <p>Title: AbstractRuntimeException </p>
     * <p>Description: 构造函数 </p>
     *
     * @param errorCode
     * @param cause
     */
    public AbstractRuntimeException(String errorCode, Throwable cause) {
        super(errorCode, cause);
        addExceptionMessage(errorCode);
    }

    /**
     * <p>Title: AbstractRuntimeException </p>
     * <p>Description: 构造函数 </p>
     *
     * @param errorCode
     * @param args
     * @param cause
     */
    public AbstractRuntimeException(String errorCode, String[] args, Throwable cause) {
        super(errorCode, cause);
        addExceptionMessage(errorCode, args);
    }

    /**
     * <p>Title: AbstractRuntimeException </p>
     * <p>Description: 构造函数 </p>
     *
     * @param errorCode
     */
    public AbstractRuntimeException(MessageId messageId) {
        super(messageId.toString());
        addExceptionMessage(messageId);
    }

    /**
     * <p>Title: AbstractRuntimeException </p>
     * <p>Description: 构造函数 </p>
     *
     * @param errorCode
     * @param args
     */
    public AbstractRuntimeException(MessageId messageId, String[] args) {
        super(messageId.toString());
        addExceptionMessage(messageId, args);
    }

    /**
     * @param errorCode
     * @Title: addExceptionMessage
     * @Description: 添加错误
     */
    public void addExceptionMessage(String errorCode) {
        ErrorMessage errorMessage = new ErrorMessage();
        errorMessage.setErrorCode(errorCode);
        errorMessages.add(errorMessage);
    }

    /**
     * @param errorCode
     * @param args
     * @Title: addExceptionMessage
     * @Description: 添加错误
     */
    public void addExceptionMessage(String errorCode, String[] args) {
        ErrorMessage errorMessage = new ErrorMessage();
        errorMessage.setErrorCode(errorCode);
        errorMessage.setArgs(args);
        errorMessages.add(errorMessage);
    }

    /**
     * @param messageId
     * @Title: addExceptionMessage
     * @Description: 添加错误
     */
    public void addExceptionMessage(MessageId messageId) {
        ErrorMessage errorMessage = new ErrorMessage();
        errorMessage.setMessageId(messageId);
        errorMessage.setErrorCode(messageId.toString());
        errorMessages.add(errorMessage);
    }

    /**
     * @param messageId
     * @param args
     * @Title: addExceptionMessage
     * @Description: 添加错误
     */
    public void addExceptionMessage(MessageId messageId, String[] args) {
        ErrorMessage errorMessage = new ErrorMessage();
        errorMessage.setMessageId(messageId);
        errorMessage.setErrorCode(messageId.toString());
        errorMessage.setArgs(args);
        errorMessages.add(errorMessage);
    }

    /**
     * @return ErrorMessage[]
     * @Title: getErrorMessages
     * @Description: 取得错误列表
     */
    public ErrorMessage[] getErrorMessages() {
        return errorMessages.toArray(new ErrorMessage[errorMessages.size()]);
    }

    /**
     * @param errorCode
     * @return ErrorMessage
     * @Title: getErrorMessage
     * @Description: 取得错误
     */
    public ErrorMessage getErrorMessage(String errorCode) {
        for (ErrorMessage errorMessage : errorMessages) {
            if (errorMessage.getErrorCode().equals(errorCode)) {
                return errorMessage;
            }
        }
        return null;
    }

    /**
     * @return ErrorMessage
     * @Title: getErrorMessage
     * @Description: 取得唯一信息
     */
    public ErrorMessage getErrorMessage() {
        if (errorMessages.size() > 0) {
            return errorMessages.get(0);
        }
        return null;
    }
}
