// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.jar.rb_platform.mapper;

import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import com.nationrel.modules.project.jar.rb_platform.entity.SysStateConfig;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @version V1.0
 * @ClassName SysStateConfigMapper
 * @Description 状态配置表 Mapper接口
 * @Author
 * @CreateDate 2013-11-21
 */
public interface SysStateConfigMapper extends BaseMapper<SysStateConfig, String> {

    /**
     * @param sysStateConfig 搜索条件
     * @return SysStateConfig
     * @Title: selectByModuleIdAndStateCode
     * @Description: 查找指定状态配置
     */
    SysStateConfig selectByModuleIdAndStateCode(SysStateConfig sysStateConfig);

    /**
     * @param moduleId  业务数据类型
     * @param stateCode 当前状态
     * @param stateType 状态类型
     * @return SysStateConfig
     * @Title: getNextStateList
     * @Description: 获取当前状态的下一状态
     */
    List<SysStateConfig> getNextStateList(@Param("moduleId") String moduleId,
                                          @Param("stateCode") String stateCode,
                                          @Param("stateType") String stateType);

    /**
     * @param moduleId  业务数据类型
     * @param stateType 状态类型
     * @return String
     * @Title: selectByStateType
     * @Description: 按状态类型取得审批状态列表
     */
    List<SysStateConfig> selectByStateType(@Param("moduleId") String moduleId,
                                           @Param("stateType") String stateType);

    /**
     * @param moduleId    业务数据类型
     * @param releaseFlag 是否发布
     * @return List<SysStateConfig>
     * @Title: selectPubStateByModuleId
     * @Description: 根据业务数据类型取得发布的审批状态
     */
    SysStateConfig selectPubStateByModuleId(@Param("moduleId") String moduleId,
                                            @Param("releaseFlag") String releaseFlag);

    /**
     * @param moduleId      业务数据
     * @param stateType     状态类型
     * @param maxState      最大可取状态
     * @param minState      最小可取状态
     * @param excludeStates 排除状态
     * @return List<SysStateConfig>
     * @Title: selectSysStateConfigList
     * @Description: 根据指定参数取得下列框列表
     */
    List<SysStateConfig> selectSysStateConfigList(@Param("moduleId") String moduleId,
                                                  @Param("stateType") String stateType,
                                                  @Param("maxState") String maxState,
                                                  @Param("minState") String minState,
                                                  @Param("excludeStates") String[] excludeStates);

    /**
     * @param moduleId  模块标识
     * @param lifeState 业务周期状态编码
     * @param stateType 状态类型
     * @return List<SysStateConfig>
     * @Title: getPrevStateList
     * @Description: 获取生命周期上一状态
     */
    List<SysStateConfig> getPrevStateList(
            @Param("moduleId") String moduleId,
            @Param("lifeState") String lifeState,
            @Param("stateType") String stateType);

}
