// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.jar.action;


import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.exception.ErrorMessage;

/**
 * @ClassName MessageSupport
 * @Description 消息处理辅助类
 * @Author 罗昊
 * @CreateDate 2013-5-28
 * @Version V1.0
 */
public interface MessageSupport {

    /**
    * @Title: addErrorMessage
    * @Description: 添加错误消息
    * @param messageId
    *            消息Id
    */
    void addErrorMessage(MessageId messageId);

   /**
    * @Title: addErrorMessage
    * @Description: 添加错误消息
    * @param messageId
    *            消息Id
    * @param args
    *            消息参数定义
    */
    void addErrorMessage(MessageId messageId, String... args);

   /**
    * @Title: addWarningMessage
    * @Description: 添加警告消息
    * @param messageId
    *            消息Id
    */
    void addWarningMessage(MessageId messageId);

    /**
    * @Title: addWarningMessage
    * @Description: 添加警告消息
    * @param messageId
    *            消息Id
    * @param args
    *            消息参数定义
    */
    void addWarningMessage(MessageId messageId, String... args);

    /**
    * @Title: addInfoMessage
    * @Description: 添加消息
    * @param messageId
    *            消息Id
    */
    void addInfoMessage(MessageId messageId);

    /**
    * @Title: addInfoMessage
    * @Description: 添加消息
    * @param messageId
    *            消息Id
    * @param args
    *            消息参数定义
    */
    void addInfoMessage(MessageId messageId, String... args);

    /**
     * @Title: addErrorMessage
     * @Description: 添加错误信息
     * @param errorMessage void
     */
    void addErrorMessage(ErrorMessage errorMessage);

    /**
     * @Title: addWarningMessage
     * @Description: 添加警告信息
     * @param errorMessage void
     */
    void addWarningMessage(ErrorMessage errorMessage);

    /**
     * @Title: addExceptionMessage
     * @Description: 处理异常信息(根据信息前缀W或E决定信息的形式)
     * @param errorMessage void
     */
    void addExceptionMessage(ErrorMessage errorMessage);

    /**
    * @param fieldName
    * @Title: addErrorMessage
    * @Description: 获取Key相应的错误信息
    * @param key 相应的信息，如果没有相对应的信息则返回null
    * @return void
    */
    String getText(String key);

    /**
    * @Title: hasErrors
    * @Description: 是否有错误
    * @return boolean
    */
    boolean hasErrors();

}
