package com.nationrel.modules.project.jar.util;


import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import org.apache.commons.lang3.StringUtils;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 日期格式通用工具类
 *
 * @author ted
 */
public class DateUtils {

    /**
     * <p>Title: </p>
     * <p>Description: </p>
     */
    private DateUtils() {
        super();
    }

    /**
     * @param dtObj
     * @param format
     * @return String
     * @throws SQLException
     * @Title: format
     * @Description: 日期转化为字符串
     */
    public static String format(final Object dtObj, final String format) {
        if (dtObj == null) {
            return Constants.BLANK;
        }

        Date dtTmp = new Date();

        // 当日期不为空时,日期转化为字符串
        if (dtObj instanceof Timestamp) {
            dtTmp = new Date(((Timestamp) dtObj).getTime());
        } else if (dtObj instanceof Date) {
            dtTmp = (Date) dtObj;
        } else {
            dtTmp = null;
        }

        if (dtTmp == null) {
            return Constants.BLANK;
        }

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(dtTmp);
    }

    /**
     * @param dtObj
     * @return Date
     * @Title: convertObjtoDate
     * @Description: 将对象转为日期
     */
    public static Date convertObjtoDate(Object dtObj) {
        if (dtObj == null) {
            return null;
        }

        Date dtTmp = new Date();

        // 当日期不为空时,日期转化为字符串
        if (dtObj instanceof Timestamp) {
            dtTmp = new Date(((Timestamp) dtObj).getTime());
        } else if (dtObj instanceof Date) {
            dtTmp = (Date) dtObj;
        } else {
            dtTmp = null;
        }

        return dtTmp;
    }

    /**
     * @param obj
     * @return Date
     * @throws ParseException
     * @Title:
     * @Description: 字符串转化为日期
     */
    public static Date convertDate(String obj) {
        if (obj == null || Constants.BLANK.equals(obj)) {
            return null;
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                return sdf.parse(obj);
            } catch (ParseException e) {
                sdf = new SimpleDateFormat("yyyy/MM/dd");
                try {
                    return sdf.parse(obj);
                } catch (ParseException e1) {
                    return null;
                }
            }
        }
    }

    /**
     * @param obj
     * @return Date
     * @throws ParseException
     * @Title:
     * @Description: 字符串转化为日期
     */
    public static Date convertDate(String obj, String formatString) {
        if (obj == null || Constants.BLANK.equals(obj)) {
            return null;
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat(formatString);
            try {
                return sdf.parse(obj);
            } catch (ParseException e) {
                return null;
            }
        }
    }

    /**
     * @param obj
     * @return Date
     * @throws ParseException
     * @Title:
     * @Description: 字符串转化为日期
     */
    public static String format(long lngDate, String formatString) {
        if (lngDate < 0) {
            return "-";
        } else {
            Date dttmp = new Date(lngDate);
            SimpleDateFormat sdf = new SimpleDateFormat(formatString);
            return sdf.format(dttmp);
        }
    }

    /**
     * @return boolean
     * @Title: isNotDateEquals
     * @Description: 比较两日期是否相等，不包括时间
     */
    public static boolean isNotDateEquals(Date dt1, Date dt2) {
        boolean ret = false;
        if (dt1 == null && dt2 == null) {

        } else if (dt1 == null || dt2 == null) {
            ret = true;
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            ret = !sdf.format(dt1).equals(sdf.format(dt2));
        }
        return ret;
    }

    /**
     * @return boolean
     * @Title: compareTo
     * @Description: 比较两日期是否大小，不包括时间
     */
    public static int compareTo(Date dt1, Date dt2) {
        int ret = 0;
        if (dt1 == null && dt2 == null) {

        } else if (dt1 == null || dt2 == null) {
            ret = -1;
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            ret = sdf.format(dt1).compareTo(sdf.format(dt2));
        }
        return ret;
    }

    /**
     * @param dtObj
     * @return String
     * @Title: convertDatetoString
     * @Description: 日期转化为字符串
     */
    public static String convertDatetoString(Date dtObj) {
        if (dtObj == null) {
            return Constants.BLANK;
        }

        // 当日期不为空时,日期转化为字符串
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(dtObj);
    }

    /**
     * @param dtObj
     * @return String
     * @Title: convertTimetoString
     * @Description: 日期转化为字符串，精确到时分秒
     */
    public static String convertTimetoString(Date dtObj) {
        if (dtObj == null) {
            return Constants.BLANK;
        }

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return format.format(dtObj);
    }

    /**
     * @param dtOld
     * @param add
     * @return Date
     * @Title: addDay
     * @Description: 在指定日期上加天数
     */
    public static Date addDay(Date dtOld, int add) {
        if (dtOld == null) {
            return null;
        }
        Calendar cdate = Calendar.getInstance();
        cdate.setTime(dtOld);
        cdate.add(Calendar.DATE, add);
        return cdate.getTime();
    }

    /**
     * @return Date
     * @Title: addTimeMillis
     * @Description: 在指定日期上加指定的毫秒数
     */
    public static Date addTimeMillis(Date dtOld, long millis) {
        return new Date(dtOld.getTime() + millis);
    }

    /**
     * @param sDate
     * @param eDate
     * @return int
     * @throws Exception
     * @Title: dayBetween
     * @Description: 计算两个日期间的天数
     */
    public static int dayBetween(Date maxDate, Date minDate) throws Exception {
        long dayMillisecond = 86400000;
        long dayBetween = maxDate.getTime() - minDate.getTime();
        return Integer.parseInt(String.valueOf(dayBetween / dayMillisecond));
    }

    /**
     * @param maxDate
     * @param minDate
     * @param intervalCnt
     * @return long
     * @Title: splitDataInterval
     * @Description: 将时间间隔划分为指定数的大小区间
     */
    public static long splitDateInterval(Date maxDate, Date minDate, int intervalCnt) {
        if (maxDate == null || minDate == null) {
            return 0;
        }
        return (maxDate.getTime() - minDate.getTime()) / 9;
    }

    /**
     * 减去指定月份的日期
     *
     * @param dtTmp  日期1
     * @param iCount 减掉的月份数
     * @return
     */
    public static Date minusMonth(Date dtTmp, int iCount) {
        if (dtTmp == null) return null;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dtTmp);
        calendar.add(Calendar.MONTH, -iCount);
        return calendar.getTime();
    }

    /**
     * 字符串月减去指定月数
     *
     * @param strTmp 字符串月
     * @param iCount 减掉的月份数
     * @return 业务月前一月
     * @throws ParseException 日期解析错误
     */
    public static String minusMonth(String strTmp, int iCount) throws ParseException {
        if (StringUtils.isEmpty(strTmp)) return "";
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat stringFormat = new SimpleDateFormat("yyyyMM");
        Date curDate = dateFormat.parse(strTmp + "01");

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(curDate);
        calendar.add(Calendar.MONTH, -iCount);
        return stringFormat.format(calendar.getTime());
    }

    /**
     * 字符串月加上指定月数
     *
     * @param strTmp 字符串月
     * @param iCount 添加的月份数
     * @return 业务月后一月
     * @throws ParseException 日期解析错误
     */
    public static String plusMonth(String strTmp, int iCount) throws ParseException {
        if (StringUtils.isEmpty(strTmp)) return "";
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat stringFormat = new SimpleDateFormat("yyyyMM");
        Date curDate = dateFormat.parse(strTmp + "01");

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(curDate);
        calendar.add(Calendar.MONTH, +iCount);
        return stringFormat.format(calendar.getTime());
    }
}
