// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.jar.util;

import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.service.RedisService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.SpringContextUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.ArrayList;
import java.util.List;

/**
 * @author jrx
 * @version V2.0
 * @ClassName SessionUtil
 * @Description 提供Session相关处理
 * @date 2022-02-25
 */
public class SessionUtil {

    private static RedisService redisService;

    /**
     * <p>Title: </p>
     * <p>Description: </p>
     */
    private SessionUtil() {
        super();
    }

    public static ServletRequestAttributes getRequestAttributes() {
        return (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    }


    /**
     * @return Map<String, Object>
     * @Title: getSession
     * @Description: 获取Session
     */
    private static HttpSession getSession() {
        HttpServletRequest request = getRequestAttributes().getRequest();
        return request.getSession();
    }

    public static void getRedisService() {
        if (redisService == null) {
            redisService = SpringContextUtils.getBean(RedisService.class);
        }
    }

    /**
     * @return 用户信息
     * @Title: getUserInfo
     * @Description: 用户信息获取
     */
    public static LoginUser getUserInfo() {
        Object value = SessionUtil.getAttribute(SessionKeys.USER_INFO);
        if (value instanceof LoginUser userInfoListener) {
            return userInfoListener;
        }
        return null;
    }

    /**
     Listener
     * @Title: setUserInfoListener
     * @Description: 用户信息监听设置，在session失效时做相应的下线处理。
     */
    public static void setUserInfoListener(LoginUser sysUser) {
        SessionUtil.setAttribute(SessionKeys.USER_INFO, sysUser);
    }

    /**
     Listener
     * @Title: getUserInfoListener
     * @Description: 用户信息监听设置，在session失效时做相应的下线处理。
     */
    public static LoginUser getUserInfoListener() {
        return (LoginUser) SessionUtil.getAttribute(SessionKeys.USER_INFO);
    }

    /**
     Listener
     * @Title: removeUserInfoListener
     * @Description: 用户信息监听设置，在session失效时做相应的下线处理。
     */
    public static void removeUserInfoListener() {
        SessionUtil.removeAttribute(SessionKeys.USER_INFO);
    }

    /**
     * @param key
     * @param value
     * @Title: setAttribute
     * @Description: 属性设置
     * 防止Session混乱，此方法不公开
     */
    private static void setAttribute(String key, Object value) {
//        SystemCurrentUser curUser = ContextUtil.getCurrentUser();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            redisService.putObject(sysUser.getUsername() + key, value, 60 * 60 * 8);
        }
    }

    /**
     * @param key
     * @return Object
     * @Title: getAttribute
     * @Description: 属性获取
     * 防止Session混乱，此方法不公开
     */
    private static Object getAttribute(String key) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            return redisService.getObject(sysUser.getUsername() + key, Object.class);
        }
        return null;
    }

    /**
     * @param key
     * @Title: removeAttribute
     * @Description: 属性清除
     * 防止Session混乱，此方法不公开
     */
    private static void removeAttribute(String key) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            redisService.delete(sysUser.getUsername() + key);
        }
    }

    /**
     * @param key   已定义key
     * @param value void
     * @Title: setAttribute
     * @Description: 属性设置
     */
    public static void setAttribute(SessionKeys key, Object value) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            redisService.putObject(sysUser.getUsername() + key.toString(), value, 60 * 60 * 8);
        }
    }

    /**
     * @param key 已定义key
     * @return Object
     * @Title: getAttribute
     * @Description: 属性获取
     */
    public static Object getAttribute(SessionKeys key) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            return redisService.getObject(sysUser.getUsername() + key.toString(), Object.class);
        }
        return null;
    }

    /**
     * @param key 已定义key
     * @Title: removeAttribute
     * @Description: 属性清除
     */
    public static void removeAttribute(SessionKeys key) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            redisService.delete(sysUser.getUsername() + key.toString());
        }
    }

    /**
     * @param key   已定义key
     * @param value
     * @Title: addAttribute
     * @Description: 属性追加，属性值以List<Object>保存
     */
    public static void addAttribute(SessionKeys key, Object value) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (sysUser != null) {
            SessionUtil.getRedisService();
            Object any = redisService.getObject(sysUser.getUsername() + key.toString(), Object.class);
            if (any instanceof List<?>) {
                @SuppressWarnings("unchecked")
                List<Object> list = (List<Object>) any;
                list.add(value);
                redisService.putObject(sysUser.getUsername() + key, list, 60 * 60 * 8);
            } else {
                List<Object> list = new ArrayList<Object>();
                list.add(value);
                redisService.putObject(sysUser.getUsername() + key, list, 60 * 60 * 8);
            }
        }
    }

    /**
     * @param loginId void
     * @Title: setLoginId
     * @Description: 设置LoginId
     */
    public static void setLoginId(String loginId) {
        SessionUtil.setAttribute(SessionKeys.LOGIN_ID, loginId);
    }

    /**
     * @return String
     * @Title: getLoginId
     * @Description: 获取LoginId
     */
    public static String getLoginId() {
        return (String) SessionUtil.getAttribute(SessionKeys.LOGIN_ID);
    }

    /**
     * @param loginSequence void
     * @Title: setLoginSequence
     * @Description: 设置登录序列
     */
    public static void setLoginSequence(String loginSequence) {
        SessionUtil.setAttribute(SessionKeys.LOGIN_SEQUENCE, loginSequence);
    }

    /**
     * @param loginSequence
     * @return String
     * @Title: getLoginSequence
     * @Description: 获取登录序列
     */
    public static String getLoginSequence() {
        return (String) SessionUtil.getAttribute(SessionKeys.LOGIN_SEQUENCE);
    }

    /**
     * @Title: invalidate
     * @Description: 无效化session
     */
    public static void invalidate() {
        HttpSession session = SessionUtil.getSession();
        if (session != null) {
            session.invalidate();
        }
    }
}
