// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PaPart;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PaPartMapper
 * @Description PA预防性措施评估部件表 Mapper接口
 * @Author
 * @CreateDate 2014-4-25
 * @version V1.0
 */
public interface PaPartMapper extends BaseMapper<PaPart, String> {

    /**
     * @Title: selectPartsInfoByAnalysisId
     * @Description: 获取预防性措施信息
     * @param paAnalysisId
     * @return List<Map<String, String>>
     */
    List<Map<String, String>> selectPartsInfoByAnalysisId(@Param("paAnalysisId") String paAnalysisId);

    /**
     * @Title: selectPartByPaAnalysisId
     * @Description: 获取零部件信息
     * @param paAnalysisId
     * @return List<PaPart>
     */
    List<PaPart> selectPartByPaAnalysisId(@Param("paAnalysisId") String paAnalysisId);

    /**
     * @Title: selectPartByPaAnalysisId
     * @Description:
     * @param paAnalysisId
     * @param levelId
     * @return List<PaTypeSub>
     */
    List<PaPart> selectPartByPaAnalysisIdAndLeveId(
            @Param("paAnalysisId") String paAnalysisId,
            @Param("riskLevelId") String riskLevelId);

    /**
     * @Title: checkPart
     * @Description: 获取不符合发布条件总数
     * @param paAnalysisId
     * @return int
     */
    int checkPart(@Param("paAnalysisId") String paAnalysisId);

    /**
     * @Title: checkPartLevel
     * @Description: 校验零部件措施分类信息是否符合发布条件
     * @param paAnalysisId
     * @return int
     */
    int checkPartLevel(@Param("paAnalysisId") String paAnalysisId);

    /**
     * @Title: checkPartQuantity
     * @Description: 校验零部件措施分类信息是否符合发布条件
     * @param paAnalysisId
     * @return int
     */
    int checkPartQuantity(@Param("paAnalysisId") String paAnalysisId);

    /**
     * @Title: updateByQuantity
     * @Description: 更新预防性措施评估部件表相关措施数量
     * @param paAnalysisId void
     */
    void updateByQuantity(@Param("paAnalysisId") String paAnalysisId);

    /**
     * @Title: selectRiskValue
     * @Description: 预防性措施评估风险值
     * @param paAnalysisId
     * @return Map<String, BigDecimal>
     */
    Map<String, BigDecimal> selectRiskValue(@Param("paAnalysisId") String paAnalysisId);
}
