// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmProject;
import com.nationrel.modules.project.entity.ProjectPageModel;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PmProjectMapper
 * @Description 项目信息表 Mapper接口
 * @Author andy
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmProjectMapper extends BaseMapper<PmProject, String> {

    /**
     * @Title: searchProjectByCondition
     * @Description:  根据搜索条件查询项目信息
     * @return List<PmProject>
     */
    List<Map<String, Object>> searchProjectByCondition(@Param("pagination") Pagination pagination, @Param("argMap") Map<String, Object> argMap);

    /**
     * @Title: countProjectByCondition
     * @Description: 根据搜索条件获取项目记录数
     * @return int
     */
    int countProjectByCondition(@Param("argMap") Map<String, Object> argMap);

    /**
     * @Title: deleteLogicByPrimaryKey
     * @Description: 根据主键逻辑删除项目
     * @return void
     */
    void deleteLogicByPrimaryKey(@Param("projectId") String projectId);

    /**
     * @Title: findProjectById
     * @Description: 获取项目信息-主要用户项目查看页面的数据展示
     * @param projectId
     * @return List
     */
    List<ProjectPageModel> findProjectById(@Param("projectId") String projectId);

    /**
     * @Title: getProjectBasicInfo
     * @Description: 获取项目基本信息
     * @param projectId
     * @return Map
     */
    Map<String, Object> getProjectBasicInfo(@Param("projectId") String projectId);

    /**
     * @Title: searchResourceLifeStateInProject
     * @Description: 取得项目下各个资源的生命周期状态
     * @param argMap key_ProjectId
     * @return List<Map<String, String>>
     */
    List<Map<String, String>> searchResourceLifeStateInProject(@Param("argMap") Map<String, String> argMap);

    /**
     * @Title: checkContainCurTimeOfTg
     * @Description: 检查当前TG时间是否在项目时间范围之内
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param projectId
     * @return PmProject
     */
    PmProject checkContainCurTimeOfTg(@Param("dtPlanStartTime") Date dtPlanStartTime, @Param("dtPlanEndTime") Date dtPlanEndTime, @Param("projectId") String projectId);
}
