// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmProjectStaff;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @ClassName PmProjectStaffMapper
 * @Description 项目人员配置表 Mapper接口
 * @Author
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmProjectStaffMapper extends BaseMapper<PmProjectStaff, String> {

    /**
     * @Title: searchProjectMemberListByProjectId
     * @Description: 获取项目成员列表数据
     * @param projectId
     * @return List<Map<String, Object>>
     */
    List<Map<String , Object>> searchProjectMemberListByProjectId(@Param("projectId") String projectId);

    /**
     * @Title: deleteStaffByProjectId
     * @Description: 删除项目成员
     * @param projectId void
     */
    void deleteStaffByProjectId(@Param("projectId") String projectId);

    /**
     * @Title: getStaffByPrimaryKey
     * @Description: 获取项目成员数据
     * @param projectId
     * @return PmProjectStaff
     */
    PmProjectStaff getStaffByPrimaryKey(@Param("projectId") String projectId, @Param("staffId") String staffId);

    /**
     * @Title: searchPerson
     * @Description: 从项目成员中选出可作为TG成员的人员
     * @param argMap
     * @return List<SysUser>
     */
    List<Map<String, String>> searchPerson(@Param("argMap") Map<String, String> argMap);

    /**
     * @Title: deleteByPrimaryKey
     * @Description: 删除项目成员
     * @param tgId
     * @param staffId void
     */
    void deleteByPrimaryKey(@Param("projectId") String projectId, @Param("staffId") String staffId);

    /**
     * @Title: selectPmProjectStaffsByTgId
     * @Description: 取得项目成员列表
     * @param tgId
     * @param staffId
     * @return List<Map<String, Object>>
     */
    List<Map<String, Object>> selectPmProjectStaffsByProjectId(@Param("projectId") String projectId, @Param("staffId") String staffId);

    /**
     * @Title: selectByPrimaryKey
     * @Description: 根据主键取得TG成员
     * @param tgId
     * @param staffId
     * @return PmTgStaff
     */
    PmProjectStaff selectByPrimaryKey(@Param("projectId") String projectId, @Param("staffId") String staffId);

    /**
     * @Title: countProjectRole
     * @Description: 获取项目角色个数
     * @param userId 用户Id
     * @param projectId 项目Id
     * @param projectRoleId 项目角色Id
     * @param projectRoleName 项目角色名称
     * @return int 项目角色个数
     */
    int countProjectRole(@Param("userId") String userId, @Param("projectId") String projectId,
            @Param("projectRoleId") String projectRoleId, @Param("projectManager") String projectManager);

    /**
     * @Title: checkCurrUserStaffType
     * @Description: 检查当前用户是否是当前项目下的所有者或者项目经理
     * @param userId:用户标识
     * @param projectId:项目标识
     * @param projectOwnerId:项目所有者标识
     * @param projectManagerId:项目经理标识
     * @return int:如果返回1,代表当前用户在当前项目下是所有者或者经理;返回0,代表当前用户在当前项目下非所有者或经理
     */
    int checkCurrUserIsProjectManager(@Param("userId") String userId, @Param("projectId") String projectId,
            @Param("projectOwnerId") String projectOwnerId, @Param("projectManagerId") String projectManagerId);
}
