// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmTaskDelivery;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @ClassName PmTaskDeliveryMapper
 * @Description TASK交付关联表 Mapper接口
 * @Author
 * @CreateDate 2014-3-5
 * @version V1.0
 */
public interface PmTaskDeliveryMapper extends BaseMapper<PmTaskDelivery, String> {
    /**
     * @Title: deleteDeliveryByTaskIdAndDeliveryId
     * @Description: 从Task交付表中移除文档
     * @param taskId
     * @param deliveryType
     * @param documentId void
     */
    void deleteDeliveryByTaskIdAndDeliveryId(@Param("taskId") String taskId,
            @Param("deliveryId") String deliveryId, @Param("deliveryType") String deliveryType);

    /**
     * @Title: searchDeliveryListByTaskId
     * @Description: 根据任务标识取得交付列表
     * @param taskId
     * @return List<Map<String , Object>>
     */
    List<Map<String , Object>> searchDeliveryListByTaskId(@Param("taskId") String taskId);

    /**
     * @Title: countDeliverysByCondition
     * @Description: 通过搜索条件获取交付列表总数
     * @param pagination
     * @param map
     * @return int
     */
    int countDeliverysByCondition(
            @Param("pagination") Pagination pagination,
            @Param("map") Map<String, Object> map);

    /**
     * @Title: selectDeliverysByCondition
     * @Description: 通过搜索条件获取交付列表
     * @param pagination
     * @param map
     * @return List<Map<String , Object>>
     */
    List<Map<String , Object>> selectDeliverysByCondition(
            @Param("pagination") Pagination pagination,
            @Param("map") Map<String, Object> map);

    /**
     * @Title: selectByPrimaryKey
     * @Description: 根据主键取得交付对象
     * @param deliveryId
     * @param taskId
     * @param deliveryType
     * @return PmTaskDelivery
     */
    PmTaskDelivery selectByPrimaryKey(@Param("deliveryId") String deliveryId, @Param("taskId")  String taskId, @Param("deliveryType")  String deliveryType);

    /**
     * @Title: deleteDeliveryByTaskNode
     * @Description: 根据任务结点删除交付及所有子任务的交付
     * @param taskId
     * @param documentId void
     */
    void deleteDeliveryByTaskNode(@Param("leftNo")  Integer leftNo, @Param("rightNo")  Integer rightNo, @Param("ordinal")  Integer ordinal, @Param("tgId")  String tgId);

    /**
     * @Title: deleteDeliveryByTgId
     * @Description: 根据TG标识删除TG下所有交付
     * @param tgId
     * return void
     */
    void deleteDeliveryByTgId(@Param("tgId")  String tgId);
}
