// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.entity.TaskPageModel;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PmTaskMapper
 * @Description TASK信息表 Mapper接口
 * @Author
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmTaskMapper extends BaseMapper<PmTask, String> {

    /**
     * @Title: countChildTaskNumber
     * @Description: 获取任务下直接子任务个数
     * @param taskId
     * @return int
     */
    int countDirectChildTaskNumber(@Param("taskId") String taskId);

    /**
     * @Title: getDirectChildTaskList
     * @Description: 获取任务下直接子任务集合
     * @param taskId
     * @return List<Map<String, Object>>
     */
    List<Map<String , Object>> getDirectChildTaskList(@Param("taskId") String taskId);

    /**
     * @Title: getTaskWholeInfoByTaskId
     * @Description: 获取任务完整信息
     * @param taskId
     * @return Map<String, Object>
     */
    Map<String , Object> getTaskWholeInfoByTaskId(@Param("taskId") String taskId);

    /**
     * @Title: selectPmTaskByTgId
     * @Description: 取得tg下所有Task信息
     * @param tgId tg标识
     * @return List<PmTask> Task列表
     */
    List<PmTask> selectPmTaskByTgId(@Param("tgId") String tgId);

    /**
     * @Title: selectTaskPageModelByTgId
     * @Description: 取得tg下所有Task信息
     * @param tgId tg标识
     * @return List<TaskPageModel> Task列表
     */
    List<TaskPageModel> selectTaskPageModelByTgId(@Param("tgId") String tgId);

    /**
     * @Title: searchPmTaskList
     * @Description: 取得页面任务列表数据
     * @param argMap
     * @return List<TaskPageModel>
     */
    List<TaskPageModel> searchPmTaskList(@Param("argMap")  Map<String, Object> argMap);

    /**
     * @Title: deleteTaskByLRNo
     * @Description: 根据任务关系表记录左右值-删除任务表中对应的记录
     * @param leftNo 当前节点的左键值
     * @param rightNo 当前节点的右键值
     * @param treeNo 当前节点所处树号
     * @param tgId TG标识
     */
    void deleteTaskByLRNo(@Param("leftNo") Integer leftNo, @Param("rightNo") Integer rightNo,
            @Param("treeNo") Integer serialNumber, @Param("tgId") String tgId);

    /**
     * @Title: countChildTaskNumber
     * @Description: 获取TG下相同任务名称的数量
     * @param taskId
     * @return int
     */
    int countTaskNameByTgId(@Param("tgId") String tgId, @Param("taskName") String taskName, @Param("taskId") String taskId);

    /**
     * @Title: deleteTaskByTgId
     * @Description: 根据TG标识删除TG下所有任务
     * @param tgId
     * return void
     */
    void deleteTaskByTgId(@Param("tgId")  String tgId);

    /**
     * @Title: checkContainCurTimeOfChildTask
     * @Description: 检查任务预计日期在上级任务的日期范围之内
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param parentTaskId
     * @return PmTask
     */
    PmTask checkContainCurTimeOfChildTask(@Param("dtPlanStartTime") Date dtPlanStartTime, @Param("dtPlanEndTime") Date dtPlanEndTime, @Param("parentTaskId") String parentTaskId);
}
