// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmTaskStaff;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @ClassName PmTaskStaffMapper
 * @Description TASK人员配置表 Mapper接口
 * @Author
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmTaskStaffMapper extends BaseMapper<PmTaskStaff, String> {

    /**
     * @Title: selectByPrimaryKey
     * @Description: 获取task记录
     * @param taskId
     * @param staffId
     * @return PmTaskStaff
     */
    PmTaskStaff selectByPrimaryKey(@Param("taskId") String taskId, @Param("staffId") String staffId);

    /**
     * @Title: deleteTaskStaffByTaskNode
     * @Description: 根据任务结点删除成员及所有子任务的成员
     * @param taskId void
     */
    void deleteTaskStaffByTaskNode(@Param("leftNo")  Integer leftNo, @Param("rightNo")  Integer rightNo, @Param("ordinal")  Integer ordinal, @Param("tgId")  String tgId);

    /**
     * @Title: deleteByPrimaryKey
     * @Description: 根据任务标识删除任务成员
     * @param taskId void
     */
    void deleteByPrimaryKey(@Param("taskId") String taskId, @Param("staffId") String staffId);

    /**
     * @Title: selectExistStaff
     * @Description: 取得任务有效成员
     * @param argMap
     * @return List<Map<String, String>>
     */
    List<Map<String, String>> selectExistStaff(@Param("argMap") Map<String, String> argMap);

    /**
     * @Title: selectPmTaskStaffsByTaskId
     * @Description: 取得任务成员列表
     * @param tgId
     * @param staffId
     * @return List<Map<String, Object>>
     */
    List<Map<String, Object>> selectPmTaskStaffsByTaskId(@Param("taskId") String taskId, @Param("staffId") String staffId);

    /**
     * @Title: countTaskRole
     * @Description: 获取Task角色个数
     * @param userId 用户Id
     * @param projectId 项目Id
     * @param projectRoleId 角色Id
     * @param tgPrincipal 角色名称
     * @return int 项目角色个数
     */
    int countTaskRole(@Param("userId") String userId, @Param("taskId") String taskId,
            @Param("projectRoleId") String projectRoleId, @Param("taskPrincipal") String taskPrincipal);

    /**
     * @Title: searchPerson
     * @Description: 从Task成员中选出可作为任务成员的人员
     * @param argMap
     * @return List<SysUser>
     */
    List<Map<String, String>> searchPerson(@Param("argMap") Map<String, String> argMap);

    /**
     * @Title: deleteStaffByTgId
     * @Description: 根据TG标识删除TG下所有成员
     * @param tgId
     * return void
     */
    void deleteStaffByTgId(@Param("tgId")  String tgId);

    /**
     * @Title: searchTaskReferenceStaff
     * @Description: 成员是否在任务中引用检查
     * @param tgId
     * @param taskFirstParentId
     * @param staffId
     * @return List<Map<String, String>>
     */
    List<Map<String, String>> searchTaskReferenceStaff(@Param("tgId") String tgId, @Param("taskFirstParentId") String taskFirstParentId, @Param("staffId") String staffId);
}
