// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.Map;

/**
 * @ClassName PmTaskTreeMapper
 * @Description TASK关系表 Mapper接口
 * @Author
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmTaskTreeMapper extends BaseMapper<PmTaskTree, String> {

    /**
     * @Title: selectMaxId
     * @Description: 取得任务树中最大的树编号
     * @return Integer
     */
    int selectMaxId(@Param("tgId") String tgId);

    /**
     * @Title: selectTaskNodeDeep
     * @Description: 取得新增节点的深度
     * @param leftNo 当前节点的左键值
     * @param rightNo 当前节点的右键值
     * @param treeNo 当前节点所处树号
     * @param tgId STEP标识
     * @return Integer
     */
    Integer selectTaskNodeDeep(@Param("leftNo") Integer leftNo, @Param("rightNo") Integer rightNo,
            @Param("treeNo") Integer treeNo , @Param("tgId") String tgId);

    /**
     * @Title: updateTaskTreeLeftNo
     * @Description: 新增节点后-更新引发变化的节点的左键值
     * @param rightNo 当前节点的右键值
     * @param treeNo 当前节点所处树号
     * @param tgId STEP标识
     */
    void updateTaskTreeLeftNo(@Param("rightNo") Integer rightNo, @Param("treeNo") Integer treeNo,
            @Param("tgId") String tgId);

    /**
     * @Title: updateTaskTreeRightNo
     * @Description: 新增节点后-更新引发变化的节点的右键值
     * @param rightNo 当前节点的右键值
     * @param treeNo 当前节点所处树号
     * @param tgId STEP标识
     */
    void updateTaskTreeRightNo(@Param("rightNo") Integer rightNo, @Param("treeNo") Integer treeNo,
            @Param("tgId") String tgId);

    /**
     * @Title: deleteTaskNodeAndChildNode
     * @Description: 删除当前节点及所属子节点
     * @param leftNo 当前节点的左键值
     * @param rightNo 当前节点的右键值
     * @param treeNo 当前节点所处树号
     * @param tgId STEP标识
     */
    void deleteTaskNodeAndChildNode(@Param("leftNo") Integer leftNo, @Param("rightNo") Integer rightNo,
            @Param("treeNo") Integer treeNo, @Param("tgId") String tgId);

    /**
     * @Title: updateTaskNodeLeftNoCaseDelete
     * @Description: 删除节点后-更新引发变化的节点的左键值
     * @param changeNum
     * @param treeNo
     * @param treeNo
     * @param tgId void
     */
    void updateTaskNodeLeftNoCaseDelete(@Param("changeNum") int changeNum, @Param("leftNo") Integer leftNo,
            @Param("treeNo") Integer treeNo, @Param("tgId") String tgId);

    /**
     * @Title: updateTaskNodeRightNoCaseDelete
     * @Description: 删除节点后-更新引发变化的节点的右键值
     * @param changeNum
     * @param rightNo
     * @param treeNo
     * @param tgId void
     */
    void updateTaskNodeRightNoCaseDelete(@Param("changeNum") int changeNum, @Param("rightNo") Integer rightNo,
            @Param("treeNo") Integer treeNo, @Param("tgId") String tgId);

    /**
     * @Title: statPercent
     * @Description: 统计进度
     * @param argMap void
     */
    void statPercent(Map<String, Object> argMap);

    /**
     * @Title: deleteTaskTreeByTgId
     * @Description: 根据TG标识删除TG下所有任务树
     * @param tgId
     * return void
     */
    void deleteTaskTreeByTgId(@Param("tgId")  String tgId);

    /**
     * @Title: checkContainTimeOfTask
     * @Description: 当前TG的时间必须包括所有子项的时间
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param tgId
     * @return int
     */
    int checkContainTimeOfTask(@Param("dtPlanStartTime")  Date dtPlanStartTime, @Param("dtPlanEndTime")  Date dtPlanEndTime, @Param("tgId")  String tgId);

    /**
     * @Title: checkContainTimeOfChildTask
     * @Description: 当前TASK的时间范围包括所有子项目的时间(编辑)
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param tgId
     * @param ordinal
     * @param leftNo
     * @param rightNo
     * @return int
     */
    int checkContainTimeOfChildTask(@Param("dtPlanStartTime") Date dtPlanStartTime, @Param("dtPlanEndTime")  Date dtPlanEndTime,
            @Param("tgId")  String tgId, @Param("ordinal")  Integer ordinal, @Param("leftNo")  Integer leftNo, @Param("rightNo")  Integer rightNo);
}
