// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PmTgMapper
 * @Description TG信息表 Mapper接口
 * @Author
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmTgMapper extends BaseMapper<PmTg, String> {

    /**
     * @Title: getMaxTgSort
     * @Description: 通过项目标识取得TG列表中最大序号
     * @param projectId
     * @return int
     */
    int getMaxTgSort(@Param("projectId") String projectId);

    /**
     * @Title: selectPmTgByProjectId
     * @Description: 取得项目下所有TG信息
     * @param projectId 项目标识
     * @return List<PmTg> TG列表
     */
    List<PmTg> selectPmTgByProjectId(@Param("projectId") String projectId);

    /**
     * @Title: checkContainTimeOfTG
     * @Description: 取得项目内TG时间不在指定时间范围内的TG
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param projectId void
     */
    List<PmTg>  checkContainTimeOfTG(@Param("dtPlanStartTime") Date dtPlanStartTime, @Param("dtPlanEndTime") Date dtPlanEndTime, @Param("projectId") String projectId);

    /**
     * @Title: checkRepeatWithOtherTg
     * @Description: 检查当前TG的时间不能与其他TG时间重合
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param projectId
     * @param tgId
     * @return List<PmTg>
     */
    List<PmTg> checkRepeatWithOtherTg(@Param("dtPlanStartTime") Date dtPlanStartTime, @Param("dtPlanEndTime") Date dtPlanEndTime, @Param("projectId") String projectId, @Param("tgId") String tgId);

    /**
     * @Title: checkContainCurTimeOfTask
     * @Description: 检查顶级任务在TG的日期范围之内
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param tgId
     * @return PmTg
     */
    PmTg checkContainCurTimeOfTask(@Param("dtPlanStartTime") Date dtPlanStartTime, @Param("dtPlanEndTime") Date dtPlanEndTime, @Param("tgId") String tgId);

    /**
     * @Title: getExCreateTg
     * @Description: 取得创建状态以外的TG列表
     * @param projectId
     * @param tgId
     * @return List<PmTg>
     */
    List<PmTg> getExCreateTg(@Param("projectId") String projectId, @Param("tgId") String tgId);

    /**
     * @Title: selectFullTGInformation
     * @Description:获取TG完整信息,包括TG完成度&&检查项统计&&交付统计&&成员/负责人.
     * @param tgId
     * @return Map<String, Object>
     */
    Map<String, Object> selectFullTGInformation(@Param("tgId") String tgId);
}
