// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PmTgStaff;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @ClassName PmTgStaffMapper
 * @Description TG人员配置表 Mapper接口
 * @Author
 * @CreateDate 2014-3-4
 * @version V1.0
 */
public interface PmTgStaffMapper extends BaseMapper<PmTgStaff, String> {

    /**
     * @Title: selectPmTgStaffsByTgId
     * @Description: 取得TG成员列表
     * @param tgId
     * @param staffId
     * @return List<Map<String, Object>>
     */
    List<Map<String, Object>> selectPmTgStaffsByTgId(@Param("tgId") String tgId, @Param("staffId") String staffId);

    /**
     * @Title: selectByPrimaryKey
     * @Description: 根据主键取得TG成员
     * @param tgId
     * @param staffId
     * @return PmTgStaff
     */
    PmTgStaff selectByPrimaryKey(@Param("tgId") String tgId, @Param("staffId") String staffId);

    /**
     * @Title: countProjectRole
     * @Description: 获取TG角色个数
     * @param userId 用户Id
     * @param projectId 项目Id
     * @param projectRoleId 角色Id
     * @param tgPrincipal 角色名称
     * @return int 项目角色个数
     */
    int countTgRole(@Param("userId") String userId, @Param("tgId") String projectId,
            @Param("projectRoleId") String projectRoleId, @Param("tgPrincipal") String tgPrincipal);

    /**
     * @Title: deleteByPrimaryKey
     * @Description: TG成员删除
     * @param tgId
     * @param staffId void
     */
    void deleteByPrimaryKey(@Param("tgId") String tgId, @Param("staffId") String staffId);

    /**
     * @Title: deleteByTgId
     * @Description: TG成员删除
     * @param tgId
     * @param staffId void
     */
    void deleteStaffByTgId(@Param("tgId") String tgId);

    /**
     * @Title: searchPerson
     * @Description: 从TG成员中选出可作为任务成员的人员
     * @param argMap
     * @return List<SysUser>
     */
    List<Map<String, String>> searchPerson(@Param("argMap") Map<String, String> argMap);

    /**
     * @Title: selectExistStaff
     * @Description: 取得TG有效成员
     * @param argMap
     * @return List<Map<String, String>>
     */
    List<Map<String, String>> selectExistStaff(@Param("argMap") Map<String, String> argMap);

    /**
     * @Title: searchTgReferenceStaff
     * @Description: 判断某人是否是项目中的TG的成员-并取出相应的TG
     * @param projectId
     * @param staffId
     * @return List<Map<String, String>>
     */
    List<Map<String, String>> searchTgReferenceStaff(@Param("projectId") String projectId, @Param("staffId") String staffId);
}
