// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.mapper;

import com.nationrel.modules.project.entity.PpPlanTask;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.rb_common.common.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PpPlanTaskMapper
 * @Description 预防性措施规划任务表 Mapper接口
 * @Author
 * @CreateDate 2014-4-25
 * @version V1.0
 */
public interface PpPlanTaskMapper extends BaseMapper<PpPlanTask, String> {

    /**
     * @Title: generateTasksAfterProactivePlanCreated
     * @Description: 预防性措施规划创建-规划任务产生
     * @param proactiveId
     * @param proactivePlanId
     * @param insertUserId
     * @param insertPgmId void
     */
    void generateTasksAfterPpPlanCreated(@Param("proactiveId") String proactiveId, @Param("proactivePlanId") String proactivePlanId,
            @Param("insertUserId") String insertUserId, @Param("insertPgmId") String insertPgmId);

    /**
     * @Title: getPpProactivePlanTasksByPpProactivePlanId
     * @Description: 获取预防性措施规划下所有任务信息
     * @param proactivePlanId
     * @return List<PpProactivePlanTask>
     */
    List<Map<String , Object>> getPpPlanTasksByPpPlanId(@Param("pagination") Pagination pagination, @Param("proactivePlanId") String proactivePlanId);

    /**
     * @Title: searchPpPlanTasksForPlanTrackByPpPlanId
     * @Description: 获取预防性措施规划下所有任务信息
     * @param proactivePlanId
     * @return List<PpProactivePlanTask>
     */
    List<Map<String , Object>> searchPpPlanTasksForPlanTrackByPpPlanId(@Param("pagination")Pagination pagination, @Param("proactivePlanId") String proactivePlanId);

    /**
     * @Title: countTasksByPageCondition
     * @Description: TODO 用途说明
     * @param pagination
     * @param planId
     * @return int
     */
    int countTasksByPageCondition(@Param("pagination")Pagination pagination, @Param("planId") String planId);

    /**
     * @Title: checkTaskNameExistInTG
     * @Description:检查任务名称在TG下是否存在
     * @param taskName
     * @param tgId
     * @return int
     */
    int checkTaskNameExistInTG(@Param("taskName") String taskName, @Param("tgId") String tgId, @Param("taskId") String taskId);

    /**
     * @Title: countPpPlanIdAndPlanTGIdByPpPlanId
     * @Description:
     * @param ppPlanId
     * @return Map<?, ?>
     */
    Map<?, ?> countPpPlanIdAndPlanTGIdByPpPlanId(@Param("ppPlanId") String ppPlanId);

    /**
     *
     * @Title: deleteByTask
     * @Description:
     * @param proactivePlanId void
     */
    void deleteTaskByPlanId(@Param("proactivePlanId")  String proactivePlanId);

    /**
     * @Title: generateTasksForPmTaskCasePpPlanCompleted
     * @Description: 生成任务数据到表[t_pm_task]当预防性规划规划完成时
     * @param planTgId void
     */
    void generatePmTasksCasePpPlanCompleted(@Param("insertUserId") String insertUserId, @Param("insertPgmId") String insertPgmId, @Param("ppPlanId") String ppPlanId);

    /**
     * @Title: generatePmTaskStaffsCasePpPlanCompleted
     * @Description: 生成任务成员数据到表[t_pm_task_staff]当预防性规划规划完成时
     * @param ppPlanId void
     */
    void generatePmTaskStaffsCasePpPlanCompleted(@Param("insertUserId") String insertUserId, @Param("insertPgmId") String insertPgmId, @Param("ppPlanId") String ppPlanId);

    /**
     * @Title: generatePmTaskTreeCasePpPlanCompleted
     * @Description: 生成任务树数据到表[t_pm_task_tree]当预防性规划规划完成时
     * @param ppPlanId void
     */
    void generatePmTaskTreeCasePpPlanCompleted(@Param("ppPlanId") String ppPlanId);

    /**
     * @Title: getPioneerStartTimeOfTaskByPpPlanId
     * @Description: 获取预防性规划下最早的任务开始时间
     * @param ppPlanId
     * @return Date
     */
    Date getEarliestStartTimeOfTaskByPpPlanId(@Param("ppPlanId") String ppPlanId);

    /**
     * @Title: getPioneerStartTimeOfTaskByPpPlanId
     * @Description: 获取预防性规划下最晚的任务结束时间
     * @param ppPlanId
     * @return Date
     */
    Date getLatestStartEndOfTaskByPpPlanId(@Param("ppPlanId") String ppPlanId);

    /**
     * @Title: statisticsPlanTaskRiskValueByPlanId
     * @Description: sum(RISK_DOWN_VALUE*TASKPERCENT)
     * @param ppPlanId
     * @return BigDecimal
     */
    Map<String, String> statisticsPlanTaskRiskValueByPlanId(@Param("projectId") String projectId, @Param("ppPlanId") String ppPlanId);

}
