package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.entity.PmAuthority;
import com.nationrel.modules.project.jar.rb_common.bizcommon.constant.BizModuleCode;
import com.nationrel.modules.project.jar.rb_common.bizcommon.constant.BizcommonConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.rb_platform.entity.SysStateConfig;
import com.nationrel.modules.project.jar.rb_platform.mapper.SysStateConfigMapper;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmAuthorityMapper;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.mapper.PmProjectRoleMapper;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @version V1.0
 * @ClassName AuthorityCtrlServiceImpl
 * @Description 可靠性项目权限控制实现类
 * @Author ted.liu
 * @CreateDate 2013-12-10
 */
@SuppressWarnings("unchecked")
@Service
public class AuthorityCtrlServiceImpl implements AuthorityCtrlService {

    @Autowired
    private PmAuthorityMapper pmAuthorityMapper;
    @Autowired
    private PmProjectRoleMapper pmProjectRoleMapper;
    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private SysStateConfigMapper sysStateConfigMapper;
    /**
     * @Fields cacheMap : 缓存权限MAP
     */
    private final Map<String, PmAuthority> cacheAuthMap = new HashMap<String, PmAuthority>();

    /**
     * @param argMap
     * @param iType
     * @return List<PmAuthority>
     * @Title: searchAuthMapfromCache
     * @Description: 取得权限列表
     */
    private List<PmAuthority> searchAuthMapfromCache(Map<String, String> argMap, int iType) {
        // 查看缓存权限MAP中是否有缓存
        String tmpKey = null;
        if (cacheAuthMap.isEmpty()) {
            Map<String, String> retMap = new HashMap<String, String>();
            List<PmAuthority> pmAuthoritys = pmAuthorityMapper.selectAuthority(retMap);
            for (PmAuthority pmAuthority : pmAuthoritys) {
                tmpKey = pmAuthority.getClassify() + pmAuthority.getClassifyValue() + pmAuthority.getResourceType() + pmAuthority.getChildResourceType();
                cacheAuthMap.put(tmpKey, pmAuthority);
            }
        }

        // 根据状态和角色进行键组合
        String prefixKey = null;
        if (iType == 0) {
            prefixKey = argMap.get(PmMapKeySet.KEY_S_AUTH_CLASSIFY) + argMap.get(PmMapKeySet.KEY_S_LIFE_STATE) + argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE);
        } else {
            prefixKey = argMap.get(PmMapKeySet.KEY_S_AUTH_CLASSIFY) + argMap.get(PmMapKeySet.KEY_S_ROLE_TYPE) + argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE);
        }

        // 如果子资源类型不为空构建权限列表
        List<PmAuthority> retAuthorityList = new ArrayList<PmAuthority>();
        if (argMap.get(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE) != null) {
            tmpKey = prefixKey + argMap.get(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE);
            retAuthorityList.add(cacheAuthMap.get(tmpKey));
        } else {
            for (int i = 1; i <= 7; i++) {
                tmpKey = prefixKey + StringUtil.leftPad(String.valueOf(i), 2, '0');
                if (cacheAuthMap.get(tmpKey) != null) {
                    retAuthorityList.add(cacheAuthMap.get(tmpKey));
                }
            }
        }

        return retAuthorityList;
    }

    /**
     * @param argMap
     * @return Map 权限集合
     * @Title: doViewAuthorityByGroup
     * @Description 分别取得状态权限的查看权限和角色权限的查看权限-处理合成为查看权限 (01:基本信息 02:检查列表 03:成员 04:文档 05:阶段 06:下级别)
     * 项目查看页面调用
     */
    @Override
    public Map<String, Map<String, String>> doViewAuthorityByGroup(Map<String, String> argMap) throws Exception {
        // 以资源结构树的形式返回每个资源的下各个子资源的查看权限(某些权限也使用了)
        Map<String, Map<String, String>> retReSourceMap = new HashMap<String, Map<String, String>>();

        // 项目资源结构树
        Map<String, Map<String, String>> mapProjectStructure = this.findResourceLifeStateInProject(argMap);
        Map<String, String> rowResource = null;
        for (Iterator<String> retiter = mapProjectStructure.keySet().iterator(); retiter.hasNext(); ) {
            String retkey = retiter.next();
            rowResource = mapProjectStructure.get(retkey);

            argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, rowResource.get(PmMapKeySet.KEY_R_RESOURCE_ID));
            argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, rowResource.get(PmMapKeySet.KEY_R_RESOURCE_TYPE));
            // ====================
            // 状态各资源子资源的查看权限
            // ====================
            argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, searchLifeState(argMap, mapProjectStructure));
            Map<String, String> retMap = findViewAuthorityByState(argMap);

            // ====================
            // 角色各资源子资源的查看权限
            // 角色类型参数设定
            // ====================
            argMap.put(PmMapKeySet.KEY_S_ROLE_TYPE, searchRoleType(argMap, mapProjectStructure));
            Map<String, String> tmpMap = findViewAuthorityByRole(argMap);

            // ====================
            // 状态权限和角色权限合并
            // ====================
            for (Iterator<String> iter = retMap.keySet().iterator(); iter.hasNext(); ) {
                String key = iter.next();
                retMap.put(key, (ReliaFlowConstants.IS_AUTHORITY_YES.equals(retMap.get(key))
                        && ReliaFlowConstants.IS_AUTHORITY_YES.equals(tmpMap.get(key))) ? ReliaFlowConstants.IS_AUTHORITY_YES : ReliaFlowConstants.IS_AUTHORITY_NO);
            }

            // 是否转化为CSS属性处理
            if (!ReliaFlowConstants.IS_AUTHORITY_SWITCH_NO.equals(argMap.get(PmMapKeySet.KEY_S_AUTHORITY_SWITCH))) {
                retReSourceMap.put(retkey, convertAuthView(retMap));
            } else {
                retReSourceMap.put(retkey, retMap);
            }
        }

        return retReSourceMap;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: doResourceAuthority
     * @Description: 取得具体资源的权限
     */
    @Override
    public Map<String, String> doResourceAuthority(Map<String, String> argMap) throws Exception {

        // ====================
        // 状态各资源子资源的查看权限
        // key_LifeState&key_ResourceType&key_ChildResourceType-Authority
        // ====================
        // 使用项目标识取出项目中所有资源结构
        Map<String, Map<String, String>> mapProjectStructure = this.findResourceLifeStateInProject(argMap);
        String lifeState = searchLifeState(argMap, mapProjectStructure);
        if (lifeState == null) {
            throw new OptimisticLockException(MessageId.W009805);
        }
        argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, lifeState);
        Map<String, String> retMap = findResourceAuthorityByState(argMap);

        // ====================
        // 角色各资源子资源的查看权限
        // key_RoleType&key_ResourceType&key_ChildResourceType-Authority
        // ====================
        argMap.put(PmMapKeySet.KEY_S_ROLE_TYPE, searchRoleType(argMap, mapProjectStructure));
        Map<String, String> tmpMap = findResourceAuthorityByRole(argMap);

        // ====================
        // 状态权限和角色权限合并
        // ====================
        for (Iterator<String> iter = retMap.keySet().iterator(); iter.hasNext(); ) {
            String key = iter.next();
            retMap.put(key, (ReliaFlowConstants.IS_AUTHORITY_YES.equals(retMap.get(key))
                    && ReliaFlowConstants.IS_AUTHORITY_YES.equals(tmpMap.get(key))) ? ReliaFlowConstants.IS_AUTHORITY_YES : ReliaFlowConstants.IS_AUTHORITY_NO);
        }

        // 是否转化为CSS属性处理
        if (!ReliaFlowConstants.IS_AUTHORITY_SWITCH_NO.equals(argMap.get(PmMapKeySet.KEY_S_AUTHORITY_SWITCH))) {
            retMap = convertAuthView(retMap);
        }

        // 当前资源状态
        Map<String, String> resourceRow = mapProjectStructure.get(argMap.get(PmMapKeySet.KEY_S_RESOURCE_ID) + argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE));
        retMap.put(PmMapKeySet.KEY_S_LIFE_STATE, resourceRow.get(PmMapKeySet.KEY_R_LIFE_STATE));
        retMap.put(PmMapKeySet.KEY_S_ROLE_TYPE, argMap.get(PmMapKeySet.KEY_S_ROLE_TYPE));

        return retMap;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: doResourceAuthority
     * @Description: 取得具体资源的权限
     */
    @Override
    public Map<String, String> doAuthorityByPage(Map<String, String> argMap) throws Exception {

        // ====================
        // 状态各资源子资源的查看权限
        // key_LifeState&key_ResourceType&key_ChildResourceType-Authority
        // ====================
        // 使用项目标识取出项目中所有资源结构
        Map<String, Map<String, String>> mapProjectStructure = this.findResourceLifeStateInProject(argMap);
        // 当前资源状态
        Map<String, String> resourceRow = mapProjectStructure.get(argMap.get(PmMapKeySet.KEY_S_RESOURCE_ID) + argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE));
        if (resourceRow == null) {
            throw new OptimisticLockException(MessageId.W009805);
        }
        argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, resourceRow.get(PmMapKeySet.KEY_R_LIFE_STATE));
        //argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, searchLifeState(argMap, mapProjectStructure));
        Map<String, String> retMap = findPageItemAuthorityByState(argMap);

        // ====================
        // 角色各资源子资源的查看权限
        // key_RoleType&key_ResourceType&key_ChildResourceType-Authority
        // ====================
        argMap.put(PmMapKeySet.KEY_S_ROLE_TYPE, searchRoleType(argMap, mapProjectStructure));
        Map<String, String> tmpMap = findPageItemAuthorityByRole(argMap);

        // ====================
        // 状态权限和角色权限合并
        // ====================
        for (Iterator<String> iter = retMap.keySet().iterator(); iter.hasNext(); ) {
            String key = iter.next();
            retMap.put(key, retMap.get(key).compareTo(tmpMap.get(key)) < 0 ? retMap.get(key) : tmpMap.get(key));
        }

        // 是否转化为CSS属性处理
        if (!ReliaFlowConstants.IS_AUTHORITY_SWITCH_NO.equals(argMap.get(PmMapKeySet.KEY_S_AUTHORITY_SWITCH))) {
            for (Iterator<String> iter = retMap.keySet().iterator(); iter.hasNext(); ) {
                String key = iter.next();
                if (ReliaFlowConstants.AUTH_CTRL_CLASS_HIDDEN.equals(retMap.get(key))) {
                    retMap.put(key, ReliaFlowConstants.AUTH_CTRL_VIEW);
                } else if (ReliaFlowConstants.AUTH_CTRL_CLASS_READONLY.equals(retMap.get(key))) {
                    retMap.put(key, ReliaFlowConstants.AUTH_CTRL_READONLY);
                } else {
                    retMap.put(key, "");
                }
            }
        }

        return retMap;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: findPageItemAuthorityByState
     * @Description: 页面项目在状态下的控制变量取得
     */
    private Map<String, String> findPageItemAuthorityByState(Map<String, String> argMap) {

        // 页面级项目状态下权限变量取得
        argMap.put(PmMapKeySet.KEY_S_AUTH_CLASSIFY, ReliaFlowConstants.AUTHORITY_CLASSIFY_PAGE_STATE);
        List<PmAuthority> pmAuthoritys = pmAuthorityMapper.selectAuthority(argMap);

        // 封装权限Map
        Map<String, String> retMap = new HashMap<String, String>();
        for (PmAuthority pmAuthority : pmAuthoritys) {
            retMap.put(pmAuthority.getAuthorityGroup1(), pmAuthority.getModiftAuthority());
        }

        return retMap;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: findPageItemAuthorityByRole
     * @Description: 页面项目在角色下的控制变量取得
     */
    private Map<String, String> findPageItemAuthorityByRole(Map<String, String> argMap) {

        // 页面级项目角色下权限变量取得
        argMap.put(PmMapKeySet.KEY_S_AUTH_CLASSIFY, ReliaFlowConstants.AUTHORITY_CLASSIFY_PAGE_ROLE);
        List<PmAuthority> pmAuthoritys = pmAuthorityMapper.selectAuthority(argMap);

        // 封装权限Map
        Map<String, String> retMap = new HashMap<String, String>();
        for (PmAuthority pmAuthority : pmAuthoritys) {
            retMap.put(pmAuthority.getAuthorityGroup1(), pmAuthority.getModiftAuthority());
        }

        return retMap;
    }

    /**
     * @param argMap
     * @return Map<String, Map < String, String>>
     * @Title: findResourceLifeStateInProject
     * @Description: 取得项目下各个资源的生命周期状态(状态可变性实时查询) TG标识参数有:查询到任务 无:查询到TG
     * @see doResourceAuthority调用时-由于取出了相应的所有资源.性能有待改进(特别是任务量比较多时.递归方法改进)
     */
    private Map<String, Map<String, String>> findResourceLifeStateInProject(Map<String, String> argMap) {
        // 各个资源的生命周期状态
        Map<String, Map<String, String>> stateMap = new HashMap<String, Map<String, String>>();

        // 取得项目下各个资源的生命周期状态
        List<Map<String, String>> stateList = pmProjectMapper.searchResourceLifeStateInProject(argMap);
        if (stateList != null && stateList.size() > 0) {
            for (Map<String, String> tmpMap : stateList) {
                stateMap.put(tmpMap.get(PmMapKeySet.KEY_R_RESOURCE_KEY), tmpMap);
            }
        }

        return stateMap;
    }

    /**
     * @param argMap
     * @param mapProjectStructure 项目中所有资源结构
     * @return String
     * @Title: searchLifeState
     * @Description: 根据资源Id及资源类型取出资源状态
     */
    private String searchLifeState(Map<String, String> argMap,
                                   Map<String, Map<String, String>> mapProjectStructure) {
        String retState = null;

        // 取得稳定状态Map
        Map<String, String> resourceStateMap = findResourceStateMap();
        // 取得当前资源信息
        Map<String, String> resourceRow = mapProjectStructure.get(argMap.get(PmMapKeySet.KEY_S_RESOURCE_ID) + argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE));
        if (resourceRow != null && resourceRow.size() > 0) {
            // 当前是稳定状态-查看上级资源的状态
            if (resourceStateMap.containsKey(resourceRow.get(PmMapKeySet.KEY_R_LIFE_STATE) + resourceRow.get(PmMapKeySet.KEY_R_RESOURCE_TYPE))) {
                retState = resourceRow.get(PmMapKeySet.KEY_R_LIFE_STATE);
                while (resourceRow.get(PmMapKeySet.KEY_R_PARENT_RESOURCE_ID) != null) {
                    resourceRow = mapProjectStructure.get(resourceRow.get(PmMapKeySet.KEY_R_PARENT_RESOURCE_KEY));
                    if (!resourceStateMap.containsKey(resourceRow.get(PmMapKeySet.KEY_R_LIFE_STATE) + resourceRow.get(PmMapKeySet.KEY_R_RESOURCE_TYPE))) {
                        retState = ReliaFlowConstants.LIFECYCLE_APPROVE_STATE;
                        break;
                    }
                }
            } else {
                retState = ReliaFlowConstants.LIFECYCLE_APPROVE_STATE;
            }
        }

        return retState;
    }

    /**
     * @param argMap
     * @param mapProjectStructure 项目中所有资源结构
     * @return String
     * @Title: searchRoleType
     * @Description: 根据资源Id及资源类型取出角色类型
     */
    private String searchRoleType(Map<String, String> argMap,
                                  Map<String, Map<String, String>> mapProjectStructure) {
        // 取出用户在当前项目中所充当的角色
        List<Map<String, String>> rolesList = findUserRoleAuthInProject(argMap);

        // 根据资源Id及资源类型取出角色类型
        Map<String, Map<String, String>> roleMap = new HashMap<String, Map<String, String>>();
        for (Map<String, String> tmpMap : rolesList) {
            if (argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE).compareTo(tmpMap.get(PmMapKeySet.KEY_R_RESOURCE_TYPE)) >= 0) {
                roleMap.put(tmpMap.get(PmMapKeySet.KEY_R_RESOURCE_ID) + tmpMap.get(PmMapKeySet.KEY_R_RESOURCE_TYPE), tmpMap);
            }
        }

        // 根据当前资源标识追朔所有的上级节点
        List<String> resourceIds = new ArrayList<String>();
        Map<String, String> resourceRow = mapProjectStructure.get(argMap.get(PmMapKeySet.KEY_S_RESOURCE_ID) + argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE));
        if (resourceRow != null && resourceRow.size() > 0) {
            resourceIds.add(resourceRow.get(PmMapKeySet.KEY_R_RESOURCE_KEY));
            while (resourceRow.get(PmMapKeySet.KEY_R_PARENT_RESOURCE_ID) != null) {
                resourceRow = mapProjectStructure.get(resourceRow.get(PmMapKeySet.KEY_R_PARENT_RESOURCE_KEY));
                resourceIds.add(resourceRow.get(PmMapKeySet.KEY_R_RESOURCE_KEY));
            }
        }

        // 取得当前资源中的角色类型
        Map<String, String> rowMap = null;
        String rowkey = Constants.BLANK;
        List<String> roletypes = new ArrayList<String>();
        for (int i = 0; i < resourceIds.size(); i++) {
            rowkey = resourceIds.get(i);
            rowMap = roleMap.get(rowkey);
            if (rowMap != null) {
                roletypes.add(StringUtil.parseString(rowMap.get(PmMapKeySet.KEY_R_ROLE_TYPE)));
            }
        }

        // 查找用户在当前资源及上级资源中权限最高的权限
        String retStr = Constants.BLANK;
        for (int i = 0; i < roletypes.size(); i++) {
            if (i == 0) {
                retStr = roletypes.get(i);
            } else {
                if (retStr.compareTo(roletypes.get(i)) > 0) {
                    retStr = roletypes.get(i);
                }
            }
        }

        // 一个都没有找到-则使用最低权限的角色类型
        if (Constants.BLANK.equals(retStr)) {
            retStr = ReliaFlowConstants.ROLE_PROJECT_STAFF;
        }

        return retStr;
    }

    /**
     * @param argMap
     * @return Map 权限集合
     * @Title: findViewAuthorityByState
     * @Description 取得状态权限的查看权限 (01:基本信息 02:检查列表 03:成员 04:文档 05:阶段 06:下级别)
     */
    private Map<String, String> findViewAuthorityByState(Map<String, String> argMap) {
        argMap.put(PmMapKeySet.KEY_S_AUTH_CLASSIFY, ReliaFlowConstants.AUTHORITY_CLASSIFY_STATE);
        Map<String, String> retMap = new HashMap<String, String>();
        List<PmAuthority> pmAuthoritys = searchAuthMapfromCache(argMap, 0);

        // 中间状态处理(状态权限表追加)
        if (pmAuthoritys == null || pmAuthoritys.size() <= 0) {
            argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, ReliaFlowConstants.LIFECYCLE_APPROVE_STATE);
            pmAuthoritys = searchAuthMapfromCache(argMap, 0);
        }

        // 各子资源查看权限
        for (PmAuthority stateAuth : pmAuthoritys) {
            switch (Integer.parseInt(stateAuth.getChildResourceType())) {
                case 1:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_BASIC, stateAuth.getViewAuthority());
                    retMap.put(ReliaFlowConstants.EDIT_AUTHORITY_BASIC, stateAuth.getModiftAuthority());
                    break;
                case 2:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_CHECK, stateAuth.getViewAuthority());
                    break;
                case 3:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_MEMBER, stateAuth.getViewAuthority());
                    break;
                case 4:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_DOCUMENT, stateAuth.getViewAuthority());
                    break;
                case 5:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_STAGE, stateAuth.getViewAuthority());
                    break;
                case 6:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_CHILD, stateAuth.getViewAuthority());
                    if (ReliaFlowConstants.RESOURCE_TASK_TYPE.equals(argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE))) {
                        retMap.put(ReliaFlowConstants.AUTHORITY_CREATE, stateAuth.getCreateAuthority());
                        retMap.put(ReliaFlowConstants.AUTHORITY_MODIFY, stateAuth.getModiftAuthority());
                        retMap.put(ReliaFlowConstants.AUTHORITY_DELETE, stateAuth.getDeleteAuthority());
                        retMap.put(ReliaFlowConstants.AUTHORITY_EXT01, stateAuth.getExt1Authority());
                    }
                    break;
                case 7:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_TOOL, stateAuth.getViewAuthority());
                    retMap.put(ReliaFlowConstants.CREATE_AUTHORITY_TOOL, stateAuth.getCreateAuthority());
                    break;
                default:
                    break;
            }
        }
        return retMap;
    }

    /**
     * @param argMap
     * @return Map 权限集合
     * @Title: findViewAuthorityByRole
     * @Description 取得角色权限的查看权限 (01:基本信息 02:检查列表 03:成员 04:文档 05:阶段 06:下级别)
     */
    private Map<String, String> findViewAuthorityByRole(Map<String, String> argMap) {
        argMap.put(PmMapKeySet.KEY_S_AUTH_CLASSIFY, ReliaFlowConstants.AUTHORITY_CLASSIFY_ROLE);
        Map<String, String> retMap = new HashMap<String, String>();
        List<PmAuthority> pmAuthoritys = searchAuthMapfromCache(argMap, 1);

        // 各子资源查看权限
        for (PmAuthority roleAuth : pmAuthoritys) {
            switch (Integer.parseInt(roleAuth.getChildResourceType())) {
                case 1:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_BASIC, roleAuth.getViewAuthority());
                    retMap.put(ReliaFlowConstants.EDIT_AUTHORITY_BASIC, roleAuth.getModiftAuthority());
                    break;
                case 2:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_CHECK, roleAuth.getViewAuthority());
                    break;
                case 3:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_MEMBER, roleAuth.getViewAuthority());
                    break;
                case 4:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_DOCUMENT, roleAuth.getViewAuthority());
                    break;
                case 5:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_STAGE, roleAuth.getViewAuthority());
                    break;
                case 6:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_CHILD, roleAuth.getViewAuthority());
                    if (ReliaFlowConstants.RESOURCE_TASK_TYPE.equals(argMap.get(PmMapKeySet.KEY_S_RESOURCE_TYPE))) {
                        retMap.put(ReliaFlowConstants.AUTHORITY_CREATE, convertMiddleStateByState(roleAuth.getCreateAuthority(), argMap));
                        retMap.put(ReliaFlowConstants.AUTHORITY_MODIFY, convertMiddleStateByState(roleAuth.getModiftAuthority(), argMap));
                        retMap.put(ReliaFlowConstants.AUTHORITY_DELETE, convertMiddleStateByState(roleAuth.getDeleteAuthority(), argMap));
                        retMap.put(ReliaFlowConstants.AUTHORITY_EXT01, convertMiddleStateByState(roleAuth.getExt1Authority(), argMap));
                    }
                    break;
                case 7:
                    retMap.put(ReliaFlowConstants.VIEW_AUTHORITY_TOOL, roleAuth.getViewAuthority());
                    retMap.put(ReliaFlowConstants.CREATE_AUTHORITY_TOOL, roleAuth.getCreateAuthority());
                    break;
                default:
                    break;
            }
        }
        return retMap;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: findResourceAuthorityByState
     * @Description: 根据状态取得特定资源的各种权限
     */
    private Map<String, String> findResourceAuthorityByState(Map<String, String> argMap) {
        argMap.put(PmMapKeySet.KEY_S_AUTH_CLASSIFY, ReliaFlowConstants.AUTHORITY_CLASSIFY_STATE);
        Map<String, String> retMap = new HashMap<String, String>();
        List<PmAuthority> pmAuthoritys = searchAuthMapfromCache(argMap, 0);

        // 中间状态处理(状态权限表追加)
        if (pmAuthoritys == null || pmAuthoritys.size() <= 0) {
            argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, ReliaFlowConstants.LIFECYCLE_APPROVE_STATE);
            pmAuthoritys = searchAuthMapfromCache(argMap, 0);
        }

        // 封装权限Map
        PmAuthority pmAuthority = pmAuthoritys.get(0);
        retMap.put(ReliaFlowConstants.AUTHORITY_CREATE, pmAuthority.getCreateAuthority());
        retMap.put(ReliaFlowConstants.AUTHORITY_MODIFY, pmAuthority.getModiftAuthority());
        retMap.put(ReliaFlowConstants.AUTHORITY_DELETE, pmAuthority.getDeleteAuthority());
        retMap.put(ReliaFlowConstants.AUTHORITY_VIEW, pmAuthority.getViewAuthority());
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT01, pmAuthority.getExt1Authority());
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT02, pmAuthority.getExt2Authority());
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT03, pmAuthority.getExt3Authority());
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT04, pmAuthority.getExt4Authority());

        return retMap;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: findResourceAuthorityByRole
     * @Description: 根据角色取得特定资源的各种权限
     */
    private Map<String, String> findResourceAuthorityByRole(Map<String, String> argMap) {
        argMap.put(PmMapKeySet.KEY_S_AUTH_CLASSIFY, ReliaFlowConstants.AUTHORITY_CLASSIFY_ROLE);
        Map<String, String> retMap = new HashMap<String, String>();
        List<PmAuthority> pmAuthoritys = searchAuthMapfromCache(argMap, 1);

        // 封装权限Map
        PmAuthority pmAuthority = pmAuthoritys.get(0);
        retMap.put(ReliaFlowConstants.AUTHORITY_CREATE, convertMiddleStateByState(pmAuthority.getCreateAuthority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_MODIFY, convertMiddleStateByState(pmAuthority.getModiftAuthority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_DELETE, convertMiddleStateByState(pmAuthority.getDeleteAuthority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_VIEW, convertMiddleStateByState(pmAuthority.getViewAuthority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT01, convertMiddleStateByState(pmAuthority.getExt1Authority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT02, convertMiddleStateByState(pmAuthority.getExt2Authority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT03, convertMiddleStateByState(pmAuthority.getExt3Authority(), argMap));
        retMap.put(ReliaFlowConstants.AUTHORITY_EXT04, convertMiddleStateByState(pmAuthority.getExt4Authority(), argMap));

        return retMap;
    }

    /**
     * @return Map<String, String>
     * @Title: findResourceStateMap
     * @Description: 取得资源状态Map
     * 被私有方法调用(searchLifeState)
     */
    private Map<String, String> findResourceStateMap() {

        Map<String, String> retStateMap = (Map<String, String>) SessionUtil.getAttribute(SessionKeys.RB_PJ_RESOURCE_SELF_STATE);

        if (retStateMap == null) {
            // 获得配置表信息
            List<SysStateConfig> sysStateConfigList = new ArrayList<SysStateConfig>();
            sysStateConfigList.addAll(sysStateConfigMapper.selectByStateType(BizModuleCode.MODULE_ID_PROJECT, BizcommonConstants.STATE_TYPE_1));
            sysStateConfigList.addAll(sysStateConfigMapper.selectByStateType(BizModuleCode.MODULE_ID_TG, BizcommonConstants.STATE_TYPE_1));
            sysStateConfigList.addAll(sysStateConfigMapper.selectByStateType(BizModuleCode.MODULE_ID_TASK, BizcommonConstants.STATE_TYPE_1));

            // 编辑状态配置
            retStateMap = new HashMap<String, String>();
            for (SysStateConfig stateConfig : sysStateConfigList) {
                if (BizModuleCode.MODULE_ID_PROJECT.equals(stateConfig.getModuleId())) {
                    retStateMap.put(stateConfig.getStateCode() + ReliaFlowConstants.RESOURCE_PROJECT_TYPE, stateConfig.getStateCode());
                } else if (BizModuleCode.MODULE_ID_TG.equals(stateConfig.getModuleId())) {
                    retStateMap.put(stateConfig.getStateCode() + ReliaFlowConstants.RESOURCE_TG_TYPE, stateConfig.getStateCode());
                } else if (BizModuleCode.MODULE_ID_TASK.equals(stateConfig.getModuleId())) {
                    retStateMap.put(stateConfig.getStateCode() + ReliaFlowConstants.RESOURCE_TASK_TYPE, stateConfig.getStateCode());
                }
            }
            SessionUtil.setAttribute(SessionKeys.RB_PJ_RESOURCE_SELF_STATE, retStateMap);
        }

        return retStateMap;
    }

    /**
     * @param argMap
     * @return List 用户在项目中所担当的角色集
     * @Title: findUserRoleAuthInProject
     * @Description 取出用户Id在当前项目中查找用户所充当的角色
     * 被私有方法调用(searchRoleType)
     */
    private List<Map<String, String>> findUserRoleAuthInProject(Map<String, String> argMap) {

        List<Map<String, String>> rolesList = null;

        if (!ReliaFlowConstants.IS_SEARCH_YES.equals(argMap.get(PmMapKeySet.KEY_S_SEARCH_ROLE))) {
            rolesList = (List<Map<String, String>>) SessionUtil.getAttribute(SessionKeys.RB_PJ_USER_ROLE_TYPE);
        } else {
            argMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_NO);
        }

        // 所担当的角色集
        if (rolesList == null || rolesList.size() <= 0) {
            rolesList = pmProjectRoleMapper.searchUserRoleAuthInProject(argMap);
            SessionUtil.setAttribute(SessionKeys.RB_PJ_USER_ROLE_TYPE, rolesList);
        }

        return rolesList;
    }

    /**
     * @param argMap
     * @return Map<String, String>
     * @Title: convertAuthView
     * @Description: 权限控制方式转换-(可见性)
     */
    private Map<String, String> convertAuthView(Map<String, String> argMap) {
        Map<String, String> authMap = new HashMap<String, String>();
        for (Iterator<String> iter = argMap.keySet().iterator(); iter.hasNext(); ) {
            String key = iter.next();
            authMap.put(key, ReliaFlowConstants.IS_AUTHORITY_YES.equals(argMap.get(key)) ? Constants.BLANK : ReliaFlowConstants.AUTH_CTRL_VIEW);
        }

        return authMap;
    }

    /**
     * @param authState
     * @param argMap
     * @return String
     * @Title: convertMiddleStateByState
     * @Description: 通过状态将角色对应的中间权限状态转化为稳定态
     */
    private String convertMiddleStateByState(String authState, Map<String, String> argMap) {
        String retAuth;

        if (ReliaFlowConstants.MID2_AUTHORITY_BY_STATE.equals(authState)) {
            if (ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.equals(argMap.get(PmMapKeySet.KEY_S_LIFE_STATE))) {
                retAuth = ReliaFlowConstants.IS_AUTHORITY_YES;
            } else {
                retAuth = ReliaFlowConstants.IS_AUTHORITY_NO;
            }
        } else {
            retAuth = authState;
        }

        return retAuth;
    }
}
