package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.config.SysConfig;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.service.PmPercentStatService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName PmStageProcessServiceImpl
 * @Description 可靠性项目状态迁移服务实现类
 * @Author ted.liu
 * @CreateDate 2013-12-10
 */
@SuppressWarnings("unchecked")
@Service
public class PmPercentStatServiceImpl implements PmPercentStatService {

    @Autowired
    private SysConfig sysConfig;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;

    /**
     * @param argMap TG标识 | 开始结点的标识（追加说明-计算那个结点用那个结点的主键做参数。存储过程中会以传入参数选取它的子结点统计。最后生成传入参数的那个记录）
     * @return Map<String, BigDecimal>
     * @Title: percent
     * @Description: 统计所有结点的完成度.
     */
    @Override
    public Map<String, BigDecimal> statAllNodePercent(Map<String, Object> argMap) {
        Map<String, BigDecimal> retMap = new HashMap<String, BigDecimal>();
        // 统计方式(0:工作中的不统计|1:工作中的统计)
        argMap.put(PmMapKeySet.KEY_S_STAT_WAY, sysConfig.getPercentStatMethod());
        // 权重是否有效(0:无效|1:有效)
        argMap.put(PmMapKeySet.KEY_S_SHARE_SCALE_SWITCH, sysConfig.getPercentScaleSwitch());
        // 返回类型
        argMap.put(PmMapKeySet.KEY_S_RETURN_WAY, ReliaFlowConstants.PERCENT_RESULT_ALL);

        // 统计计算
        pmTaskTreeMapper.statPercent(argMap);

        // 取得统计结果
        List<Map<String, Object>> listPercent = (List<Map<String, Object>>) argMap.get(PmMapKeySet.KEY_S_RETURN);
        if (listPercent != null && listPercent.size() > 0) {
            for (Map<String, Object> map : listPercent) {
                retMap.put(String.valueOf(map.get(PmMapKeySet.KEY_R_TASK_ID)), new BigDecimal(StringUtil.parseString(map.get(PmMapKeySet.KEY_R_PERCENT), "0")));
            }
        }
        return retMap;
    }

    /**
     * @param argMap TG标识 | 开始结点的标识（追加说明-计算那个结点用那个结点的主键做参数。存储过程中会以传入参数选取它的子结点统计。最后生成传入参数的那个记录）
     * @return BigDecimal
     * @Title: statCurrentNodePercent
     * @Description: 统计当前结点的完成度.仅仅统计工作中的
     */
    @Override
    public BigDecimal statCurrentNodePercent(Map<String, Object> argMap) {
        // 统计方式(0:工作中的不统计|1:工作中的统计)
        argMap.put(PmMapKeySet.KEY_S_STAT_WAY, sysConfig.getPercentStatMethod());
        // 权重是否有效(0:无效|1:有效)
        argMap.put(PmMapKeySet.KEY_S_SHARE_SCALE_SWITCH, sysConfig.getPercentScaleSwitch());
        // 返回类型
        argMap.put(PmMapKeySet.KEY_S_RETURN_WAY, ReliaFlowConstants.PERCENT_RESULT_CURRENT);

        // 统计计算
        pmTaskTreeMapper.statPercent(argMap);

        // 取得统计结果
        List<Map<String, Object>> listPercent = (List<Map<String, Object>>) argMap.get(PmMapKeySet.KEY_S_RETURN);
        if (listPercent != null && listPercent.size() > 0) {
            return new BigDecimal(StringUtil.parseString(listPercent.get(0).get(PmMapKeySet.KEY_R_PERCENT), Constants.C_STAT_0));
        } else {
            return new BigDecimal(Constants.C_NUM_0);
        }
    }

    /**
     * @param pjLifeState
     * @param tgPercents
     * @return BigDecimal
     * @Title: statProjectPercent
     * @Description: 根据TG列表统计项目完成度
     */
    @Override
    public BigDecimal statProjectPercent(String pjLifeState, List<Map<String, Object>> tgPercents) {
        // 项目创建阶段
        if (ReliaFlowConstants.LIFECYCLE_RELIA_PJ_INWORK.compareTo(pjLifeState) > 0) {
            return null;
        }

        if (tgPercents == null || tgPercents.size() <= 0) {
            return new BigDecimal(Constants.C_NUM_0);
        }

        // 统计项目进度
        BigDecimal percentAll = new BigDecimal(Constants.C_NUM_0);
        BigDecimal taskScale = new BigDecimal(Constants.C_NUM_0);
        for (Map<String, Object> tmpTg : tgPercents) {
            percentAll = percentAll.add(tmpTg.get(PmMapKeySet.KEY_R_PERCENT) == null ? new BigDecimal(Constants.C_NUM_0) : (BigDecimal) tmpTg.get(PmMapKeySet.KEY_R_PERCENT));
            taskScale = taskScale.add(new BigDecimal(100));
        }

        return percentAll.multiply(new BigDecimal(100)).divide(taskScale, 2, RoundingMode.HALF_UP);
    }
}
