package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.PmProjectService;
import com.nationrel.modules.project.service.PmTgService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @version V1.0
 * @ClassName PmProjectServiceImpl
 * @Description 可靠性项目服务接口
 * @Author andy
 * @CreateDate 2014-4-3
 */
@Service
public class PmProjectServiceImpl implements PmProjectService {

    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private PmProjectCheckMapper pmProjectCheckMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmProjectDeliveryMapper pmProjectDeliveryMapper;
    @Autowired
    private PmTgService pmTgService;

    /**
     * @param projectId
     * @Title deleteWholeProject
     * @Description 项目删除
     */
    @Override
    public void deleteWholeProject(String projectId) {
        if (projectId == null || projectId.equals(Constants.BLANK)) {
            return;
        }

        // 删除项目下所有TG
        List<PmTg> tgList = pmTgMapper.selectPmTgByProjectId(projectId);
        if (tgList != null && tgList.size() > 0) {
            for (PmTg pmTg : tgList) {
                pmTgService.deleteWholeTG(pmTg.getTgId());
            }
        }

        // 删除项目下检查列表
        pmProjectCheckMapper.deleteCheckItemByProjectId(projectId);

        // 删除项目下成员列表
        pmProjectStaffMapper.deleteStaffByProjectId(projectId);

        // 删除项目下交付列表
        pmProjectDeliveryMapper.deleteDeliveryByProjectId(projectId);

        // 删除项目信息
        pmProjectMapper.deleteByPrimaryKey(projectId);
    }

    /**
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param projectId
     * @throws Exception
     * @Title checkContainTimeOfTG
     * @Description 检查新输入时间是否包括所有TG的时间
     */
    @Override
    public void checkContainTimeOfTG(Date dtPlanStartTime, Date dtPlanEndTime, String projectId) throws Exception {

        // 取得项目内TG时间不在指定时间范围内的TG
        List<PmTg> pmTgs = pmTgMapper.checkContainTimeOfTG(dtPlanStartTime, dtPlanEndTime, projectId);

        // 若记录存在抛出异常
        if (pmTgs.size() > 0) {
            throw new ApplicationRuntimeException(MessageId.E300116);
        }
    }
}
