package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.PmTaskService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
public class PmTaskServiceImpl implements PmTaskService {

    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;
    @Autowired
    private PmTaskCheckMapper pmTaskCheckMapper;
    @Autowired
    private PmTaskDeliveryMapper pmTaskDeliveryMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmTaskToolMapper pmTaskToolMapper;

    /**
     * @param taskId: task标识
     * @return void
     * @Title: checkTaskExist
     * @Description:检查Task是否存在
     */
    @Override
    public void checkTaskExist(String taskId) throws Exception {
        // 获取Task信息
        PmTask pmTask = pmTaskMapper.selectByPrimaryKey(taskId);

        if (pmTask == null) {
            throw new OptimisticLockException(MessageId.W300104);
        }
    }

    /**
     * @param taskId void
     * @Title: deleteTaskByTaskId
     * @Description: 删除整个Task下所有子资源(人员 \ 工具等)
     */
    @Override
    public void deleteTaskByTaskNode(Integer leftNo, Integer rightNo, Integer ordinal, String tgId) {
        if (StringUtils.isEmpty(tgId)) {
            return;
        }

        // 删除Task下检查列表
        pmTaskCheckMapper.deleteCheckItemByTaskNode(leftNo, rightNo, ordinal, tgId);

        // 删除Task下成员列表
        pmTaskStaffMapper.deleteTaskStaffByTaskNode(leftNo, rightNo, ordinal, tgId);

        // 删除Task下交付列表
        pmTaskDeliveryMapper.deleteDeliveryByTaskNode(leftNo, rightNo, ordinal, tgId);

        // 删除Task下工具
        pmTaskToolMapper.deleteTaskToolByTaskNode(leftNo, rightNo, ordinal, tgId);

        // 删除Task信息
        //pmTaskMapper.deleteByPrimaryKey(taskId);
    }

    /**
     * @param taskId void
     * @Title: deleteTaskByTgId
     * @Description: 删除Tg下所有任务、任务树、交付、检查项、人员
     */
    @Override
    public void deleteWholeTaskByTgId(String tgId) {
        if (StringUtils.isEmpty(tgId)) {
            return;
        }

        // 删除TG下的所有Task下的检查列表
        pmTaskCheckMapper.deleteCheckItemByTgId(tgId);

        // 删除TG下的所有Task下的成员列表
        pmTaskStaffMapper.deleteStaffByTgId(tgId);

        // 删除TG下的所有Task下的交付列表
        pmTaskDeliveryMapper.deleteDeliveryByTgId(tgId);

        // 删除Task下工具
        pmTaskToolMapper.deleteTaskToolByTgId(tgId);

        // 删除TG下的所有Task信息
        pmTaskMapper.deleteTaskByTgId(tgId);

        // 删除TG下的所有任务树
        pmTaskTreeMapper.deleteTaskTreeByTgId(tgId);
    }

    /**
     * @param planStartTime
     * @param planEndTime
     * @param projectId
     * @param tgId
     * @param taskId
     * @param parentTaskId
     * @throws Exception void
     * @Title: checkTimeValid
     * @Description: 任务整合性检查
     */
    @Override
    public void checkTimeValid(Date dtPlanStartTime, Date dtPlanEndTime, String projectId, String tgId, String taskId, String parentTaskId) throws Exception {

        // 1.当前TASK的时间范围在TG或上级TASK时间范围之内
        if (ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(parentTaskId)) {

            // 在TG的日期范围之内检查
            PmTg pmTg = pmTgMapper.checkContainCurTimeOfTask(dtPlanStartTime, dtPlanEndTime, tgId);
            if (pmTg == null) {
                throw new ApplicationRuntimeException(MessageId.E300119);
            }
        } else {

            // 在上级任务的日期范围之内检查
            PmTask pmTask = pmTaskMapper.checkContainCurTimeOfChildTask(dtPlanStartTime, dtPlanEndTime, parentTaskId);
            if (pmTask == null) {
                throw new ApplicationRuntimeException(MessageId.E300120);
            }
        }

        // 2.当前TASK的时间范围包括所有子项目的时间(编辑)
        if (StringUtils.isNotEmpty(taskId)) {
            PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
            int errorTimeTasks = pmTaskTreeMapper.checkContainTimeOfChildTask(dtPlanStartTime, dtPlanEndTime, tgId,
                    pmTaskTree.getOrdinal(), pmTaskTree.getLeftNo(), pmTaskTree.getRightNo());
            if (errorTimeTasks > 0) {
                throw new ApplicationRuntimeException(MessageId.E300121);
            }
        }
    }
}
