package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.entity.PpPlan;
import com.nationrel.modules.project.entity.PpPlanTrace;
import com.nationrel.modules.project.entity.PtTrace;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMapper;
import com.nationrel.modules.project.mapper.PpPlanTraceMapper;
import com.nationrel.modules.project.mapper.PtTraceMapper;
import com.nationrel.modules.project.service.PpPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * @version V1.0
 * @ClassName PmStageProcessServiceImpl
 * @Description 预防性措施规划服务实现类
 * @Author ted.liu
 * @CreateDate 2013-12-10
 */
@Service
public class PpPlanServiceImpl implements PpPlanService {

    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PpPlanTraceMapper ppPlanTraceMapper;
    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PtTraceMapper ptTraceTaskMapper;

    /**
     * @param planId:预防性措施规划标识
     * @return String[] 例如:{[20140501, 20140502,.....],[10, 9,....]}
     * @throws Exception
     * @Title: generateChatForPlan
     * @Description:生成预防性措施规划作图数据
     */
    @Override
    public String[] generateChatDataForPlan(String planId) throws Exception {
        String[] result = null;
        if (planId == null || planId.equals(Constants.BLANK)) {
            return result;
        }
        result = new String[2];

        // 获取预防性规划下所有风险系数规划数据
        List<PpPlanTrace> traceList = ppPlanTraceMapper.selectPpPlanTraceListByPpPlanId(planId);

        // 获取预防性措施规划数据
        PpPlan ppPlan = ppPlanMapper.selectByPrimaryKey(planId);

        // 构造最终返回数据
        BigDecimal currentValue = ppPlan.getRiskBeforeValue();
        StringBuffer timeStr = new StringBuffer();
        StringBuffer dataStr = new StringBuffer();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat format = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", Locale.US);
        for (int i = 0; i < traceList.size(); i++) {
            PpPlanTrace ppPlanTrace = traceList.get(i);
            if (i > 0) {
                currentValue = currentValue.subtract(ppPlanTrace.getRiskDownValue());
            }
            if (i < traceList.size() - 1) {
                timeStr.append(dateFormat.format(format.parse(ppPlanTrace.getPpTime().toString())) + ",");
                dataStr.append(currentValue + ",");
            } else {
                timeStr.append(dateFormat.format(format.parse(ppPlanTrace.getPpTime().toString())));
                dataStr.append(currentValue);
            }
        }
        result[0] = timeStr.toString();
        result[1] = dataStr.toString();

        return result;
    }

    /**
     * @param planId:预防性措施规划标识
     * @return String[] 例如:{[20140501, 20140502,.....],[10, 9,....],[10, 8,....]}
     * @throws Exception
     * @Title: generateChatDataForPractice
     * @Description: 生成预防性规划实际风险降低数据
     */
    @Override
    public List<String[]> generateChatDataForProactice(String planId) throws Exception {
        List<String[]> result = null;
        if (planId == null || planId.equals(Constants.BLANK)) {
            return result;
        }
        result = new ArrayList<String[]>();

        // 获取预防性措施规划数据
        PpPlan ppPlan = ppPlanMapper.selectByPrimaryKey(planId);

        //====================
        // 获取预防性规划下所有风险系数规划数据
        //====================
        List<PpPlanTrace> traceList = ppPlanTraceMapper.selectPpPlanTraceListByPpPlanId(planId);

        // 构造最终返回数据
        BigDecimal currentValue = ppPlan.getRiskBeforeValue();
        StringBuffer timeStr = new StringBuffer();
        StringBuffer dataStr = new StringBuffer();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat format = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", Locale.US);
        for (int i = 0; i < traceList.size(); i++) {
            PpPlanTrace ppPlanTrace = traceList.get(i);
            if (i > 0) {
                currentValue = currentValue.subtract(ppPlanTrace.getRiskDownValue());
            }
            if (i < traceList.size() - 1) {
                timeStr.append(dateFormat.format(format.parse(ppPlanTrace.getPpTime().toString())) + ",");
                dataStr.append(currentValue + ",");
            } else {
                timeStr.append(dateFormat.format(format.parse(ppPlanTrace.getPpTime().toString())));
                dataStr.append(currentValue);
            }
        }
        String[] planResult = new String[2];
        planResult[0] = timeStr.toString();
        planResult[1] = dataStr.toString();
        result.add(planResult);

        // 获取预防性规划下风险系数追踪数据
        List<PtTrace> ptTraceList = ptTraceTaskMapper.getPtTraceListByPlanId(planId);
        StringBuffer ptTimeStr = new StringBuffer();
        StringBuffer ptDataStr = new StringBuffer();
        SimpleDateFormat simpleFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (int i = 0; i < ptTraceList.size(); i++) {
            PtTrace ptTrace = ptTraceList.get(i);
            if (i == 0) {
                ptTimeStr.append(DateUtils.convertDatetoString(traceList.get(0).getPpTime()) + ",");
                ptDataStr.append(ppPlan.getRiskBeforeValue() + ",");
            }
            if (i < ptTraceList.size() - 1) {
                ptTimeStr.append(dateFormat.format(simpleFormat.parse(ptTrace.getPpTime())) + ",");
                ptDataStr.append(ptTrace.getRiskValue() + ",");
            } else {
                ptTimeStr.append(dateFormat.format(simpleFormat.parse(ptTrace.getPpTime())));
                ptDataStr.append(ptTrace.getRiskValue());
            }
        }
        String[] ptResult = new String[2];
        ptResult[0] = ptTimeStr.toString();
        ptResult[1] = ptDataStr.toString();
        result.add(ptResult);

        return result;
    }

}
