package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300101InitInput;
import com.nationrel.modules.project.dto.output.Rb0300101InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.service.Rb0300101InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300101InitServiceImpl
 * @Description 项目列表初始化业务类
 * @Author andy
 * @CreateDate 2013-12-4
 */
@Service
public class Rb0300101InitServiceImpl implements Rb0300101InitService {

    @Autowired
    private PmProjectMapper pmProjectMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0300101InitInput input, Rb0300101InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        SessionUtil.removeAttribute(SessionKeys.RB_PJ_SEARCH_CONDITION);
        // 查询条件做成
        Map<String, Object> argMap = new HashMap<String, Object>();
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());

        // 翻页处理
        int totalCount = pmProjectMapper.countProjectByCondition(argMap);
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            pagination = new Pagination();
        }
        pagination.setRowTotal(totalCount);
        // 统计显示操作按钮的数量
        int ctrOptCount = 1;
        // 数据取得
        List<Map<String, Object>> mapList = pmProjectMapper.searchProjectByCondition(pagination, argMap);
        for (Map<String, Object> pmProjectMap : mapList) {
            if (StringUtil.parseString(pmProjectMap.get(PmMapKeySet.KEY_R_LIFE_STATE)).equals(ReliaFlowConstants.LIFECYCLE_RELIA_PJ_CREATE)) {
                ctrOptCount++;
                break;
            }
        }
        output.setPagination(pagination);
        output.setResultList(mapList);
        output.setCtrOptCount(ctrOptCount);

        // 创建权限处理(特殊权限系统级)
        SessionUtil.setAttribute(SessionKeys.RB_PJ_CREATE_AUTHORITY, ReliaFlowConstants.IS_AUTHORITY_NO);
        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
