package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300101SearchInput;
import com.nationrel.modules.project.dto.output.Rb0300101SearchOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.service.Rb0300101SearchService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300101SearchServiceImpl
 * @Description 流程列表搜索业务类
 * @Author ted.liu
 * @CreateDate 2013-12-4
 */
@Service
public class Rb0300101SearchServiceImpl implements Rb0300101SearchService {

    @Autowired
    private PmProjectMapper pmProjectMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @SuppressWarnings("unchecked")
    @Override
    public String perform(Rb0300101SearchInput input, Rb0300101SearchOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 查询条件
        Map<String, Object> argMap = new HashMap<String, Object>();

        // 翻页处理
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            editSearchCondition(input, argMap);
            SessionUtil.setAttribute(SessionKeys.RB_PJ_SEARCH_CONDITION, argMap);
            pagination = new Pagination();
        } else {
            argMap = (HashMap<String, Object>) SessionUtil.getAttribute(SessionKeys.RB_PJ_SEARCH_CONDITION);
            if (argMap == null) {
                argMap = new HashMap<String, Object>();
                editSearchCondition(null, argMap);
            }
        }

        // 数据条数取得
        int totalCount = pmProjectMapper.countProjectByCondition(argMap);
        pagination.setRowTotal(totalCount);

        // 数据取得
        List<Map<String, Object>> mapList = pmProjectMapper.searchProjectByCondition(pagination, argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 1;
        for (Map<String, Object> pmProjectMap : mapList) {
            if (StringUtil.parseString(pmProjectMap.get(PmMapKeySet.KEY_R_LIFE_STATE)).equals(ReliaFlowConstants.LIFECYCLE_RELIA_PJ_CREATE)) {
                ctrOptCount++;
                break;
            }
        }

        // 设置输出
        output.setPagination(pagination);
        output.setResultList(mapList);
        output.setQueryMap(argMap);
        output.setCtrOptCount(ctrOptCount);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**

     * @param input
     * @param argMap   void
     * @Title: editSearchCondition
     * @Description: 设定查询条件
     */
    private void editSearchCondition(
            Rb0300101SearchInput input, Map<String, Object> argMap) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        if (input != null) {
            argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
            argMap.put(PmMapKeySet.KEY_S_PROJECT_CODE, input.getProjectCode());
            argMap.put(PmMapKeySet.KEY_S_PROJECT_NAME, input.getProjectName());
            argMap.put(PmMapKeySet.KEY_S_PROJECT_TYPE, input.getProjectType());
            argMap.put(PmMapKeySet.KEY_S_RELATED_OBJECT, input.getRelatedObject());
            argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, input.getLifeState());
        } else {
            argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
            argMap.put(PmMapKeySet.KEY_S_PROJECT_CODE, Constants.BLANK);
            argMap.put(PmMapKeySet.KEY_S_PROJECT_NAME, Constants.BLANK);
            argMap.put(PmMapKeySet.KEY_S_PROJECT_TYPE, Constants.BLANK);
            argMap.put(PmMapKeySet.KEY_S_RELATED_OBJECT, Constants.BLANK);
            argMap.put(PmMapKeySet.KEY_S_LIFE_STATE, Constants.BLANK);
        }
    }
}
