package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300102DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300102DeleteOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300102DeleteService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300102DeleteServiceImpl
 * @Description TG列表页面初始化的业务类
 * @Author andy
 * @CreateDate 2013-12-17
 */
@Service
public class Rb0300102DeleteServiceImpl implements Rb0300102DeleteService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTgMapper pmTgMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG列表初始化的业务方法
     */
    @Override
    public String perform(Rb0300102DeleteInput input, Rb0300102DeleteOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        // 获取tgId
        String tgId = input.getTgId();

        // 取得TG信息
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);
        if (pmTg == null) {
            throw new OptimisticLockException(MessageId.W300103);
        }

        // 页面权限控制
        Map<String, String> argTgAuthMap = new HashMap<String, String>();
        argTgAuthMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argTgAuthMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argTgAuthMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argTgAuthMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argTgAuthMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
        argTgAuthMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

        // 取得具体资源的权限
        Map<String, String> tgAuthMap = authorityCtrlService.doResourceAuthority(argTgAuthMap);
        String delAuth = tgAuthMap.get(ReliaFlowConstants.AUTHORITY_DELETE);

        if (!ReliaFlowConstants.LIFECYCLE_RELIA_TG_CREATE.equals(pmTg.getLifeState())) {
            delAuth = ReliaFlowConstants.AUTH_CTRL_VIEW;
        }

        // 权限发生变化-操作不能进行
        if (StringUtils.isNotEmpty(delAuth)) {
            throw new OptimisticLockException(MessageId.W009805);
        }

        // 根据tgId删除Tg对象
        pmTgService.deleteWholeTG(tgId);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
