package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300102InitInput;
import com.nationrel.modules.project.dto.output.Rb0300102InitOutput;
import com.nationrel.modules.project.entity.ProjectPageModel;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmPercentStatService;
import com.nationrel.modules.project.service.Rb0300102InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300102InitServiceImpl
 * @Description 查看项目对象初始化的业务实现类
 * @Author andy
 * @CreateDate 2013-12-12
 */
@Service
public class Rb0300102InitServiceImpl implements Rb0300102InitService {

    @Autowired
    private PmProjectMapper projectMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmPercentStatService pmPercentStatService;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0300102InitInput input, Rb0300102InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 页面权限控制
        SessionUtil.removeAttribute(SessionKeys.RB_PJ_USER_ROLE_TYPE);

        // 取得页面项目ID
        String projectId = input.getProjectId();
        if (projectId == null || Constants.BLANK.equals(projectId)) {
            projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03001_PROJECT_ID, projectId);
        }

        // 查询项目信息
        List<ProjectPageModel> projects = projectMapper.findProjectById(projectId);

        // 设置TG能否上移下移
        //setTgMoveAble(projects);

        // 页面权限控制
        Map<String, String> argTgAuthMap = new HashMap<String, String>();
        argTgAuthMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argTgAuthMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argTgAuthMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argTgAuthMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argTgAuthMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);

        // 取得具体资源的权限
        Map<String, String> tgAuthMap = authorityCtrlService.doResourceAuthority(argTgAuthMap);

        // 列表对象查看权限
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);
        Map<String, Map<String, String>> authResourceMap = authorityCtrlService.doViewAuthorityByGroup(argMap);

        // 完成度计算
        List<Map<String, Object>> tgPercents = new ArrayList<Map<String, Object>>();
        Map<String, Object> tgMap;
        Map<String, Object> argPercentMap = new HashMap<String, Object>();
        Map<String, String> pjAuthMap;
        boolean ctrlDelResult = false;
        // 统计显示操作按钮的数量
        int ctrOptCount = 0;
        for (ProjectPageModel object : projects) {
            pjAuthMap = authResourceMap.get(object.getId() + object.getDataType());
            // 项目下级TG权限控制
            if (ReliaFlowConstants.RESOURCE_TG_TYPE.equals(object.getDataType())) {
                // 控制TG是否可以删除 && 移动
                pjAuthMap.put(ReliaFlowConstants.AUTHORITY_DELETE, tgAuthMap.get(ReliaFlowConstants.AUTHORITY_DELETE));
                pjAuthMap.put(ReliaFlowConstants.AUTHORITY_EXT01, tgAuthMap.get(ReliaFlowConstants.AUTHORITY_EXT01));
                if (!object.getLifeState().equals(ReliaFlowConstants.LIFECYCLE_RELIA_TG_CREATE)) {
                    // 如果TG处于工作中或者完成，则TG不可移动
                    pjAuthMap.put(ReliaFlowConstants.AUTHORITY_DELETE, ReliaFlowConstants.AUTH_CTRL_VIEW);
                    pjAuthMap.put(ReliaFlowConstants.AUTHORITY_EXT01, ReliaFlowConstants.AUTH_CTRL_VIEW);
                }

                //  统计项目完成度参数作成
                tgMap = new HashMap<String, Object>();
                tgMap.put(PmMapKeySet.KEY_TG_ID, object.getId());
                tgMap.put(PmMapKeySet.KEY_R_PERCENT, object.getPercent());

                // 工作中TG完成度统计
                if (ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK.equals(object.getLifeState())
                        || ReliaFlowConstants.LIFECYCLE_RELIA_TG_WAIT_COMPLETE.equals(object.getLifeState())) {
                    argPercentMap.put(PmMapKeySet.KEY_S_TG_ID, object.getId());
                    argPercentMap.put(PmMapKeySet.KEY_S_START_NODE_ID, ReliaFlowConstants.TASK_FIRST_PARENT_ID);
                    BigDecimal percent = pmPercentStatService.statCurrentNodePercent(argPercentMap);
                    object.setPercent(percent);
                    tgMap.put(PmMapKeySet.KEY_R_PERCENT, percent);
                }
                tgPercents.add(tgMap);
                String deleteObj = pjAuthMap.get(ReliaFlowConstants.AUTHORITY_DELETE);
                if (StringUtils.isEmpty(deleteObj) && !ctrlDelResult) {
                    ctrlDelResult = true;
                }
            }
            object.setAuthMap(pjAuthMap);
        }

        if (ctrlDelResult) {
            ctrOptCount++;
        }

        // 项目完成度计算
        for (ProjectPageModel object : projects) {
            if (ReliaFlowConstants.RESOURCE_PROJECT_TYPE.equals(object.getDataType())) {
                // 统计项目完成度
                BigDecimal percent = pmPercentStatService.statProjectPercent(object.getLifeState(), tgPercents);
                object.setPercent(percent);
                output.setProjectLifeState(object.getLifeState());
                break;
            }
        }

        // 结果页面设定
        output.setCreateTGAble(tgAuthMap.get(ReliaFlowConstants.AUTHORITY_CREATE));
        output.setProjects(projects);
        output.setProjectId(projectId);
        output.setCtrOptCount(ctrOptCount);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @Title: setTgMoveAble
     * @Description: 设置TG能否上移下移
     * @param projects void
     */
//    private void setTgMoveAble(List<ProjectPageModel> projects) {
//        for (int i = 0 ; i < projects.size(); i++) {
//            ProjectPageModel item = projects.get(i);
//            if (ReliaFlowConstants.RESOURCE_TG_TYPE.equals(item.getDataType())) {
//
//                // 如果当前TG的状态是创建
//                if (ReliaFlowConstants.LIFECYCLE_RELIA_TG_CREATE.equals(item.getLifeState())) {
//                    if (projects.size() > 2) {
//                        ProjectPageModel previousItem = projects.get(i - 1 <= 0 ? 0 : i - 1);
//                        if (!ReliaFlowConstants.LIFECYCLE_RELIA_TG_CREATE.equals(previousItem.getLifeState()) || i == 1) {
//                            item.setUpMoveAble(false);
//                        }
//                        if (i == projects.size() - 1) {
//                            item.setDownMoveAble(false);
//                        }
//                    } else {
//                        item.setUpMoveAble(false);
//                        item.setDownMoveAble(false);
//                    }
//
//                } else { // 如果当前TG状态是工作中或者完成
//                    item.setUpMoveAble(false);
//                    item.setDownMoveAble(false);
//                }
//            }
//        }
//    }
}
