package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300103InitInput;
import com.nationrel.modules.project.dto.output.Rb0300103InitOutput;
import com.nationrel.modules.project.entity.PmProject;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.bizcommon.constant.BizModuleCode;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_platform.entity.SysStateConfig;
import com.nationrel.modules.project.jar.rb_platform.mapper.SysStateConfigMapper;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.PmPercentStatService;
import com.nationrel.modules.project.service.Rb0300103InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300103InitServiceImpl
 * @Description 项目视图页面初始化的业务实现类
 * @Author andy
 * @CreateDate 2014-03-03
 */
@Service
public class Rb0300103InitServiceImpl implements Rb0300103InitService {

    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private SysStateConfigMapper sysStateConfigMapper;
    @Autowired
    private PmPercentStatService pmPercentStatService;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0300103InitInput input, Rb0300103InitOutput output) throws Exception {

        // 取得页面项目ID
        String projectId = input.getProjectId();
        if (projectId == null || Constants.BLANK.equals(projectId)) {
            projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03001_PROJECT_ID, projectId);
        }

        // 获取数据
        PmProject pmProject = pmProjectMapper.selectByPrimaryKey(projectId);
        List<PmTg> tgList = pmTgMapper.selectPmTgByProjectId(projectId);

        // 进度统计
        Map<String, Object> argPercentMap = new HashMap<String, Object>();
        List<Map<String, Object>> tgPercents = new ArrayList<Map<String, Object>>();
        Map<String, Object> tgMap;
        String activeTgId = null;
        boolean hasWorkTg = false;
        for (PmTg tgTmp : tgList) {
            // 统计项目完成度参数作成
            tgMap = new HashMap<String, Object>();
            tgMap.put(PmMapKeySet.KEY_TG_ID, tgTmp.getTgId());
            tgMap.put(PmMapKeySet.KEY_R_PERCENT, tgTmp.getPercent());

            // 首次创建TG
            if (activeTgId == null && ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK.compareTo(tgTmp.getLifeState()) > 0) {
                activeTgId = tgTmp.getTgId();
            }

            // 工作中TG完成度统计
            if (ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK.equals(tgTmp.getLifeState())
                    || ReliaFlowConstants.LIFECYCLE_RELIA_TG_WAIT_COMPLETE.equals(tgTmp.getLifeState())) {
                hasWorkTg = true;
                argPercentMap.put(PmMapKeySet.KEY_S_TG_ID, tgTmp.getTgId());
                argPercentMap.put(PmMapKeySet.KEY_S_START_NODE_ID, ReliaFlowConstants.TASK_FIRST_PARENT_ID);
                BigDecimal percent = pmPercentStatService.statCurrentNodePercent(argPercentMap);
                tgTmp.setPercent(percent);
                tgMap.put(PmMapKeySet.KEY_R_PERCENT, percent);
            }
            tgPercents.add(tgMap);
        }

        // TG视图数据编辑
        List<List<PmTg>> finalResult = new ArrayList<List<PmTg>>();
        formatTgList(tgList, finalResult);

        // 生命周期状态名称键
        SysStateConfig sysStateConfig = new SysStateConfig();
        sysStateConfig.setModuleId(BizModuleCode.MODULE_ID_PROJECT);
        sysStateConfig.setStateCode(pmProject.getLifeState());
        sysStateConfig = sysStateConfigMapper.selectByModuleIdAndStateCode(sysStateConfig);

        // 结果页面设定
        if (hasWorkTg) {
            activeTgId = null;
        }
        output.setActiveTgId(activeTgId);
        output.setProjectId(pmProject.getProjectId());
        output.setProjectName(pmProject.getProjectName());
        output.setLifeState(pmProject.getLifeState());
        output.setPjlifeStateName(sysStateConfig.getStateName());
        output.setPercent(pmPercentStatService.statProjectPercent(pmProject.getLifeState(), tgPercents));
        output.setPmTgs(finalResult);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param tgList
     * @param finalResult void
     * @Title: formatTgList
     * @Description: 编辑TG列表-每行显示5条(奇数行从左至右/偶数行反之)
     */
    private void formatTgList(List<PmTg> tgList, List<List<PmTg>> finalResult) {
        // 页面TG显示行数
        int lineNumber = tgList.size() % 5 == 0 ? tgList.size() / 5 : tgList.size() / 5 + 1;
        int lineBorder = 0;
        for (int k = 1; k <= lineNumber; k++) {
            if (k % 2 != 0) {
                // 奇数行
                if (k * 5 > tgList.size()) {
                    lineBorder = tgList.size();
                } else {
                    lineBorder = k * 5;
                }
                List<PmTg> result = new ArrayList<PmTg>();
                for (int m = (k - 1) * 5; m < lineBorder; m++) {
                    result.add(tgList.get(m));
                }
                finalResult.add(result);
            } else {
                // 偶数行
                if (k * 5 - 1 >= tgList.size()) {
                    lineBorder = tgList.size() - 1;
                } else {
                    lineBorder = k * 5 - 1;
                }
                List<PmTg> result = new ArrayList<PmTg>();
                for (int m = lineBorder; m >= (k - 1) * 5; m--) {
                    result.add(tgList.get(m));
                }
                finalResult.add(result);
            }
        }
    }
}
