package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300105InitInput;
import com.nationrel.modules.project.dto.output.Rb0300105InitOutput;
import com.nationrel.modules.project.entity.TaskPageModel;
import com.nationrel.modules.project.entity.TaskToolModel;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmPercentStatService;
import com.nationrel.modules.project.service.Rb0300105InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300105InitServiceImpl
 * @Description TG审批查看页面初始化业务类
 * @Author andy
 * @CreateDate 2014-05-14
 */
@Service
public class Rb0300105InitServiceImpl implements Rb0300105InitService {

    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmPercentStatService pmPercentStatService;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG编辑初始化业务方法
     */
    @Override
    public String perform(Rb0300105InitInput input, Rb0300105InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //获取TgId并设置到session中
        String tgId = input.getTgId();
        if (tgId == null || tgId.equals(Constants.BLANK)) {
            tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }
        // 获取TG信息
        Map<String, Object> tgMap = pmTgMapper.selectFullTGInformation(tgId);

        // 取得项目标识
        String projectId = (String) tgMap.get(PmMapKeySet.KEY_PROJECT_ID);
        SessionUtil.setAttribute(SessionKeys.RB03001_PROJECT_ID, projectId);

        // 任务列表数据取得
        Map<String, Object> argMap = new HashMap<String, Object>();
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_OWNER_MARK, Constants.C_STAT_1);
        List<TaskPageModel> tasks = pmTaskMapper.searchPmTaskList(argMap);

        // Task完成度统计
        Map<String, Object> taskPercentParmMap = new HashMap<String, Object>();
        taskPercentParmMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        taskPercentParmMap.put(PmMapKeySet.KEY_S_START_NODE_ID, ReliaFlowConstants.TASK_FIRST_PARENT_ID);
        Map<String, BigDecimal> percentMap = pmPercentStatService.statAllNodePercent(taskPercentParmMap);

        // 页面TG下级权限控制
        Map<String, String> paramMap = new HashMap<String, String>();
        paramMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        paramMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        paramMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        paramMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
        paramMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
        paramMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);

        // 是否所有任务都在创建状态
        int isAllCreate = 1;

        // 统计显示操作按钮的数量
        int nodePadding = 19;

        // task列表对象查看权限
        Map<String, String> argTaskParamMap = new HashMap<String, String>();
        argTaskParamMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argTaskParamMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argTaskParamMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        Map<String, Map<String, String>> authResourceMap = authorityCtrlService.doViewAuthorityByGroup(argTaskParamMap);
        if (tasks != null && tasks.size() > 0) {

            Map<String, String> resultTaskParam;
            String[] toolLinks = null;
            String[] toolNames = null;
            for (TaskPageModel task : tasks) {

                toolLinks = null;
                toolNames = null;
                nodePadding = 19;

                // 计算树形结点间距
                nodePadding = nodePadding * (task.getDeep() - 1);

                // 通过判断是否叶子结点-设定行样式
                if (ReliaFlowConstants.TREE_LEAF.equals(task.getType())) {
                    task.setTrTreeClass(ReliaFlowConstants.CONST_TR_TREE_CLASS_LEAF);
                    task.setTdTreeClass(ReliaFlowConstants.CONST_TD_TREE_CONTEXT.replace("REPLACE_PADDING", String.valueOf(nodePadding))
                            .replace("REPLACE_OPERATE", "")
                            .replace("REPLACE_ONCLICK", ""));
                } else {
                    task.setTrTreeClass(ReliaFlowConstants.CONST_TR_TREE_CLASS_BRANCH);
                    task.setTdTreeClass(ReliaFlowConstants.CONST_TD_TREE_CONTEXT.replace("REPLACE_PADDING", String.valueOf(nodePadding))
                            .replace("REPLACE_OPERATE", ReliaFlowConstants.CONST_TD_OPERATE)
                            .replace("PEPLACE_TITLE_OPERATE", input.getFlexNodeTitle())
                            .replace("REPLACE_ONCLICK", ReliaFlowConstants.CONST_FLEX_BRANCH));
                }

                // 是否所有任务都在创建状态
                if (isAllCreate == 1 && ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.compareTo(task.getLifeState()) <= 0) {
                    isAllCreate = 0;
                }

                // 权限数据取得
                resultTaskParam = authResourceMap.get(task.getTaskId() + ReliaFlowConstants.RESOURCE_TASK_TYPE);

                // 根据taskId获取工具集
                if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.VIEW_AUTHORITY_TOOL))) {
                    List<TaskToolModel> taskToolList = new ArrayList<TaskToolModel>();
                    if (task.getToolLink() != null) {
                        toolNames = task.getToolName().split(",");
                        toolLinks = task.getToolLink().split(",");
                    }
                    if (toolLinks != null) {
                        for (int t = 0; t < toolLinks.length; t++) {
                            TaskToolModel taskToolModel = new TaskToolModel();
                            taskToolModel.setToolName(toolNames[t]);
                            /*
                             * 2014/05/15 注掉(当前需求下不允许点击工具链接)
                            taskToolModel.setToolLink(toolLinks[t]);
                            */
                            taskToolList.add(taskToolModel);
                        }
                    }
                    /*
                     * 2014/05/15 注掉(当前需求下不允许在TG审批页面添加工具)
                    if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.CREATE_AUTHORITY_TOOL))) {
                        TaskToolModel taskToolModel = new TaskToolModel();
                        taskToolModel.setToolName(ReliaFlowConstants.TASK_TOOL_CREATE_I18N);
                        taskToolModel.setToolLink(ReliaFlowConstants.TASK_TOOL_CREATE_URI);
                        taskToolList.add(taskToolModel);
                    }
                    */
                    task.setTaskToolList(taskToolList);
                }

                // 工作中Task完成度统计
                if (ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.equals(task.getLifeState())
                        || ReliaFlowConstants.LIFECYCLE_RELIA_TASK_WAIT_COMPLETE.equals(task.getLifeState())) {
                    if (percentMap != null && percentMap.get(task.getTaskId()) != null) {
                        BigDecimal percent = new BigDecimal(StringUtil.parseString(percentMap.get(task.getTaskId())));
                        BigDecimal zero = new BigDecimal(0);
                        if (!percent.equals(zero)) {
                            task.setPercent(percent);
                        }
                    }
                }
            }
        }

        // 设置输出
        output.setTgMap(tgMap);
        output.setTasks(tasks);
        output.setIsAllCreate(isAllCreate);
        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
