package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300106InitInput;
import com.nationrel.modules.project.dto.output.Rb0300106InitOutput;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.entity.TaskPageModel;
import com.nationrel.modules.project.entity.TaskToolModel;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.PmPercentStatService;
import com.nationrel.modules.project.service.Rb0300106InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @version V1.0
 * @ClassName Rb0300106InitServiceImpl
 * @Description 任务审批查看初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-3-7
 */
@Service
public class Rb0300106InitServiceImpl implements Rb0300106InitService {

    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmPercentStatService pmPercentStatService;

    /**
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务审批查看初始化的业务方法
     */
    @Override
    public String perform(Rb0300106InitInput input, Rb0300106InitOutput output) throws Exception {

        // 取得任务标识
        String taskId = input.getTaskId();
        if (taskId == null || taskId.equals(Constants.BLANK)) {
            taskId = (String) SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03003_TASK_ID, taskId);
        }
        PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
        String tgId = pmTaskTree.getTgId();
        SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);

        // 取得项目标识
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);
        String projectId = pmTg.getProjectId();
        SessionUtil.setAttribute(SessionKeys.RB03001_PROJECT_ID, projectId);

        // 任务审批查看数据取得
        Map<String, Object> argMap = new HashMap<String, Object>();
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_OWNER_MARK, Constants.C_STAT_1);
        argMap.put(PmMapKeySet.KEY_TREE_NO, pmTaskTree.getOrdinal());
        argMap.put(PmMapKeySet.KEY_LEFT_NO, pmTaskTree.getLeftNo());
        argMap.put(PmMapKeySet.KEY_RIGHT_NO, pmTaskTree.getRightNo());
        List<TaskPageModel> tasks = pmTaskMapper.searchPmTaskList(argMap);

        // Task完成度统计
        Map<String, Object> taskPercentParmMap = new HashMap<String, Object>();
        taskPercentParmMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        taskPercentParmMap.put(PmMapKeySet.KEY_S_START_NODE_ID, taskId);
        Map<String, BigDecimal> percentMap = pmPercentStatService.statAllNodePercent(taskPercentParmMap);

        if (tasks != null && tasks.size() > 0) {

            List<TaskToolModel> taskToolList = null;
            String[] toolLinks = null;
            String[] toolNames = null;
            int isAllCreate = 1;              // 是否所有任务都在创建状态
            int nodePadding = 19;
            for (TaskPageModel task : tasks) {

                // 计算树形结点间距
                nodePadding = 19;
                nodePadding = nodePadding * (task.getDeep() - 1);

                // 通过判断是否叶子结点-设定行样式
                if (ReliaFlowConstants.TREE_LEAF.equals(task.getType())) {
                    task.setTrTreeClass(ReliaFlowConstants.CONST_TR_TREE_CLASS_LEAF);
                    task.setTdTreeClass(ReliaFlowConstants.CONST_TD_TREE_CONTEXT.replace("REPLACE_PADDING", String.valueOf(nodePadding))
                            .replace("REPLACE_OPERATE", "")
                            .replace("REPLACE_ONCLICK", ""));
                } else {
                    task.setTrTreeClass(ReliaFlowConstants.CONST_TR_TREE_CLASS_BRANCH);
                    task.setTdTreeClass(ReliaFlowConstants.CONST_TD_TREE_CONTEXT.replace("REPLACE_PADDING", String.valueOf(nodePadding))
                            .replace("REPLACE_OPERATE", ReliaFlowConstants.CONST_TD_OPERATE)
                            .replace("PEPLACE_TITLE_OPERATE", input.getFlexNodeTitle())
                            .replace("REPLACE_ONCLICK", ReliaFlowConstants.CONST_FLEX_BRANCH));
                }

                // 是否所有任务都在创建状态
                if (isAllCreate == 1 && ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.compareTo(task.getLifeState()) <= 0) {
                    isAllCreate = 0;
                }

                // 根据taskId获取工具集
                toolNames = null;
                toolLinks = null;
                taskToolList = new ArrayList<TaskToolModel>();
                if (task.getToolLink() != null) {
                    toolNames = task.getToolName().split(",");
                    toolLinks = task.getToolLink().split(",");
                }
                if (toolLinks != null) {
                    for (int t = 0; t < toolLinks.length; t++) {
                        TaskToolModel taskToolModel = new TaskToolModel();
                        taskToolModel.setToolName(toolNames[t]);
                        // taskToolModel.setToolLink(toolLinks[t]);
                        taskToolList.add(taskToolModel);
                    }
                }
                task.setTaskToolList(taskToolList);

                // 工作中Task完成度统计
                if (ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.equals(task.getLifeState())
                        || ReliaFlowConstants.LIFECYCLE_RELIA_TASK_WAIT_COMPLETE.equals(task.getLifeState())) {
                    if (percentMap != null && percentMap.get(task.getTaskId()) != null) {
                        BigDecimal percent = new BigDecimal(StringUtil.parseString(percentMap.get(task.getTaskId())));
                        BigDecimal zero = new BigDecimal(0);
                        if (!percent.equals(zero)) {
                            task.setPercent(percent);
                        }
                    }
                }
            }
            // 任务创建状态
            output.setIsAllCreate(isAllCreate);
        }

        // 任务审批查看页面设定
        output.setTasks(tasks);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
