package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300110InitInput;
import com.nationrel.modules.project.dto.output.Rb0300110InitOutput;
import com.nationrel.modules.project.jar.rb_common.bizcommon.constant.BizModuleCode;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.Rb0300110InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300210InitServiceImpl
 * @Description 项目基本信息初始化的业务类
 * @Author andy
 * @CreateDate 2013-12-16
 */
@Service
public class Rb0300110InitServiceImpl implements Rb0300110InitService {

    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;

    /**
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目基本信息初始化的业务方法
     */
    @Override
    public String perform(Rb0300110InitInput input, Rb0300110InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 根据Id获取项目对象数据
        Map<String, Object> result = pmProjectMapper.getProjectBasicInfo(projectId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.BASE_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 设置输出
        output.setAuthMap(authMap);
        output.setBusinessId(projectId);
        output.setProjectMap(result);
        output.setModuleId(BizModuleCode.MODULE_ID_PROJECT);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
