package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300111SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300111SaveOutput;
import com.nationrel.modules.project.entity.PmProject;
import com.nationrel.modules.project.entity.PmProjectStaff;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.service.Rb0300111SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @version V1.0
 * @ClassName Rb0300111SaveServiceImpl
 * @Description 项目对象创建业务实现类
 * @Author andy
 * @CreateDate 2013-12-10
 */
@Service
public class Rb0300111SaveServiceImpl implements Rb0300111SaveService {

    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;

    /**
     * @param input    输入参数
     * @param output   输出参数
     * @return String 返回结果
     * @Title perform
     * @Description 项目对象创建业务方法
     */
    @Override
    public String perform(Rb0300111SaveInput input, Rb0300111SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 设置项目对象的基本信息
        PmProject pmProject = new PmProject();
        pmProject.setProjectCode(input.getProjectCode());
        pmProject.setProjectName(input.getProjectName());
        if (input.getProjectType() != null && !input.getProjectType().equals(Constants.BLANK)) {
            pmProject.setProjectType(input.getProjectType());
        }
        if (input.getProjectApp() != null && !input.getProjectApp().equals(Constants.BLANK)) {
            pmProject.setProjectApp(input.getProjectApp());
        }
        pmProject.setPlanEndTime(DateUtils.convertDate(input.getPlanEndTime()));
        pmProject.setPlanStartTime(DateUtils.convertDate(input.getPlanStartTime()));
        pmProject.setDescription(input.getDescription());

        // templateId 暂时设置为null
        pmProject.setTemplateId(null);
        pmProject.setDelFlag(Constants.C_STAT_0);
        pmProject.setObjectName(input.getObjectName());
        pmProject.setLifeState(ReliaFlowConstants.LIFECYCLE_RELIA_PJ_CREATE);

        // 设置系统信息到项目对象
        pmProject.setInsertUserId(sysUser.getUsername());
        pmProject.setInsertPgmId(sysUser.getPgmId());

        // 保存项目对象到数据库
        pmProjectMapper.insert(pmProject);

        // 设置项目人员信息并保存到数据库
        PmProjectStaff pmProjectStaff = new PmProjectStaff();
        pmProjectStaff.setProjectId(pmProject.getProjectId());
        pmProjectStaff.setStaffId(sysUser.getUsername());
        pmProjectStaff.setProjectRoleId(ReliaFlowConstants.PROJECT_OWNER_ID);
        pmProjectStaff.setStartTime(new Date(System.currentTimeMillis()));
        pmProjectStaff.setIsApply(ReliaFlowConstants.IS_APPLY_YES);
        pmProjectStaff.setInsertUserId(sysUser.getUsername());
        pmProjectStaff.setInsertPgmId(sysUser.getPgmId());
        pmProjectStaffMapper.insert(pmProjectStaff);

        // 设置输出
        output.setProjectId(pmProject.getProjectId());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
