package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300130InitInput;
import com.nationrel.modules.project.dto.output.Rb0300130InitOutput;
import com.nationrel.modules.project.entity.PmProjectCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectCheckMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.Rb0300130InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300130InitServiceImpl
 * @Description 项目检查列表初始化业务类
 * @Author andy
 * @CreateDate 2014-03-06
 */
@Service
public class Rb0300130InitServiceImpl implements Rb0300130InitService {

    @Autowired
    private PmProjectCheckMapper pmProjectCheckMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目检查列表初始化的业务方法
     */
    @Override
    public String perform(Rb0300130InitInput input, Rb0300130InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 根据项目标识取出检查列表
        List<PmProjectCheck> pmChecks = pmProjectCheckMapper.findCheckListByProjectId(projectId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHECK_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_MODIFY))) {
            ctrOptCount++;
        }
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }

        // 检查列表页面设定
        output.setPmChecks(pmChecks);
        output.setAuthMap(authMap);
        output.setCtrOptCount(ctrOptCount);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
