package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300130SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300130SaveOutput;
import com.nationrel.modules.project.entity.PmProjectCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectCheckMapper;
import com.nationrel.modules.project.service.Rb0300130SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @version V1.0
 * @ClassName Rb0300130SaveServiceImpl
 * @Description 项目检查列表保存业务类
 * @Author andy
 * @CreateDate 2013-12-19
 */
@Service
public class Rb0300130SaveServiceImpl implements Rb0300130SaveService {

    @Autowired
    private PmProjectCheckMapper pmProjectCheckMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目检查列表保存业务方法
     */
    @Override
    public String perform(Rb0300130SaveInput input, Rb0300130SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 项目标识取得
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 取得页面项目检查列表
        List<PmProjectCheck> pmProjectChecks = input.getPmProjectCheck();

        // 项目检查列表保存
        PmProjectCheck updateEntity = null;
        PmProjectCheck entity = null;
        for (int i = 0; i < pmProjectChecks.size(); i++) {
            // 取得页面检查项实体
            entity = pmProjectChecks.get(i);
            // 根据主键从DB取得检查项目
            updateEntity = pmProjectCheckMapper.selectByPrimaryKey(entity.getProjectCheckId(), projectId);

            // 系统信息
            updateEntity.setInsertUserId(sysUser.getUsername());
            updateEntity.setInsertPgmId(sysUser.getPgmId());

            // 页面与DB检查结果一致
            if (ReliaFlowConstants.CHECK_RESULT_PASS.equals(entity.getResult())) {
                if (ReliaFlowConstants.CHECK_RESULT_PASS.equals(updateEntity.getResult())) {
                    continue;
                } else {
                    updateEntity.setResult(ReliaFlowConstants.CHECK_RESULT_PASS);
                    // 检查项保存数据库
                    pmProjectCheckMapper.updateByPrimaryKey(updateEntity);
                }
            } else {
                if (ReliaFlowConstants.CHECK_RESULT_INIT.equals(updateEntity.getResult())) {
                    continue;
                } else {
                    updateEntity.setResult(ReliaFlowConstants.CHECK_RESULT_INIT);
                    // 检查项保存数据库
                    pmProjectCheckMapper.updateByPrimaryKey(updateEntity);
                }
            }
        }

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
