package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300140InitInput;
import com.nationrel.modules.project.dto.output.Rb0300140InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectRoleMapper;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.Rb0300140InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @version V1.0
 * @ClassName Rb0300140InitServiceImpl
 * @Description 项目成员列表页面初始化的业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300140InitServiceImpl implements Rb0300140InitService {

    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmProjectRoleMapper pmProjectRoleMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG列表初始化的业务方法
     */
    @Override
    public String perform(Rb0300140InitInput input, Rb0300140InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 取得项目成员列表
        List<Map<String, Object>> pmProjectStaffs = pmProjectStaffMapper.searchProjectMemberListByProjectId(projectId);

        // 控制删除是否可见,符合当前login userId==staffId && 角色是项目经理的成员不可以删除.
        int disableOperateCnt = 0;
        if (pmProjectStaffs != null && pmProjectStaffs.size() > 0) {
            for (Map<String, Object> pmProjectStaffMap : pmProjectStaffs) {
                String staffId = StringUtil.parseString(pmProjectStaffMap.get(PmMapKeySet.KEY_R_STAFF_ID));
                String roleId = StringUtil.parseString(pmProjectStaffMap.get(PmMapKeySet.KEY_R_STAFF_ROLEID));
                if ((staffId.equals(sysUser.getUsername()) && roleId.equals(ReliaFlowConstants.PROJECT_MANAGER_ID)) ||
                        roleId.equals(ReliaFlowConstants.PROJECT_OWNER_ID)) {
                    pmProjectStaffMap.put(ReliaFlowConstants.AUTHORITY_MODIFY, ReliaFlowConstants.AUTH_CTRL_VIEW);
                    pmProjectStaffMap.put(ReliaFlowConstants.AUTHORITY_DELETE, ReliaFlowConstants.AUTH_CTRL_VIEW);
                    disableOperateCnt++;
                }
            }
        }

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.MEMBER_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;

        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_MODIFY))) {
            ctrOptCount++;
        }
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }
        if (disableOperateCnt == pmProjectStaffs.size() && ctrOptCount > 0) {
            ctrOptCount = 0;
        }

        // 统计显示操作按钮的数量
        output.setCtrOptCount(ctrOptCount);
        output.setAuthMap(authMap);
        output.setPmProjectStaffs(pmProjectStaffs);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
