package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300141AddMemberInput;
import com.nationrel.modules.project.dto.output.Rb0300141AddMemberOutput;
import com.nationrel.modules.project.entity.PmProjectStaff;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectRoleMapper;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.service.Rb0300141AddMemberService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @version V1.0
 * @ClassName Rb0300141AddMemberServiceImpl
 * @Description 项目成员选择页面添加成员业务接口
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300141AddMemberServiceImpl implements Rb0300141AddMemberService {

   /* @Autowired
    private SysUserGroupMapper sysUserGroupMapper;
    @Autowired
    private SysUserMapper sysUserMapper;*/
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmProjectRoleMapper pmProjectRoleMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目成员选择页面添加成员的业务方法
     */
    @Override
    public String perform(Rb0300141AddMemberInput input, Rb0300141AddMemberOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取参数
        String[] groupIds = input.getGroupIds();
        String[] membersIds = input.getMemberIds();

        List<Map<String, Object>> result = null;

        // 获取session中的projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        if (groupIds != null && groupIds.length > 0) {

            // 获取项目成员集合
            result = bulidProjectMemberByGroup(groupIds, projectId);
        } else if (membersIds != null && membersIds.length > 0) {

            // 根据用户Id集合构建新的项目成员集合
            result = bulidProjectMemberByUserIds(membersIds, projectId);
        } else {

            // 返回失败
            return Constants.SERVICE_FAIL;
        }

        // 添加项目成员到项目成员表
        if (result != null && result.size() > 0) {
            for (Map<String, Object> obj : result) {
                PmProjectStaff staff = new PmProjectStaff();
                staff.setStaffId(StringUtil.parseString(obj.get(PmMapKeySet.KEY_R_USER_ID)));
                staff.setProjectId(projectId);
                staff.setProjectRoleId(StringUtil.parseString(obj.get(PmMapKeySet.KEY_R_PROJECT_ROLE_ID)));
                staff.setInsertUserId(sysUser.getUsername());
                staff.setInsertPgmId(sysUser.getPgmId());
                staff.setIsApply(ReliaFlowConstants.IS_APPLY_YES);
                staff.setStartTime(new Date(System.currentTimeMillis()));
                pmProjectStaffMapper.insert(staff);
            }
        }

        // 返回成功
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param membersIds:用户Id集合
     * @param projectId:项目Id
     * @return List<Map < String, Object>>
     * @Title: bulidProjectMemberByUserIds
     * @Description: 根据用户Id集合构建新的项目成员集合
     */
    private List<Map<String, Object>> bulidProjectMemberByUserIds(
            String[] membersIds, String projectId) {
        List<Map<String, Object>> result;
        result = new ArrayList<Map<String, Object>>();
        Set<String> userIdSet = new HashSet<String>();

        // 根据projectId获取项目成员集合
        List<Map<String, Object>> oldMemberList = pmProjectStaffMapper.searchProjectMemberListByProjectId(projectId);

        // 将选择的userId添加到集合中
        Collections.addAll(userIdSet, membersIds);

        // 去除成员集合中的重复数据
        for (Map<String, Object> obj : oldMemberList) {
            String userIdFromDB = StringUtil.parseString(obj.get(PmMapKeySet.KEY_R_STAFF_ID));
            boolean exist = userIdSet.contains(userIdFromDB);
            if (exist) {
                userIdSet.remove(userIdFromDB);
            }
        }

        // 构造新的项目成员列表数据
        /*for (Iterator<String> it = userIdSet.iterator(); it.hasNext(); ) {
            SysUser newUser = sysUserMapper.selectByPrimaryKey(it.next());
            Map<String, Object> userMap = new HashMap<String, Object>();
            userMap.put(PmMapKeySet.KEY_R_USER_ID, newUser.getUserId());
            userMap.put(PmMapKeySet.KEY_R_USER_NAME, newUser.getUsername());
            userMap.put(PmMapKeySet.KEY_R_EMAIL, newUser.getEmail());
            userMap.put(PmMapKeySet.KEY_R_TELEPHONE, newUser.getTelephone());
            PmProjectStaff staff = pmProjectStaffMapper.getStaffByPrimaryKey(projectId, newUser.getUserId());
            if (staff != null) {
                userMap.put(PmMapKeySet.KEY_R_PROJECT_ROLE_ID, staff.getProjectRoleId());
            } else {
                userMap.put(PmMapKeySet.KEY_R_PROJECT_ROLE_ID, ReliaFlowConstants.PROJECT_ROLE_PROJECT_MEMBER_ID);
            }
            result.add(userMap);
        }*/
        return result;
    }

    /**
     * @param groupIds:组Id集合
     * @param projectId:项目Id
     * @return List<Map < String, Object>>
     * @Title: bulidProjectMemberByGroup
     * @Description: 根据组Id集合构建新的项目成员集合
     */
    private List<Map<String, Object>> bulidProjectMemberByGroup(
            String[] groupIds, String projectId) {
        List<Map<String, Object>> result;
        result = new ArrayList<Map<String, Object>>();

        // 根据projectId获取项目已有成员集合
        List<Map<String, Object>> oldMemberList = pmProjectStaffMapper.searchProjectMemberListByProjectId(projectId);
        Set<String> userIdSet = new HashSet<String>();
        /*for (String groupId : groupIds) {

            // 根据组Id获取组内所有成员
            List<SysUser> sysUsers = sysUserMapper.selectByGroupId(groupId);
            for (SysUser currentUser : sysUsers) {
                userIdSet.add(currentUser.getUserId());
            }
        }*/

        // 去除成员集合中的重复数据
        for (Map<String, Object> obj : oldMemberList) {
            String userIdFromDB = StringUtil.parseString(obj.get(PmMapKeySet.KEY_R_STAFF_ID));
            boolean exist = userIdSet.contains(userIdFromDB);
            if (exist) {
                userIdSet.remove(userIdFromDB);
            }
        }

        // 构造新的项目成员列表数据
        /*for (Iterator<String> it = userIdSet.iterator(); it.hasNext(); ) {
            SysUser newUser = sysUserMapper.selectByPrimaryKey(it.next());
            Map<String, Object> userMap = new HashMap<String, Object>();
            userMap.put(PmMapKeySet.KEY_R_USER_ID, newUser.getUserId());
            userMap.put(PmMapKeySet.KEY_R_USER_NAME, newUser.getUsername());
            userMap.put(PmMapKeySet.KEY_R_EMAIL, newUser.getEmail());
            userMap.put(PmMapKeySet.KEY_R_TELEPHONE, newUser.getTelephone());
            PmProjectStaff staff = pmProjectStaffMapper.getStaffByPrimaryKey(projectId, newUser.getUserId());
            if (staff != null) {
                userMap.put(PmMapKeySet.KEY_R_PROJECT_ROLE_ID, staff.getProjectRoleId());
            } else {
                userMap.put(PmMapKeySet.KEY_R_PROJECT_ROLE_ID, ReliaFlowConstants.PROJECT_ROLE_PROJECT_MEMBER_ID);
            }
            result.add(userMap);
        }*/
        return result;
    }

}
