package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300142InitInput;
import com.nationrel.modules.project.dto.output.Rb0300143InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectRoleMapper;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300142InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300142InitServiceImpl
 * @Description 成员编辑初始化业务类
 * @Author andy
 * @CreateDate 2014-03-07
 */
@Service
public class Rb0300142InitServiceImpl implements Rb0300142InitService {

    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmProjectRoleMapper pmProjectRoleMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 成员编辑初始化业务方法
     */
    @Override
    public String perform(Rb0300142InitInput input, Rb0300143InitOutput output) throws Exception {

        // 获取参数
        String staffId = input.getStaffId();
        Integer selectStaffType = input.getSelectType();

        //===========================
        // 根据成员的选择方式分别处理Project\TG\TASK成员编辑初始化
        //===========================
        switch (selectStaffType) {
            case 1:
                // 获取当前项目成员信息
                String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
                List<Map<String, Object>> pmProjectStaffs = pmProjectStaffMapper.selectPmProjectStaffsByProjectId(projectId, staffId);
                output.setStaffMap(pmProjectStaffs.get(0));

                // 成员角色取得并设置到输出
                output.setRoles(pmProjectRoleMapper.selectRoleByResourceType(ReliaFlowConstants.RESOURCE_PROJECT_TYPE));
                break;
            case 2:
                String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 检查TG是否存在
                pmTgService.checkTgExist(tgId);

                // 获取当前TG成员信息
                List<Map<String, Object>> pmTgStaffs = pmTgStaffMapper.selectPmTgStaffsByTgId(tgId, staffId);

                // 检查成员信息是否存在
                if (pmTgStaffs == null || pmTgStaffs.size() <= 0) {
                    throw new OptimisticLockException(MessageId.W009803);
                }

                // 保存version number 到session
                Object versionNumber = pmTgStaffs.get(0).get(PmMapKeySet.KEY_S_PJ_VERSION_NO);
                SessionUtil.setAttribute(SessionKeys.RB_PJ_TG_STAFF_VERSION, versionNumber);

                // 设置成员信息到输出
                output.setStaffMap(pmTgStaffs.get(0));
                output.setRoles(pmProjectRoleMapper.selectRoleByResourceType(ReliaFlowConstants.RESOURCE_TG_TYPE));
                break;
            case 3:
                String tgIdTask = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 检查TG是否存在
                pmTgService.checkTgExist(tgIdTask);

                // 获取当前Task成员信息
                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                // 检查任务是否存在
                pmTaskService.checkTaskExist(taskId);

                List<Map<String, Object>> pmTaskStaffs = pmTaskStaffMapper.selectPmTaskStaffsByTaskId(taskId, staffId);
                output.setStaffMap(pmTaskStaffs.get(0));

                // 保存version number 到session
                Object versionNumberTask = pmTaskStaffs.get(0).get(PmMapKeySet.KEY_S_PJ_VERSION_NO);
                SessionUtil.setAttribute(SessionKeys.RB_PJ_TASK_MEMBER_EDIT_VERSION, versionNumberTask);

                // 成员角色取得并设置到输出
                output.setRoles(pmProjectRoleMapper.selectRoleByResourceType(ReliaFlowConstants.RESOURCE_TASK_TYPE));
                break;
            default:
                // 返回失败
                return Constants.SERVICE_FAIL;
        }

        // 设置输出
        output.setSelectType(selectStaffType);

        // 返回成功
        return Constants.SERVICE_SUCCESS;
    }

}
