package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300142SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300142SaveOutput;
import com.nationrel.modules.project.entity.PmProjectStaff;
import com.nationrel.modules.project.entity.PmTaskStaff;
import com.nationrel.modules.project.entity.PmTgStaff;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectRoleMapper;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300142SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300243SaveServiceImpl
 * @Description 成员编辑保存的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300142SaveServiceImpl implements Rb0300142SaveService {

    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmProjectRoleMapper pmProjectRoleMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 成员编辑保存业务方法
     */
    @Override
    public String perform(Rb0300142SaveInput input, Rb0300142SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取参数
        Integer selectStaffType = input.getSelectType();

        // 设置返回结果
        String result = Constants.SERVICE_FAIL;

        //===========================
        // 根据成员的选择方式分别对Project\TG\TASK成员编辑保存
        //===========================
        switch (selectStaffType) {
            case 1:

                // 项目成员取得
                String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
                String staffId = input.getStaffId();
                PmProjectStaff pmProjectStaff = pmProjectStaffMapper.selectByPrimaryKey(projectId, staffId);

                // 保存项目成员信息到数据库
                pmProjectStaff.setProjectRoleId(input.getRoleId());
                //pmProjectStaff.setStartTime(input.getStartTime());
                pmProjectStaff.setUpdateUserId(sysUser.getUsername());
                pmProjectStaff.setUpdatePgmId(sysUser.getPgmId());
                pmProjectStaffMapper.updateByPrimaryKey(pmProjectStaff);
                result = ReliaFlowConstants.PROJECT_STAFF_LIST;
                break;
            case 2:

                // TG成员取得
                String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 1.检查TG是否存在
                pmTgService.checkTgExist(tgId);

                // 2.检查pmTgStaff 是否存在
                PmTgStaff pmTgStaff = pmTgStaffMapper.selectByPrimaryKey(tgId, input.getStaffId());
                if (pmTgStaff == null) {
                    throw new OptimisticLockException(MessageId.W009803);
                } else {

                    // 获取session中TG version number
                    Integer versionNumber = Integer.parseInt(SessionUtil.getAttribute(SessionKeys.RB_PJ_TG_STAFF_VERSION).toString());

                    // 3.检查version number 是否一致
                    if (!pmTgStaff.getVersionNo().equals(versionNumber)) {
                        throw new OptimisticLockException(MessageId.W009801);
                    }
                }

                // 保存TG成员信息到数据库
                pmTgStaff.setProjectRoleId(input.getRoleId());
                //pmTgStaff.setStartTime(input.getStartTime());
                pmTgStaff.setUpdateUserId(sysUser.getUsername());
                pmTgStaff.setUpdatePgmId(sysUser.getPgmId());
                pmTgStaffMapper.updateByPrimaryKey(pmTgStaff);
                result = ReliaFlowConstants.TG_STAFF_LIST;
                break;
            case 3:
                // TG成员取得
                String tgIdTask = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 1.检查TG是否存在
                pmTgService.checkTgExist(tgIdTask);

                // task成员取得
                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                // 2.检查任务是否存在
                pmTaskService.checkTaskExist(taskId);

                // 3.任务人员基本信息编辑排它处理
                PmTaskStaff pmTaskStaff = pmTaskStaffMapper.selectByPrimaryKey(taskId, input.getStaffId());
                Integer versionLock = Integer.parseInt(StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_MEMBER_EDIT_VERSION)));
                if (pmTaskStaff == null || !versionLock.equals(pmTaskStaff.getVersionNo())) {
                    throw new OptimisticLockException(MessageId.W009801);
                }

                // 保存task成员信息到数据库
                pmTaskStaff.setProjectRoleId(input.getRoleId());
                pmTaskStaff.setUpdateUserId(sysUser.getUsername());
                pmTaskStaff.setUpdatePgmId(sysUser.getPgmId());
                pmTaskStaffMapper.updateByPrimaryKey(pmTaskStaff);
                result = ReliaFlowConstants.TASK_STAFF_LIST;
                break;
            default:
                break;
        }

        // 返回成功
        return result;
    }

    /**
     * @param role
     * @param userId
     * @return MessageId
     * @Title: checkRoleForReliaFlow
     * @Description: 项目总监&&项目经理唯一性检查
     */
    @Override
    public MessageId checkRoleForProject(Integer type, String roleId, String userId) {
        MessageId messageId = null;
        int count = 0;
        switch (type) {
            case 1:

                // 项目角色检查
                String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
                count = pmProjectStaffMapper.countProjectRole(userId, projectId, roleId, ReliaFlowConstants.ROLE_PROJECT_MANAGER);
                if (count > 0) {
                    messageId = MessageId.E300106;
                }
                break;
            case 2:

                // TG角色检查
                String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
                count = pmTgStaffMapper.countTgRole(userId, tgId, roleId, ReliaFlowConstants.ROLE_TG_MANAGER);
                if (count > 0) {
                    messageId = MessageId.E300106;
                }
                break;
            case 3:

                // Task角色检查
                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
                count = pmTaskStaffMapper.countTaskRole(userId, taskId, roleId, ReliaFlowConstants.ROLE_TASK_MANAGER);
                if (count > 0) {
                    messageId = MessageId.E300106;
                }
                break;
            default:
                break;
        }
        return messageId;
    }
}
