package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300143DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300143DeleteOutput;
import com.nationrel.modules.project.entity.PmTaskStaff;
import com.nationrel.modules.project.entity.PmTgStaff;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300143DeleteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;


/**
 * @version V1.0
 * @ClassName Rb0300143DeleteServiceImpl
 * @Description 成员删除的业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300143DeleteServiceImpl implements Rb0300143DeleteService {

    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;
  /*  @Autowired
    private SysUserMapper sysUserMapper;*/

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 成员删除业务方法
     */
    @Override
    public String perform(Rb0300143DeleteInput input, Rb0300143DeleteOutput output) throws Exception {

        // 获取参数
        Integer selectStaffType = input.getSelectType();
        String result = Constants.SERVICE_FAIL;
        String staffId = input.getStaffId();
        //===========================
        // 根据成员的选择方式分别删除PROEJCT\TG\TASK成员
        //===========================
        switch (selectStaffType) {
            case 1:
                // 项目成员删除
                String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
                List<Map<String, String>> tgReferenceStaff = pmTgStaffMapper.searchTgReferenceStaff(projectId, staffId);
                if (tgReferenceStaff.size() > 0) {

                    /*SysUser sysUser = sysUserMapper.selectByPrimaryKey(staffId);
                    if (sysUser != null) {
                        output.setUserName(sysUser.getUsername());
                    }*/
                    output.setStaffReferences(tgReferenceStaff);
                } else {

                    pmProjectStaffMapper.deleteByPrimaryKey(projectId, staffId);
                    result = ReliaFlowConstants.PROJECT_STAFF_LIST;
                }
                break;
            case 2:
                String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 1.检查TG是否存在
                pmTgService.checkTgExist(tgId);

                // 2.检查成员是否存在
                PmTgStaff pmTgStaff = pmTgStaffMapper.selectByPrimaryKey(tgId, staffId);
                if (pmTgStaff == null) {
                    throw new OptimisticLockException(MessageId.W009803);
                }

                // 3.是否被任务引用
                List<Map<String, String>> taskReferenceStaff = pmTaskStaffMapper.searchTaskReferenceStaff(tgId, ReliaFlowConstants.TASK_FIRST_PARENT_ID, staffId);
                if (taskReferenceStaff.size() > 0) {

                    /*SysUser sysUser = sysUserMapper.selectByPrimaryKey(staffId);
                    if (sysUser != null) {
                        output.setUserName(sysUser.getUsername());
                    }*/
                    output.setStaffReferences(taskReferenceStaff);
                } else {

                    pmTgStaffMapper.deleteByPrimaryKey(tgId, staffId);
                    result = ReliaFlowConstants.TG_STAFF_LIST;
                }
                break;
            case 3:
                String tgIdTask = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 1.检查TG是否存在
                pmTgService.checkTgExist(tgIdTask);

                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                // 2.检查任务是否存在
                pmTaskService.checkTaskExist(taskId);

                // 3.任务人员基本信息删除排它处理
                PmTaskStaff pmTaskStaff = pmTaskStaffMapper.selectByPrimaryKey(taskId, input.getStaffId());
                if (pmTaskStaff == null) {
                    throw new OptimisticLockException(MessageId.W009803);
                }

                // 4.是否被子任务引用
                List<Map<String, String>> innerTaskReferenceStaff = pmTaskStaffMapper.searchTaskReferenceStaff(tgIdTask, taskId, staffId);
                if (innerTaskReferenceStaff.size() > 0) {

                    /*SysUser sysUser = sysUserMapper.selectByPrimaryKey(staffId);
                    if (sysUser != null) {
                        output.setUserName(sysUser.getUsername());
                    }*/
                    output.setStaffReferences(innerTaskReferenceStaff);
                } else {

                    // 任务成员删除
                    pmTaskStaffMapper.deleteByPrimaryKey(taskId, staffId);
                    result = ReliaFlowConstants.TASK_STAFF_LIST;
                }
                break;
            default:
                return Constants.SERVICE_FAIL;
        }

        // 返回成功
        return result;
    }

}
