package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300150InitInput;
import com.nationrel.modules.project.dto.output.Rb0300150InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectDeliveryMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.Rb0300150InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300150InitServiceImpl
 * @Description 项目文档列表页面初始化业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300150InitServiceImpl implements Rb0300150InitService {

    @Autowired
    private PmProjectDeliveryMapper pmProjectDeliveryMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;


    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目文档列表页面初始化业务方法
     */
    @Override
    public String perform(Rb0300150InitInput input, Rb0300150InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目ID
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 根据projectId获取文档列表
        List<Map<String, Object>> docList = pmProjectDeliveryMapper.searchDeliveryListByProjectId(projectId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.DOCUMENT_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }

        // 设置输出
        output.setDeliveryList(docList);
        output.setAuthMap(authMap);
        output.setCtrOptCount(ctrOptCount);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
