package com.nationrel.modules.project.service.impl;


import com.nationrel.modules.project.dto.input.Rb0300151AddDeliveryInput;
import com.nationrel.modules.project.dto.output.Rb0300151AddDeliveryOutput;
import com.nationrel.modules.project.entity.PmProjectDelivery;
import com.nationrel.modules.project.entity.PmTaskDelivery;
import com.nationrel.modules.project.entity.PmTgDelivery;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300151AddDeliveryService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300151AddDeliveryServiceImpl
 * @Description 任务交付物添加的业务类
 * @Author aaron
 * @CreateDate 2014-03-17
 */
@Service
public class Rb0300151AddDeliveryServiceImpl implements Rb0300151AddDeliveryService {

    @Autowired
    private PmProjectDocumentMapper pmProjectDocumentMapper;
    @Autowired
    private PmTgDeliveryMapper pmTgDeliveryMapper;
    @Autowired
    private PmTaskToolMapper pmTaskToolMapper;
    @Autowired
    private PmTaskDeliveryMapper pmTaskDeliveryMapper;
    @Autowired
    private PmProjectDeliveryMapper pmProjectDeliveryMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务交付物添加的业务方法
     */
    @Override
    public String perform(Rb0300151AddDeliveryInput input, Rb0300151AddDeliveryOutput output) throws Exception {

        // 获取参数
        String[] deliveryIds = input.getDeliveryIds();
        Integer selectType = input.getSelectType();

        // 业务处理
        if (deliveryIds == null || deliveryIds.length <= 0) {
            // 返回失败
            return Constants.SERVICE_FAIL;
        }
        String result = Constants.SERVICE_FAIL;
        switch (selectType) {
            case 1:
                String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
                addDeliveryForProject(deliveryIds, projectId);
                result = ReliaFlowConstants.PROJECT_DOCUMNET_LIST;
                break;
            case 2:
                String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
                // 检查TG是否存在
                pmTgService.checkTgExist(tgId);
                addDeliveryForTG(deliveryIds, tgId);
                result = ReliaFlowConstants.TG_DOCUMNET_LIST;
                break;
            case 3:
                String tgIdTask = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
                // 1.检查TG是否存在
                pmTgService.checkTgExist(tgIdTask);

                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
                // 2.检查任务是否存在
                pmTaskService.checkTaskExist(taskId);

                addDeliveryForTask(deliveryIds, taskId);
                result = ReliaFlowConstants.TASK_DOCUMNET_LIST;
                break;
            default:
                return Constants.SERVICE_FAIL;
        }
        return result;

    }

    /**

     * @param projectId void
     * @Title: addDeliveryForProject
     * @Description: 添加项目交付
     */
    private void addDeliveryForProject(String[] deliveryIds, String projectId) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取Project已存在的交付列表
        for (int i = 0; i < deliveryIds.length; i++) {
            String deliveryIdAndType = deliveryIds[i];
            String deliveryId = null;
            if (deliveryIdAndType.indexOf(Constants.COMMA) != -1) {
                deliveryId = deliveryIdAndType.substring(0, deliveryIdAndType.indexOf(Constants.COMMA));
            }
            if (StringUtils.isNotEmpty(deliveryIdAndType)) {
                if (deliveryIdAndType.indexOf(Constants.COMMA) != -1) {
                    // 保存Tg交付关系到数据库
                    deliveryId = deliveryIdAndType.substring(0, deliveryIdAndType.indexOf(Constants.COMMA));
                    String deliveryType = deliveryIdAndType.substring(deliveryIdAndType.indexOf(Constants.COMMA) + 1);
                    PmProjectDelivery pmProjectDelivery = new PmProjectDelivery();
                    pmProjectDelivery.setDeliveryId(deliveryId);
                    pmProjectDelivery.setDeliveryType(deliveryType);
                    pmProjectDelivery.setProjectId(projectId);
                    pmProjectDelivery.setInsertUserId(sysUser.getUsername());
                    pmProjectDelivery.setInsertPgmId(sysUser.getPgmId());
                    pmProjectDeliveryMapper.insert(pmProjectDelivery);
                }
            }
        }

    }

    /**

     * @param projectId void
     * @Title: addDeliveryForTG
     * @Description: 添加TG交付
     */
    private void addDeliveryForTG(String[] deliveryIds, String tgId) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        for (int i = 0; i < deliveryIds.length; i++) {
            String deliveryIdAndType = deliveryIds[i];
            String deliveryId = null;
            if (deliveryIdAndType.indexOf(Constants.COMMA) != -1) {
                deliveryId = deliveryIdAndType.substring(0, deliveryIdAndType.indexOf(Constants.COMMA));
            }
            if (StringUtils.isNotEmpty(deliveryIdAndType)) {
                if (deliveryIdAndType.indexOf(Constants.COMMA) != -1) {
                    // 保存Tg交付关系到数据库
                    deliveryId = deliveryIdAndType.substring(0, deliveryIdAndType.indexOf(Constants.COMMA));
                    String deliveryType = deliveryIdAndType.substring(deliveryIdAndType.indexOf(Constants.COMMA) + 1);
                    PmTgDelivery pmTgDelivery = new PmTgDelivery();
                    pmTgDelivery.setDeliveryId(deliveryId);
                    pmTgDelivery.setDeliveryType(deliveryType);
                    pmTgDelivery.setTgId(tgId);
                    pmTgDelivery.setInsertUserId(sysUser.getUsername());
                    pmTgDelivery.setInsertPgmId(sysUser.getPgmId());
                    pmTgDeliveryMapper.insert(pmTgDelivery);
                }
            }
        }
    }

    /**

     * @param projectId void
     * @Title: addDeliveryForTask
     * @Description: 添加Task交付
     */
    private void addDeliveryForTask(String[] deliveryIds, String taskId) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 保存Task交付关系到数据库
        for (int i = 0; i < deliveryIds.length; i++) {
            String deliveryIdAndType = deliveryIds[i];
            String deliveryId = null;
            if (deliveryIdAndType.indexOf(Constants.COMMA) != -1) {
                deliveryId = deliveryIdAndType.substring(0, deliveryIdAndType.indexOf(Constants.COMMA));
            }
            if (StringUtils.isNotEmpty(deliveryIdAndType)) {
                if (deliveryIdAndType.indexOf(Constants.COMMA) != -1) {
                    String deliveryType = deliveryIdAndType.substring(deliveryIdAndType.indexOf(Constants.COMMA) + 1);
                    PmTaskDelivery pmTaskDelivery = new PmTaskDelivery();
                    pmTaskDelivery.setDeliveryId(deliveryId);
                    pmTaskDelivery.setDeliveryType(deliveryType);
                    pmTaskDelivery.setTaskId(taskId);
                    pmTaskDelivery.setInsertUserId(sysUser.getUsername());
                    pmTaskDelivery.setInsertPgmId(sysUser.getPgmId());
                    pmTaskDeliveryMapper.insert(pmTaskDelivery);
                }
            }
        }
    }
}
