// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300151InitInput;
import com.nationrel.modules.project.dto.output.Rb0300151InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskDeliveryMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300151InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300151InitServiceImpl
 * @Description 交付列表页面初始化业务类
 * @Author aaron
 * @CreateDate 2014-03-17
 */
@Service
public class Rb0300151InitServiceImpl implements Rb0300151InitService {

    @Autowired
    private PmTaskDeliveryMapper pmTaskDeliveryMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**
      用户信息
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @throws Exception 异常处理
     * @Title: perform
     * @Description 交付列表页面初始化业务方法
     */
    @Override
    public String perform(Rb0300151InitInput input, Rb0300151InitOutput output) throws Exception {

        Integer selectType = input.getSelectType();
        if (selectType == null) {
            return Constants.SERVICE_FAIL;
        }
        // 查询条件做成
        Map<String, Object> argMap = new HashMap<String, Object>();

        //===========================
        // 根据交付的选择方式分别取出交付信息
        //===========================
        switch (selectType) {
            case 1:
                /*
                 *针对Project处理
                 */
                String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

                argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);

                initDeliverys(input, output, argMap);
                break;
            case 2:
                /*
                 *针对TG处理
                 */

                String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 检查TG是否存在
                pmTgService.checkTgExist(tgId);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
                initDeliverys(input, output, argMap);
                break;
            case 3:
                /*
                 *针对Task处理
                 */

                // 检查TG是否存在
                pmTgService.checkTgExist(StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID)));

                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                // 检查任务是否存在
                pmTaskService.checkTaskExist(taskId);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, taskId);
                initDeliverys(input, output, argMap);
                break;
            default:
                break;
        }

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

    /**
      用户信息
     * @param input    页面输入
     * @param output   页面输出
     * @param argMap   其它参数（任务/TG/项目）
     * @Title: initDeliverys
     * @Description: 初期化交付列表
     */
    private void initDeliverys(Rb0300151InitInput input, Rb0300151InitOutput output, Map<String, Object> argMap) {

        // 翻页处理
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            pagination = new Pagination();
        }

        // 初始化搜索条件
        SessionUtil.removeAttribute(SessionKeys.BS_DELIVERY_SEARCH_PARAM);

        // 取得记录总数
        int rowTotal = pmTaskDeliveryMapper.countDeliverysByCondition(pagination, argMap);
        // 设置记录总数
        pagination.setRowTotal(rowTotal);

        // 分页查询
        List<Map<String, Object>> deliveryList = pmTaskDeliveryMapper.selectDeliverysByCondition(pagination, argMap);

        // 设置分页查询结果
        output.setDeliveryList(deliveryList);
        output.setPagination(pagination);

        // 查询条件再保存
        output.setDeliveryName(input.getDeliveryName());
        output.setDirectoryId(input.getDirectoryId());
        output.setSelectType(input.getSelectType());
    }
}
