// Copyright(c) 2013 ReliaBench. All rights reserved.
package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300151SearchInput;
import com.nationrel.modules.project.dto.output.Rb0300151SearchOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskDeliveryMapper;
import com.nationrel.modules.project.service.Rb0300151SearchService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300151SearchServiceImpl
 * @Description 任务交付物列表页面初始化业务类
 * @Author aaron
 * @CreateDate 2014-03-17
 */
@Service
public class Rb0300151SearchServiceImpl implements Rb0300151SearchService {

    @Autowired
    private PmTaskDeliveryMapper pmTaskDeliveryMapper;

    /**
      用户信息
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @throws Exception 异常处理
     * @Title: perform
     * @Description 任务交付物列表页面初始化业务方法
     */
    @Override
    public String perform(Rb0300151SearchInput input, Rb0300151SearchOutput output) throws Exception {

        //===========================
        // 根据交付的选择方式分别取出交付信息
        //===========================
        Integer selectDocumentType = input.getSelectType();
        if (selectDocumentType == null) {
            return Constants.SERVICE_FAIL;
        }

        searchDelivery(input, output, selectDocumentType);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

    /**

     * @param input
     * @param output             void
     * @param selectDocumentType
     * @Title: searchDelivery
     * @Description: 交付搜索
     */
    @SuppressWarnings("unchecked")
    private void searchDelivery(Rb0300151SearchInput input, Rb0300151SearchOutput output, Integer selectDocumentType) {

        // 查询条件做成
        Map<String, Object> argMap = new HashMap<String, Object>();

        // 翻页处理
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            pagination = new Pagination();
        }

        // 点击搜索，设置搜索条件
        if (!Constants.BLANK.equals(input.getIsSearch()) && Constants.C_STAT_1.equals(input.getIsSearch())) {
            editSearchCondition(input, argMap);
            SessionUtil.setAttribute(SessionKeys.BS_DELIVERY_SEARCH_PARAM, argMap);
        } else {
            argMap = (HashMap<String, Object>) SessionUtil.getAttribute(SessionKeys.BS_DELIVERY_SEARCH_PARAM);
            if (argMap == null) {
                argMap = new HashMap<String, Object>();
                editSearchCondition(null, argMap);
            }
        }

        // 点击搜索后，清楚当前页信息
        if (!Constants.BLANK.equals(input.getIsSearch()) && Constants.C_STAT_1.equals(input.getIsSearch())) {
            input.setIsSearch(Constants.BLANK);
            pagination.setPageCurrent(Constants.C_NUM_1);
        }

        switch (selectDocumentType) {
            case 1:
                // 交付类型为项目
                String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));

                argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);

                break;
            case 2:
                // 交付类型为TG
                String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);

                break;
            case 3:
                // 交付类型为任务
                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
                argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, taskId);

                break;
            default:
                break;
        }

        // 取得记录总数
        int rowTotal = pmTaskDeliveryMapper.countDeliverysByCondition(pagination, argMap);
        // 设置记录总数
        pagination.setRowTotal(rowTotal);

        // 分页查询
        List<Map<String, Object>> deliveryList = pmTaskDeliveryMapper.selectDeliverysByCondition(pagination, argMap);

        // 设置分页查询结果
        output.setDeliveryList(deliveryList);
        output.setPagination(pagination);

        // 查询条件再保存
        output.setDirectoryId(input.getDirectoryId());
        output.setDeliveryName(input.getDeliveryName());
        output.setSelectType(input.getSelectType());
    }

    /**

     * @param input
     * @param argMap   void
     * @Title: editSearchCondition
     * @Description: 设定查询条件
     */
    private void editSearchCondition(Rb0300151SearchInput input, Map<String, Object> argMap) {
        if (input != null) {
            argMap.put(PmMapKeySet.KEY_S_DELIVERY_TYPE, input.getDirectoryId());
            argMap.put(PmMapKeySet.KEY_S_DELIVERY_NAME, input.getDeliveryName());
        } else {
            argMap.put(PmMapKeySet.KEY_S_DELIVERY_TYPE, Constants.BLANK);
            argMap.put(PmMapKeySet.KEY_S_DELIVERY_NAME, Constants.BLANK);
        }
    }
}
