package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300160InitInput;
import com.nationrel.modules.project.dto.output.Rb0300160InitOutput;
import com.nationrel.modules.project.jar.rb_common.bizcommon.constant.BizModuleCode;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.Rb0300160InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300260InitServiceImpl
 * @Description 项目阶段初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300160InitServiceImpl implements Rb0300160InitService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目阶段初始化的业务方法
     */
    @Override
    public String perform(Rb0300160InitInput input, Rb0300160InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 页面项目值设定
        output.setProjectId(projectId);
        output.setModuleId(BizModuleCode.MODULE_ID_PROJECT);

        //========================================
        // 页面权限控制
        // key_LifeState&_key_ResourceType&key_ChildResourceType-StateAuthority
        // key_RoleType&_key_ResourceType&key_ChildResourceType-RoleAuthority
        //========================================
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.STAGE_TYPE);
        argMap.put(PmMapKeySet.KEY_S_AUTHORITY_SWITCH, ReliaFlowConstants.IS_AUTHORITY_SWITCH_NO);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        output.setAuthMap(authMap);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
