package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300210InitInput;
import com.nationrel.modules.project.dto.output.Rb0300210InitOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300210InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;


/**
 * @version V1.0
 * @ClassName Rb0300210InitServiceImpl
 * @Description TG查看页面初始化的业务类
 * @Author andy
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300210InitServiceImpl implements Rb0300210InitService {

    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG编辑页面初始化的业务方法
     */
    @Override
    public String perform(Rb0300210InitInput input, Rb0300210InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目ID
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        //获取TgId并设置到session中
        String tgId = input.getTgId();
        if (tgId == null || tgId.equals(Constants.BLANK)) {
            tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 获取Tg信息
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.BASE_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        output.setAuthMap(authMap);

        //输出Tg信息
        output.setTgId(pmTg.getTgId());
        output.setTgName(pmTg.getTgName());
        output.setDescription(pmTg.getDescription());
        output.setGradation(pmTg.getTgSort());
        output.setPlanEndTime(DateUtils.convertDatetoString(pmTg.getPlanEndTime()));
        output.setPlanStartTime(DateUtils.convertDatetoString(pmTg.getPlanStartTime()));
        output.setEndTime(DateUtils.convertDatetoString(pmTg.getEndTime()));
        output.setStartTime(DateUtils.convertDatetoString(pmTg.getStartTime()));

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
