package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300211SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300211SaveOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300211SaveService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300211SaveServiceImpl
 * @Description TG创建业务类
 * @Author andy
 * @CreateDate 2013-12-17
 */
@Service
public class Rb0300211SaveServiceImpl implements Rb0300211SaveService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG创建业务方法
     */
    @Override
    public String perform(Rb0300211SaveInput input, Rb0300211SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 页面权限控制
        Map<String, String> argTgAuthMap = new HashMap<String, String>();
        argTgAuthMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argTgAuthMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argTgAuthMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, projectId);
        argTgAuthMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_PROJECT_TYPE);
        argTgAuthMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
        argTgAuthMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

        // 取得具体资源的权限
        Map<String, String> tgAuthMap = authorityCtrlService.doResourceAuthority(argTgAuthMap);
        String createAuth = tgAuthMap.get(ReliaFlowConstants.AUTHORITY_CREATE);

        // 权限发生变化-操作不能进行
        if (StringUtils.isNotEmpty(createAuth)) {
            throw new OptimisticLockException(MessageId.W009805);
        }

        // 时间整合性检查
        Date dtPlanStartTime = DateUtils.convertDate(input.getPlanStartTime());
        Date dtPlanEndTime = DateUtils.convertDate(input.getPlanEndTime());
        pmTgService.checkTimeValid(dtPlanStartTime, dtPlanEndTime, projectId, null);

        // 获取参数并赋给TG对象
        PmTg pmTg = new PmTg();
        int tgSort = pmTgMapper.getMaxTgSort(projectId);
        pmTg.setProjectId(projectId);
        pmTg.setTgName(input.getTgName());
        pmTg.setTgSort(tgSort);
        pmTg.setPlanStartTime(dtPlanStartTime);
        pmTg.setPlanEndTime(dtPlanEndTime);
        pmTg.setLifeState(ReliaFlowConstants.LIFECYCLE_RELIA_PJ_CREATE);
        pmTg.setDescription(StringUtil.parseString(input.getDescription()).trim());

        // 设置系统信息到TG对象
        pmTg.setInsertUserId(sysUser.getUsername());
        pmTg.setInsertPgmId(sysUser.getPgmId());

        // 保存Tg到数据库
        pmTgMapper.insert(pmTg);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
