package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300212EditInput;
import com.nationrel.modules.project.dto.output.Rb0300212EditOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300212EditService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300212EditServiceImpl
 * @Description TG基本信息编辑业务类
 * @Author andy
 * @CreateDate 2013-12-17
 */
@Service
public class Rb0300212EditServiceImpl implements Rb0300212EditService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG编辑业务方法
     */
    @Override
    public String perform(Rb0300212EditInput input, Rb0300212EditOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);

        // 设置TG信息
        PmTg tg = pmTgMapper.selectByPrimaryKey(tgId);

        // 检查TG是否存在
        if (tg == null) {
            throw new OptimisticLockException(MessageId.W300103);
        }

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.BASE_TYPE);
        argMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        String editAuth = authMap.get(ReliaFlowConstants.AUTHORITY_MODIFY);

        // 权限发生变化-操作不能进行
        if (StringUtils.isNotEmpty(editAuth)) {
            throw new OptimisticLockException(MessageId.W009805);
        }

        // 获取session中TG version number
        Integer versionNumber = (Integer) SessionUtil.getAttribute(SessionKeys.RB_PJ_TG_BASIC_EDIT_VERSION);

        // 检查当前TG的版本与session中值是否相等
        if (!tg.getVersionNo().equals(versionNumber)) {
            throw new OptimisticLockException(MessageId.W009801);
        }

        // 时间有变动，时间整合性检查
        Date dtPlanStartTime = DateUtils.convertDate(input.getPlanStartTime());
        Date dtPlanEndTime = DateUtils.convertDate(input.getPlanEndTime());
        if (DateUtils.isNotDateEquals(tg.getPlanStartTime(), dtPlanStartTime) ||
                DateUtils.isNotDateEquals(tg.getPlanEndTime(), dtPlanEndTime)) {
            pmTgService.checkTimeValid(dtPlanStartTime, dtPlanEndTime, projectId, tgId);
        }

        // 更新TG
        tg.setTgId(tgId);
        tg.setProjectId(projectId);
        tg.setTgName(input.getTgName());
        tg.setUpdatePgmId(sysUser.getPgmId());
        tg.setUpdateUserId(sysUser.getUsername());
        tg.setDescription(input.getDescription());
        tg.setPlanStartTime(dtPlanStartTime);
        tg.setPlanEndTime(dtPlanEndTime);
        pmTgMapper.updateByPrimaryKey(tg);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
