package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300212InitInput;
import com.nationrel.modules.project.dto.output.Rb0300212InitOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.Rb0300212InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300212InitServiceImpl
 * @Description TG编辑初始化业务类
 * @Author andy
 * @CreateDate 2013-12-17
 */
@Service
public class Rb0300212InitServiceImpl implements Rb0300212InitService {

    @Autowired
    private PmTgMapper pmTgMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG编辑初始化业务方法
     */
    @Override
    public String perform(Rb0300212InitInput input, Rb0300212InitOutput output) throws Exception {

        //获取TgId并设置到session中
        String tgId = input.getTgId();
        if (tgId == null || tgId.equals(Constants.BLANK)) {
            tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }

        // 获取Tg信息
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);

        if (pmTg == null) {
            throw new OptimisticLockException(MessageId.W300103);
        }

        // 保存version number 到session
        SessionUtil.setAttribute(SessionKeys.RB_PJ_TG_BASIC_EDIT_VERSION, pmTg.getVersionNo());

        // 设置输出
        output.setTgId(tgId);
        output.setTgId(pmTg.getTgId());
        output.setTgName(pmTg.getTgName());
        output.setDescription(pmTg.getDescription());
        output.setPlanEndTime(DateUtils.convertDatetoString(pmTg.getPlanEndTime()));
        output.setPlanStartTime(DateUtils.convertDatetoString(pmTg.getPlanStartTime()));
        output.setSort(StringUtil.parseString(pmTg.getTgSort()));

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
