package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300220DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300220DeleteOutput;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300220DeleteService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;


/**
 * @version V1.0
 * @ClassName Rb0300220DeleteServiceImpl
 * @Description 任务删除的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300220DeleteServiceImpl implements Rb0300220DeleteService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务删除的业务方法
     */
    @Override
    public String perform(Rb0300220DeleteInput input, Rb0300220DeleteOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 取得Task标识
        String taskId = input.getTaskId();

        // 根据任务子节点ID查找跟节点ID
        PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
        String firstParentId = Constants.BLANK;
        String curParentId = Constants.BLANK;
        if (pmTaskTree != null && ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            firstParentId = pmTaskTree.getTaskId();
            curParentId = pmTaskTree.getParentTaskId();
        } else if (pmTaskTree != null && !ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            firstParentId = findFirstParentId(pmTaskTree.getParentTaskId(), pmTaskTree);
        }
        if (StringUtils.isEmpty(firstParentId)) {
            // 数据已经被删除的异常
            throw new OptimisticLockException(MessageId.W009803);
        }

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, input.getTaskId());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
        argMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        String delAuth = authMap.get(ReliaFlowConstants.AUTHORITY_DELETE);

        if (ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(curParentId)) {
            // 页面TG下级权限控制
            Map<String, String> paramMap = new HashMap<String, String>();
            paramMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
            paramMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
            paramMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
            paramMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
            paramMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
            paramMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
            paramMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

            // 取得具体资源的权限
            Map<String, String> authTgMap = authorityCtrlService.doResourceAuthority(paramMap);
            if (ReliaFlowConstants.AUTH_CTRL_VIEW.equals(authMap.get(ReliaFlowConstants.AUTHORITY_CREATE))) {
                delAuth = authTgMap.get(ReliaFlowConstants.AUTHORITY_DELETE);
            }
        }

        // 权限发生变化-操作不能进行
        if (StringUtils.isNotEmpty(delAuth)) {
            throw new OptimisticLockException(MessageId.W009805);
        }

        //================================
        // 任务删除排他处理
        //================================
        @SuppressWarnings("unchecked")
        Map<String, Integer> lockTask = (Map<String, Integer>) SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION);
        Integer oldVersionNo = lockTask.get(ReliaFlowConstants.PJ_TASK_PAGE_NUMBER + firstParentId);
        /*String lockId = ReliaFlowConstants.PJ_TASK_PAGE_NUMBER + firstParentId;
        dataLockService.checkDataLock(lockId, oldVersionNo);*/

        //================================
        // 采用左右键值维护树型关系-[当前节点]被删除的处理
        // 1、取得[当前节点]的[父节点]-如果[父节点]已经没有子节点，则更新为叶子节点;
        // 2、[当前节点]及所属子节点删除处理;
        // 3、[当前节点]删除引发的其它节点的左右键值的修改;
        //================================
        // 2、[当前节点]及所属子节点删除处理;
        PmTaskTree curNode = pmTaskTreeMapper.selectByPrimaryKey(taskId);
        pmTaskMapper.deleteTaskByLRNo(curNode.getLeftNo(), curNode.getRightNo(), curNode.getOrdinal(), tgId);
        pmTaskTreeMapper.deleteTaskNodeAndChildNode(curNode.getLeftNo(), curNode.getRightNo(), curNode.getOrdinal(), tgId);

        if (!ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(curNode.getParentTaskId())) {
            // 1、取得[当前节点]的[父节点]-如果[父节点]已经没有子节点，则更新为叶子节点;
            PmTaskTree parentNode = pmTaskTreeMapper.selectByPrimaryKey(curNode.getParentTaskId());
            int parentChildNodeCnt = (parentNode.getRightNo() - parentNode.getLeftNo() - 1) / 2;
            int deleteCnt = (curNode.getRightNo() - curNode.getLeftNo() + 1) / 2;
            if (parentChildNodeCnt == deleteCnt) {
                parentNode.setType(ReliaFlowConstants.TREE_LEAF);
                pmTaskTreeMapper.updateByPrimaryKey(parentNode);
            }

            // 3、[当前节点]删除引发的其它节点的左右键值的修改;
            pmTaskTreeMapper.updateTaskNodeLeftNoCaseDelete(deleteCnt * 2, curNode.getLeftNo(), curNode.getOrdinal(), tgId);
            pmTaskTreeMapper.updateTaskNodeRightNoCaseDelete(deleteCnt * 2, curNode.getRightNo(), curNode.getOrdinal(), tgId);
        }

        // 4、删除任务下相关的检查项、人员、交付
        pmTaskService.deleteTaskByTaskNode(curNode.getLeftNo(), curNode.getRightNo(), curNode.getOrdinal(), tgId);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param taskId
     * @param pmTaskTree
     * @return firstParentId
     * @Title: findFirstParentId
     * @Description 递归查找任务根节点Id
     */
    public String findFirstParentId(String taskId, PmTaskTree pmTaskTree) {
        String firstParentId = Constants.BLANK;
        pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
        if (pmTaskTree != null && ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            firstParentId = pmTaskTree.getTaskId();
            return firstParentId;
        } else if (pmTaskTree != null && !ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            return findFirstParentId(pmTaskTree.getParentTaskId(), pmTaskTree);
        } else {
            return firstParentId;
        }
    }
}
