package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300220InitInput;
import com.nationrel.modules.project.dto.output.Rb0300220InitOutput;
import com.nationrel.modules.project.entity.TaskPageModel;
import com.nationrel.modules.project.entity.TaskToolModel;
import com.nationrel.modules.project.config.SysConfig;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmPercentStatService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300220InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300220InitServiceImpl
 * @Description 任务列表初始化的业务类
 * @Author aaron
 * @CreateDate 2013-3-7
 */
@Service
public class Rb0300220InitServiceImpl implements Rb0300220InitService {

    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmPercentStatService pmPercentStatService;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private SysConfig sysConfig;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务列表初始化的业务方法
     */
    @Override
    public String perform(Rb0300220InitInput input, Rb0300220InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));

        // 取得TG标识
        String tgId = input.getTgId();
        if (tgId == null || Constants.BLANK.equals(tgId)) {
            tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 任务列表数据取得
        Map<String, Object> argMap = new HashMap<String, Object>();
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_OWNER_MARK, Constants.C_STAT_1);
        List<TaskPageModel> tasks = pmTaskMapper.searchPmTaskList(argMap);

        // Task完成度统计
        Map<String, Object> taskPercentParmMap = new HashMap<String, Object>();
        taskPercentParmMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        taskPercentParmMap.put(PmMapKeySet.KEY_S_START_NODE_ID, ReliaFlowConstants.TASK_FIRST_PARENT_ID);
        Map<String, BigDecimal> percentMap = pmPercentStatService.statAllNodePercent(taskPercentParmMap);

        // 页面TG下级权限控制
        Map<String, String> paramMap = new HashMap<String, String>();
        paramMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        paramMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        paramMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        paramMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
        paramMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
        paramMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(paramMap);

        // task列表对象查看权限
        Map<String, String> argTaskParamMap = new HashMap<String, String>();
        argTaskParamMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argTaskParamMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argTaskParamMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        Map<String, Map<String, String>> authResourceMap = authorityCtrlService.doViewAuthorityByGroup(argTaskParamMap);

        // 获取配置文件中画面树的深度不能超过的最大值
        int treeMaxDeep = Integer.parseInt(sysConfig.getTaskTreeMaxDeep());

        // 获取任务树初始化 页面编号+树的根记录标识 集合
        Map<String, Integer> lockTask = new HashMap<String, Integer>();

        // 操作按钮是否隐藏
        boolean ctrlAddResult = false;
        boolean ctrlEditResult = false;
        boolean ctrlDelResult = false;

        // 是否所有任务都在创建状态
        int isAllCreate = 1;

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;
        int nodePadding = 19;

        if (tasks != null && tasks.size() > 0) {

            Map<String, String> resultTaskParam;
            String[] toolLinks = null;
            String[] toolNames = null;
            for (TaskPageModel task : tasks) {

                toolLinks = null;
                toolNames = null;
                nodePadding = 19;

                // 计算树形结点间距
                nodePadding = nodePadding * (task.getDeep() - 1);

                // 通过判断是否叶子结点-设定行样式
                if (ReliaFlowConstants.TREE_LEAF.equals(task.getType())) {
                    task.setTrTreeClass(ReliaFlowConstants.CONST_TR_TREE_CLASS_LEAF);
                    task.setTdTreeClass(ReliaFlowConstants.CONST_TD_TREE_CONTEXT.replace("REPLACE_PADDING", String.valueOf(nodePadding))
                            .replace("REPLACE_OPERATE", "")
                            .replace("REPLACE_ONCLICK", ""));
                } else {
                    task.setTrTreeClass(ReliaFlowConstants.CONST_TR_TREE_CLASS_BRANCH);
                    task.setTdTreeClass(ReliaFlowConstants.CONST_TD_TREE_CONTEXT.replace("REPLACE_PADDING", String.valueOf(nodePadding))
                            .replace("REPLACE_OPERATE", ReliaFlowConstants.CONST_TD_OPERATE)
                            .replace("PEPLACE_TITLE_OPERATE", input.getFlexNodeTitle())
                            .replace("REPLACE_ONCLICK", ReliaFlowConstants.CONST_FLEX_BRANCH));
                }

                // 是否所有任务都在创建状态
                if (isAllCreate == 1 && ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.compareTo(task.getLifeState()) <= 0) {
                    isAllCreate = 0;
                }

                // 权限数据取得
                resultTaskParam = authResourceMap.get(task.getTaskId() + ReliaFlowConstants.RESOURCE_TASK_TYPE);

                // task是否顶级任务 ,是: 删除权限:编辑权限:任务子节点增加权限三个权限用authMap对应的覆盖 ,否:用自身权限
                if (ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(task.getParentTaskId())) {
                    // 如果TG的下级有创建权限，则顶级任务(因为此处顶级任务和非顶级任务都是通过doViewAuthorityByGroup取得的)与TG的下级权限相同，不做处理；
                    // 否则，如果TG的下级没有创建权限，则顶级任务(因为此处顶级任务和非顶级任务都是通过doViewAuthorityByGroup取得的)与TG的下级权限仅不同点是删除。
                    if (ReliaFlowConstants.AUTH_CTRL_VIEW.equals(authMap.get(ReliaFlowConstants.AUTHORITY_CREATE))) {
                        // 删除权限
                        resultTaskParam.put(ReliaFlowConstants.AUTHORITY_DELETE, authMap.get(ReliaFlowConstants.AUTHORITY_DELETE));
                    }
                }

                // 设置任务锁版本号
                if (ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(task.getParentTaskId())) {
                    lockTask.put(ReliaFlowConstants.PJ_TASK_PAGE_NUMBER + task.getTaskId(), task.getLockVersionNo());
                }

                // 根据taskId获取工具集
                if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.VIEW_AUTHORITY_TOOL))) {
                    List<TaskToolModel> taskToolList = new ArrayList<TaskToolModel>();
                    if (task.getToolLink() != null) {
                        toolNames = task.getToolName().split(",");
                        toolLinks = task.getToolLink().split(",");
                    }
                    if (toolLinks != null) {
                        for (int t = 0; t < toolLinks.length; t++) {
                            TaskToolModel taskToolModel = new TaskToolModel();
                            taskToolModel.setToolName(toolNames[t]);
                            taskToolModel.setToolLink(toolLinks[t]);
                            taskToolList.add(taskToolModel);
                        }
                    }
                    if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.CREATE_AUTHORITY_TOOL))) {
                        TaskToolModel taskToolModel = new TaskToolModel();
                        taskToolModel.setToolName(ReliaFlowConstants.TASK_TOOL_CREATE_I18N);
                        taskToolModel.setToolLink(ReliaFlowConstants.TASK_TOOL_CREATE_URI);
                        taskToolList.add(taskToolModel);
                    }
                    task.setTaskToolList(taskToolList);
                }

                // 工作中Task完成度统计
                if (ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK.equals(task.getLifeState())
                        || ReliaFlowConstants.LIFECYCLE_RELIA_TASK_WAIT_COMPLETE.equals(task.getLifeState())) {
                    if (percentMap != null && percentMap.get(task.getTaskId()) != null) {
                        BigDecimal percent = new BigDecimal(StringUtil.parseString(percentMap.get(task.getTaskId())));
                        BigDecimal zero = new BigDecimal(0);
                        if (!percent.equals(zero)) {
                            task.setPercent(percent);
                        }
                    }
                }

                // 如果页面任务树的深度超过最大值、则隐藏添加子任务的按钮
                if (task.getDeep() >= treeMaxDeep) {
                    resultTaskParam.put(ReliaFlowConstants.AUTHORITY_EXT01, ReliaFlowConstants.AUTH_CTRL_VIEW);
                }

                // 设置权限
                task.setAuthMap(resultTaskParam);

                if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.AUTHORITY_EXT01)) && !ctrlAddResult) {
                    ctrlAddResult = true;
                }
                if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.AUTHORITY_MODIFY)) && !ctrlEditResult) {
                    ctrlEditResult = true;
                }
                if (StringUtils.isEmpty(resultTaskParam.get(ReliaFlowConstants.AUTHORITY_DELETE)) && !ctrlDelResult) {
                    ctrlDelResult = true;
                }
            }
        }

        if (ctrlAddResult) {
            ctrOptCount++;
        }
        if (ctrlEditResult) {
            ctrOptCount++;
        }
        if (ctrlDelResult) {
            ctrOptCount++;
        }

        // 统计显示操作按钮的数量
        output.setCtrOptCount(ctrOptCount);

        // 设置任务锁
        SessionUtil.setAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION, lockTask);

        // 任务列表页面设定
        output.setTasks(tasks);
        // 任务创建权限
        output.setCreateTaskAuthority(authMap.get(ReliaFlowConstants.AUTHORITY_CREATE));
        output.setIsAllCreate(isAllCreate);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
