package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300230DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300230DeleteOutput;
import com.nationrel.modules.project.entity.PmTgCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgCheckMapper;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300230DeleteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300230DeleteServiceImpl
 * @Description TG检查项删除业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300230DeleteServiceImpl implements Rb0300230DeleteService {

    @Autowired
    private PmTgCheckMapper pmTgCheckMapper;
    @Autowired
    private PmTgService pmTgService;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG检查项删除业务方法
     */
    @Override
    public String perform(Rb0300230DeleteInput input, Rb0300230DeleteOutput output) throws Exception {

        // 获取参数
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        Integer tgCheckId = input.getTgCheckId();

        // 1.检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 2.检查check item 是否存在
        PmTgCheck pmTgCheck = pmTgCheckMapper.selectByPrimaryKey(tgCheckId, tgId);
        if (pmTgCheck == null) {
            throw new OptimisticLockException(MessageId.W009803);
        }

        // 3.删除当前TG检查项
        pmTgCheckMapper.deleteByPrimaryKey(tgCheckId, tgId);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
