package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300230InitInput;
import com.nationrel.modules.project.dto.output.Rb0300230InitOutput;
import com.nationrel.modules.project.entity.PmTgCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgCheckMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300230InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300230InitServiceImpl
 * @Description TG检查列表初始化业务类
 * @Author andy
 * @CreateDate 2014-03-10
 */
@Service
public class Rb0300230InitServiceImpl implements Rb0300230InitService {

    @Autowired
    private PmTgCheckMapper pmTgCheckMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG检查列表初始化业务方法
     */
    @Override
    public String perform(Rb0300230InitInput input, Rb0300230InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取project标识 && TG标识
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        String tgId = input.getTgId();
        if (tgId == null || Constants.BLANK.equals(tgId)) {
            tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        String lockId = ReliaFlowConstants.PJ_TG_CHECK_PAGE_NUMBER + tgId;

        // 根据项目标识取出检查列表
        List<PmTgCheck> pmChecks = pmTgCheckMapper.findCheckListByTgId(tgId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHECK_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_MODIFY))) {
            ctrOptCount++;
        }
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }

        // 检查列表页面设定
        output.setPmChecks(pmChecks);
        output.setAuthMap(authMap);
        output.setCtrOptCount(ctrOptCount);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
