package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300230SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300230SaveOutput;
import com.nationrel.modules.project.entity.PmTgCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgCheckMapper;
import com.nationrel.modules.project.service.Rb0300230SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * @version V1.0
 * @ClassName Rb0300230SaveServiceImpl
 * @Description 项目检查列表保存业务类
 * @Author andy
 * @CreateDate 2013-12-19
 */
@Service
public class Rb0300230SaveServiceImpl implements Rb0300230SaveService {

    @Autowired
    private PmTgCheckMapper pmTgCheckMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目检查列表保存业务方法
     */
    @Override
    public String perform(Rb0300230SaveInput input, Rb0300230SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // tg标识取得
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);

        // 取得页面TG检查列表
        List<PmTgCheck> pmProjectChecks = input.getPmTgCheck();

        //================================
        // TG检查列表保存排他处理
        //================================
        String lockId = ReliaFlowConstants.PJ_TG_CHECK_PAGE_NUMBER + tgId;
        Integer oldVersionNo = (Integer) SessionUtil.getAttribute(SessionKeys.RB_PJ_TG_CHECK_SAVE_VERSION);

        // TG检查列表保存
        PmTgCheck updateEntity = null;
        PmTgCheck entity = null;
        for (int i = 0; i < pmProjectChecks.size(); i++) {

            // 取得页面检查项实体
            entity = pmProjectChecks.get(i);

            // 根据主键从DB取得检查项
            updateEntity = pmTgCheckMapper.selectByPrimaryKey(entity.getTgCheckId(), tgId);

            // 设置系统信息
            updateEntity.setInsertUserId(sysUser.getUsername());
            updateEntity.setInsertPgmId(sysUser.getPgmId());

            // 页面与DB检查结果一致
            if (ReliaFlowConstants.CHECK_RESULT_PASS.equals(entity.getResult())) {
                if (ReliaFlowConstants.CHECK_RESULT_PASS.equals(updateEntity.getResult())) {
                    continue;
                } else {
                    updateEntity.setResult(ReliaFlowConstants.CHECK_RESULT_PASS);

                    // 检查项更新
                    pmTgCheckMapper.updateByPrimaryKey(updateEntity);
                }
            } else {
                if (ReliaFlowConstants.CHECK_RESULT_INIT.equals(updateEntity.getResult())) {
                    continue;
                } else {
                    updateEntity.setResult(ReliaFlowConstants.CHECK_RESULT_INIT);

                    // 检查项更新
                    pmTgCheckMapper.updateByPrimaryKey(updateEntity);
                }
            }
        }

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
