package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300231SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300231SaveOutput;
import com.nationrel.modules.project.entity.PmTgCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgCheckMapper;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300231SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300231SaveServiceImpl
 * @Description TG检查项保存业务类
 * @Author andy
 * @CreateDate 2013-12-19
 */
@Service
public class Rb0300231SaveServiceImpl implements Rb0300231SaveService {

    @Autowired
    private PmTgCheckMapper pmTgCheckMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG检查项保存业务方法
     */
    @Override
    public String perform(Rb0300231SaveInput input, Rb0300231SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得TG标识
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 构建检查项实体
        PmTgCheck entity = new PmTgCheck();

        // 基本信息
        entity.setTgCheckId(pmTgCheckMapper.selectMaxId(tgId));
        entity.setTgId(tgId);
        entity.setObject(input.getObject());
        entity.setContent(input.getContent());
        entity.setDescription(input.getDescription());
        entity.setResult(ReliaFlowConstants.CHECK_RESULT_INIT);

        // 系统信息
        entity.setInsertUserId(sysUser.getUsername());
        entity.setInsertPgmId(sysUser.getPgmId());

        // 检查项保存数据库
        pmTgCheckMapper.insert(entity);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
