package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300232InitInput;
import com.nationrel.modules.project.dto.output.Rb0300232InitOutput;
import com.nationrel.modules.project.entity.PmTgCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgCheckMapper;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300232InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300232InitServiceImpl
 * @Description TG检查项编辑初始化业务类
 * @Author andy
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300232InitServiceImpl implements Rb0300232InitService {

    @Autowired
    private PmTgCheckMapper pmTgCheckMapper;
    @Autowired
    private PmTgService pmTgService;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG检查项编辑初始化业务方法
     */
    @Override
    public String perform(Rb0300232InitInput input, Rb0300232InitOutput output) throws Exception {

        // 获取参数
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        Integer tgCheckId = input.getTgCheckId();

        // 1.检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 2.检查check item 是否存在
        PmTgCheck pmTgCheck = pmTgCheckMapper.selectByPrimaryKey(tgCheckId, tgId);
        if (pmTgCheck == null) {
            throw new OptimisticLockException(MessageId.W009803);
        }

        // 保存version number 到session
        SessionUtil.setAttribute(SessionKeys.RB_PJ_TG_CHECKITEM_VERSION, pmTgCheck.getVersionNo());

        // 编辑页面元素值设定
        output.setTgCheckId(pmTgCheck.getTgCheckId());
        output.setObject(pmTgCheck.getObject());
        output.setContent(pmTgCheck.getContent());
        output.setDescription(pmTgCheck.getDescription());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
