package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300241InitInput;
import com.nationrel.modules.project.dto.output.Rb0300241InitOutput;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300241InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300241InitServiceImpl
 * @Description TG添加成员初始化业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300241InitServiceImpl implements Rb0300241InitService {

    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目内部增加成员选择页面页面初始化业务方法
     */
    @Override
    public String perform(Rb0300241InitInput input, Rb0300241InitOutput output) throws Exception {

        int selectType = input.getSelectType();
        Map<String, String> argMap = new HashMap<String, String>();
        switch (selectType) {
            case 1://TG
                argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
                argMap.put(PmMapKeySet.KEY_S_TG_ID, (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 检查TG是否存在
                pmTgService.checkTgExist((String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
                List<Map<String, String>> srcStaffs = pmProjectStaffMapper.searchPerson(argMap);
                List<Map<String, String>> existStaffs = pmTgStaffMapper.selectExistStaff(argMap);
                output.setUserList(combineStaff(srcStaffs, existStaffs));
                output.setSelectType(selectType);
                break;
            case 2://Task
                String tgIdTask = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

                // 检查TG是否存在
                pmTgService.checkTgExist(tgIdTask);

                // 获取当前Task成员信息
                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                // 检查任务是否存在
                pmTaskService.checkTaskExist(taskId);

                PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
                argMap.put(PmMapKeySet.KEY_S_TASK_ID, taskId);
                if (ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
                    argMap.put(PmMapKeySet.KEY_S_TG_ID, tgIdTask);
                    List<Map<String, String>> srcTaskStaffs = pmTgStaffMapper.searchPerson(argMap);
                    List<Map<String, String>> existTaskStaffs = pmTaskStaffMapper.selectExistStaff(argMap);
                    output.setUserList(combineStaff(srcTaskStaffs, existTaskStaffs));
                } else {
                    argMap.put(PmMapKeySet.KEY_S_PARENT_TASK_ID, pmTaskTree.getParentTaskId());
                    List<Map<String, String>> srcTaskStaffs = pmTaskStaffMapper.searchPerson(argMap);
                    List<Map<String, String>> existTaskStaffs = pmTaskStaffMapper.selectExistStaff(argMap);
                    output.setUserList(combineStaff(srcTaskStaffs, existTaskStaffs));
                }
                output.setSelectType(selectType);
                break;
            default:
                break;
        }

        // 返回成功
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param srcStaffs
     * @param existStaffs
     * @return List<Map < String, String>>
     * @Title: combineStaff
     * @Description: 编辑可选的成员列表（去除已增加的人员）
     */
    private List<Map<String, String>> combineStaff(List<Map<String, String>> srcStaffs,
                                                   List<Map<String, String>> existStaffs) {
        List<Map<String, String>> retList = new ArrayList<Map<String, String>>();

        // 编辑已增加过的成员
        Map<String, String> existMap = new HashMap<String, String>();
        for (Map<String, String> tmpMap : existStaffs) {
            existMap.put(tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID), tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID));
        }

        // 构建可选未被增加过的成员
        for (Map<String, String> staff : srcStaffs) {
            if (!existMap.containsKey(staff.get(PmMapKeySet.KEY_R_STAFF_ID))) {
                retList.add(staff);
            }
        }

        return retList;
    }

}
