package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300241SearchInput;
import com.nationrel.modules.project.dto.output.Rb0300241SearchOutput;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300241SearchService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300242SearchServiceImpl
 * @Description 项目内部增加成员选择页面页面查询的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300241SearchServiceImpl implements Rb0300241SearchService {

    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;
    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 项目内部增加成员选择页面页面查询业务方法
     */
    @Override
    public String perform(Rb0300241SearchInput input, Rb0300241SearchOutput output) throws Exception {

        Integer selectType = input.getSelectType();

        if (selectType == null) {
            return Constants.SERVICE_FAIL;
        }

        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        switch (selectType) {
            case 1:

                // 检查TG是否存在
                pmTgService.checkTgExist(tgId);

                // 构造查询参数
                Map<String, String> argMap = new HashMap<String, String>();
                argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
                argMap.put(PmMapKeySet.KEY_S_STAFF_NAME, input.getStaffName());
                argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);

                // 获取当前TG可用成员集合
                List<Map<String, String>> srcStaffs = pmProjectStaffMapper.searchPerson(argMap);
                // 获取当前TG有效成员集合
                List<Map<String, String>> existStaffs = pmTgStaffMapper.selectExistStaff(argMap);

                // 获取可用成员(未被增加过的)TG成员集合并设置到输出
                output.setUserList(combineStaff(srcStaffs, existStaffs));
                output.setStaffName(input.getStaffName());
                output.setSelectType(Integer.valueOf(Constants.C_NUM_1));
                break;
            case 2:

                // 检查TG是否存在
                pmTgService.checkTgExist(tgId);

                // 获取当前Task成员信息
                String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

                // 检查任务是否存在
                pmTaskService.checkTaskExist(taskId);

                // 构造查询参数
                Map<String, String> staffMap = new HashMap<String, String>();
                staffMap.put(PmMapKeySet.KEY_S_STAFF_NAME, input.getStaffName());
                staffMap.put(PmMapKeySet.KEY_S_TASK_ID, taskId);

                PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
                if (ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
                    staffMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
                    // 获取当前Task可用成员集合
                    List<Map<String, String>> srcTaskStaffs = pmTgStaffMapper.searchPerson(staffMap);
                    // 获取当前Task有效成员集合
                    List<Map<String, String>> existTaskStaffs = pmTaskStaffMapper.selectExistStaff(staffMap);
                    // 获取可用成员(未被增加过的)Task成员集合并设置到输出
                    output.setUserList(combineStaff(srcTaskStaffs, existTaskStaffs));
                } else {
                    staffMap.put(PmMapKeySet.KEY_S_PARENT_TASK_ID, pmTaskTree.getParentTaskId());
                    // 获取当前Task可用成员集合
                    List<Map<String, String>> srcTaskStaffs = pmTaskStaffMapper.searchPerson(staffMap);
                    // 获取当前Task有效成员集合
                    List<Map<String, String>> existTaskStaffs = pmTaskStaffMapper.selectExistStaff(staffMap);
                    // 获取可用成员(未被增加过的)Task成员集合并设置到输出
                    output.setUserList(combineStaff(srcTaskStaffs, existTaskStaffs));
                }
                output.setStaffName(input.getStaffName());
                output.setSelectType(Integer.valueOf(Constants.C_NUM_2));
                break;
            default:
                break;
        }
        // 返回成功
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param srcStaffs
     * @param existStaffs
     * @return List<Map < String, String>>
     * @Title: combineStaff
     * @Description: 编辑可选的成员列表（去除已增加的人员）
     */
    private List<Map<String, String>> combineStaff(List<Map<String, String>> srcStaffs,
                                                   List<Map<String, String>> existStaffs) {
        List<Map<String, String>> retList = new ArrayList<Map<String, String>>();

        // 编辑已增加过的成员
        Map<String, String> existMap = new HashMap<String, String>();
        for (Map<String, String> tmpMap : existStaffs) {
            existMap.put(tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID), tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID));
        }

        // 构建可选未被增加过的成员
        for (Map<String, String> staff : srcStaffs) {
            if (!existMap.containsKey(staff.get(PmMapKeySet.KEY_R_STAFF_ID))) {
                retList.add(staff);
            }
        }

        return retList;
    }

}
