package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300250DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300250DeleteOutput;
import com.nationrel.modules.project.entity.PmTgDelivery;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgDeliveryMapper;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300250DeleteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300250DeleteServiceImpl
 * @Description TG文档删除业务类
 * @Author andy
 * @CreateDate 2014-01-07
 */
@Service
public class Rb0300250DeleteServiceImpl implements Rb0300250DeleteService {

    @Autowired
    private PmTgDeliveryMapper pmTgDeliveryMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG文档删除业务方法
     */
    @Override
    public String perform(Rb0300250DeleteInput input, Rb0300250DeleteOutput output) throws Exception {

        // 获取参数
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        String deliveryId = input.getDeliveryId();
        String deliveryType = input.getDeliveryType();

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查交付是否存在
        PmTgDelivery pmTgDelivery = pmTgDeliveryMapper.selectByTGIdAndDeliveryId(tgId, deliveryId, deliveryType);
        if (pmTgDelivery == null) {
            throw new OptimisticLockException(MessageId.W009803);
        }

        // 删除TG文档
        pmTgDeliveryMapper.deleteDeliveryByTgIdAndDeliveryId(tgId, deliveryId, deliveryType);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
