package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300250InitInput;
import com.nationrel.modules.project.dto.output.Rb0300250InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgDeliveryMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300250InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300250InitServiceImpl
 * @Description TG文档列表页面初始化业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300250InitServiceImpl implements Rb0300250InitService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTgDeliveryMapper pmTgDeliveryMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TG文档列表页面初始化业务方法
     */
    @Override
    public String perform(Rb0300250InitInput input, Rb0300250InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取参数
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        String tgId = input.getTgId();
        if (tgId == null) {
            tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 根据tgId获取文档列表
        List<Map<String, Object>> deliveryList = pmTgDeliveryMapper.searchDeliveryListByTgId(tgId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.DOCUMENT_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }

        // 设置输出
        output.setDeliveryList(deliveryList);
        output.setAuthMap(authMap);
        output.setCtrOptCount(ctrOptCount);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
