package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300260CheckInput;
import com.nationrel.modules.project.dto.output.Rb0300260CheckOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.PmStageCtrlService;
import com.nationrel.modules.project.service.Rb0300260CheckService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300260CheckServiceImpl
 * @Description Task阶段检查的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300260CheckServiceImpl implements Rb0300260CheckService {

    @Autowired
    private PmStageCtrlService pmStageCtrlService;
    @Autowired
    private PmTgMapper pmTgMapper;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description Task阶段检查的业务方法
     */
    @Override
    public String perform(Rb0300260CheckInput input, Rb0300260CheckOutput output) throws Exception {
        // 项目标识
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        // TG标识
        String tgId = (String) SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID);
        // TG生命周期状态
        String prevLifeState = (String) SessionUtil.getAttribute(SessionKeys.RB_PJ_TG_LIFE_STATE_VERSION);
        // 页面设定
        input.setTgId(tgId);

        // 检查TG是否存在
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);

        if (pmTg == null) {
            throw new OptimisticLockException(MessageId.W300103);
        }
        if (!prevLifeState.equals(pmTg.getLifeState())) {
            throw new OptimisticLockException(MessageId.W009804);
        }

        // 当前结点向 完成状态迁移时-下级结点是否已完成
        pmStageCtrlService.ctrlTgState(projectId, tgId);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
