package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300311ChooseInitInput;
import com.nationrel.modules.project.dto.output.Rb0300311ChooseInitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300311ChooseInitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300311ChooseInitServiceImpl
 * @Description 任务成员选择页面初始化业务类
 * @Author aaron
 * @CreateDate 2014-01-20
 */
@Service
public class Rb0300311ChooseInitServiceImpl implements Rb0300311ChooseInitService {

//    @Autowired
//    private SysToolMapper sysToolMapper;
    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务成员选择页面初始化业务方法
     */
    @Override
    public String perform(Rb0300311ChooseInitInput input, Rb0300311ChooseInitOutput output) throws Exception {

        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_TG_ID, StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID)));

        // 翻页处理
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            pagination = new Pagination();
        }

        List<Map<String, String>> taskStaffsFroTask = pmTgStaffMapper.searchPerson(argMap);
        // 取得记录总数
        int rowTotal = taskStaffsFroTask.size();
        pagination.setRowTotal(rowTotal);

        Map<String, String> argExistMap = new HashMap<String, String>();
        argExistMap.put(PmMapKeySet.KEY_S_PROJECT_ID, StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID)));
        argExistMap.put(PmMapKeySet.KEY_S_TASK_ID, StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID)));
        // 获取当前任务下已经存在的人员
        List<Map<String, String>> existTaskStaffs = pmTaskStaffMapper.selectExistStaff(argExistMap);

        // 设置输出信息
        output.setUserList(combineStaff(taskStaffsFroTask, existTaskStaffs));
        output.setPagination(pagination);

        // 查询条件再保存
        output.setUserName(input.getUsername());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param srcStaffs
     * @param existStaffs
     * @return List<Map < String, String>>
     * @Title: combineStaff
     * @Description: 编辑可选的成员列表（去除已增加的人员）
     */
    private List<Map<String, String>> combineStaff(List<Map<String, String>> srcStaffs,
                                                   List<Map<String, String>> existStaffs) {
        List<Map<String, String>> retList = new ArrayList<Map<String, String>>();

        // 编辑已增加过的成员
        Map<String, String> existMap = new HashMap<String, String>();
        for (Map<String, String> tmpMap : existStaffs) {
            existMap.put(tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID), tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID));
        }

        // 构建可选未被增加过的成员
        for (Map<String, String> staff : srcStaffs) {
            if (!existMap.containsKey(staff.get(PmMapKeySet.KEY_R_STAFF_ID))) {
                retList.add(staff);
            }
        }

        return retList;
    }
}
