package com.nationrel.modules.project.service.impl;


import com.nationrel.modules.project.dto.input.Rb0300311ChooseStaffInput;
import com.nationrel.modules.project.dto.output.Rb0300311ChooseStaffOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.mapper.PmTgStaffMapper;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300311ChooseStaffService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300311ChooseStaffServiceImpl
 * @Description 任务成员选择页面初始化业务类
 * @Author andy
 * @CreateDate 2014-01-20
 */
@Service
public class Rb0300311ChooseStaffServiceImpl implements Rb0300311ChooseStaffService {

//    @Autowired
//    private SysToolMapper sysToolMapper;
    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务成员选择页面初始化业务方法
     */
    @SuppressWarnings("unchecked")
    @Override
    public String perform(Rb0300311ChooseStaffInput input, Rb0300311ChooseStaffOutput output) throws Exception {

        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        Map<String, String> argExistMap = new HashMap<String, String>();
        argExistMap.put(PmMapKeySet.KEY_S_PROJECT_ID, StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID)));
        argExistMap.put(PmMapKeySet.KEY_S_TASK_ID, StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID)));
        // 获取当前任务下已经存在的人员
        List<Map<String, String>> existTaskStaffs = pmTaskStaffMapper.selectExistStaff(argExistMap);

        Map<String, String> argMap = new HashMap<String, String>();

        // 翻页处理
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            pagination = new Pagination();
        }

        // 点击搜索，设置搜索条件
        if (!Constants.BLANK.equals(input.getIsSearch()) && Constants.C_STAT_1.equals(input.getIsSearch())) {
            editSearchCondition(input, argMap);
            SessionUtil.setAttribute(SessionKeys.BS_TASK_MEMBER_SEARCH_PARAM, argMap);
        } else {
            argMap = (HashMap<String, String>) SessionUtil.getAttribute(SessionKeys.BS_TASK_MEMBER_SEARCH_PARAM);
            if (argMap == null) {
                argMap = new HashMap<String, String>();
                editSearchCondition(null, argMap);
            }
        }

        // 点击搜索后，清楚当前页信息
        if (!Constants.BLANK.equals(input.getIsSearch()) && Constants.C_STAT_1.equals(input.getIsSearch())) {
            input.setIsSearch(Constants.BLANK);
            pagination.setPageCurrent(Constants.C_NUM_1);
        }

        List<Map<String, String>> tgStaffsFroTask = pmTgStaffMapper.searchPerson(argMap);
        //如果上级任务ID不为空
        String parentTaskId = input.getParentTaskId();
        List<Map<String, String>> taskStaffs = null;
        if (parentTaskId != null && !parentTaskId.equals(Constants.BLANK)) {
            argMap.put(PmMapKeySet.KEY_S_TASK_ID, parentTaskId);
            taskStaffs = pmTaskStaffMapper.selectExistStaff(argMap);
            List<Map<String, String>> result = new ArrayList<Map<String, String>>();
            for (int i = 0; i < taskStaffs.size(); i++) {
                Map<String, String> obj = taskStaffs.get(i);
                for (int k = 0; k < tgStaffsFroTask.size(); k++) {
                    if (obj.get(PmMapKeySet.KEY_R_STAFF_ID).equals(tgStaffsFroTask.get(k).get(PmMapKeySet.KEY_R_STAFF_ID))) {
                        result.add(tgStaffsFroTask.get(k));
                    }
                }
            }

            // 取得记录总数
            int rowTotal = result.size();
            pagination.setRowTotal(rowTotal);

            // 设置输出信息
            output.setUserList(combineStaff(tgStaffsFroTask, existTaskStaffs));
            output.setPagination(pagination);
        } else {
            // 取得记录总数
            int rowTotal = tgStaffsFroTask.size();
            pagination.setRowTotal(rowTotal);

            // 设置输出信息
            output.setUserList(combineStaff(tgStaffsFroTask, existTaskStaffs));
            output.setPagination(pagination);
        }

        // 查询条件再保存
        output.setUserName(input.getUsername());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**

     * @param input
     * @param argMap   void
     * @Title: editSearchCondition
     * @Description: 设定查询条件
     */
    private void editSearchCondition(Rb0300311ChooseStaffInput input, Map<String, String> argMap) {
        argMap.put(PmMapKeySet.KEY_S_TG_ID, StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID)));
        if (input != null) {
            argMap.put(PmMapKeySet.KEY_S_STAFF_NAME, input.getUsername());
        } else {
            argMap.put(PmMapKeySet.KEY_S_STAFF_NAME, Constants.BLANK);
        }
    }

    /**
     * @param srcStaffs
     * @param existStaffs
     * @return List<Map < String, String>>
     * @Title: combineStaff
     * @Description: 编辑可选的成员列表（去除已增加的人员）
     */
    private List<Map<String, String>> combineStaff(List<Map<String, String>> srcStaffs,
                                                   List<Map<String, String>> existStaffs) {
        List<Map<String, String>> retList = new ArrayList<Map<String, String>>();

        // 编辑已增加过的成员
        Map<String, String> existMap = new HashMap<String, String>();
        for (Map<String, String> tmpMap : existStaffs) {
            existMap.put(tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID), tmpMap.get(PmMapKeySet.KEY_R_STAFF_ID));
        }

        // 构建可选未被增加过的成员
        for (Map<String, String> staff : srcStaffs) {
            if (!existMap.containsKey(staff.get(PmMapKeySet.KEY_R_STAFF_ID))) {
                retList.add(staff);
            }
        }
        return retList;
    }
}
