package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300311InitInput;
import com.nationrel.modules.project.dto.output.Rb0300311InitOutput;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300311InitService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;


/**
 * @version V1.0
 * @ClassName Rb0300311InitServiceImpl
 * @Description 任务创建初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300311InitServiceImpl implements Rb0300311InitService {

    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务创建初始化的业务方法
     */
    @SuppressWarnings("unchecked")
    @Override
    public String perform(Rb0300311InitInput input, Rb0300311InitOutput output) throws Exception {

        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务父节点是否存在，若任务父节点不存在则不能追加子节点并且给出提示
        if (StringUtils.isNotEmpty(input.getParentTaskId())) {
            // 检查任务是否存在
            pmTaskService.checkTaskExist(input.getParentTaskId());

            // 任务创建页面设定
            output.setParentTaskId(input.getParentTaskId());

            // 获取任务锁
            Map<String, Integer> lockTask = (Map<String, Integer>) SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION);

            // 根据任务子节点ID查找跟节点ID
            PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(input.getParentTaskId());
            String firstParentId = Constants.BLANK;
            if (pmTaskTree != null && ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
                firstParentId = pmTaskTree.getTaskId();
            } else {
                firstParentId = findFirstParentId(pmTaskTree.getParentTaskId(), pmTaskTree);
            }
            if (lockTask != null) {
                String lockId = ReliaFlowConstants.PJ_TASK_PAGE_NUMBER + firstParentId;
                lockTask.remove(lockTask.get(lockId));
                // 获取锁版本号
//                Integer versionNo = dataLockService.selectVersionNoByLockId(lockId);
//                lockTask.put(lockId, versionNo);
            }
            // 设置任务锁
            SessionUtil.setAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION, lockTask);
        }

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**
     * @param taskId
     * @param pmTaskTree
     * @return firstParentId
     * @Title: findFirstParentId
     * @Description 递归查找任务根节点Id
     */
    public String findFirstParentId(String taskId, PmTaskTree pmTaskTree) {
        String firstParentId = Constants.BLANK;
        pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
        if (pmTaskTree != null && ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            firstParentId = pmTaskTree.getTaskId();
            return firstParentId;
        } else if (pmTaskTree != null && !ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            return findFirstParentId(pmTaskTree.getParentTaskId(), pmTaskTree);
        } else {
            return firstParentId;
        }
    }
}
