package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300311SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300311SaveOutput;
import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.entity.PmTaskTree;
import com.nationrel.modules.project.jar.action.MessageSupport;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTaskTreeMapper;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300311SaveService;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300311SaveServiceImpl
 * @Description 任务保存初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300311SaveServiceImpl implements Rb0300311SaveService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgMapper pmTgMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务保存初始化的业务方法
     */
    @Override
    public String perform(Rb0300311SaveInput input, Rb0300311SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 取得tg标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 根据输入值查询表中是否已存在
        String taskName = input.getTaskName();
        int repCount = pmTaskMapper.countTaskNameByTgId(tgId, taskName, null);

        // 判断当前任务名称是否已经存在
        if (repCount > 0) {
            throw new ApplicationRuntimeException(MessageId.E300129,
                    new String[]{taskName, pmTgMapper.selectByPrimaryKey(tgId).getTgName()});
        }

        if (StringUtils.isEmpty(input.getParentTaskId())) {

            // 页面TG下级权限控制
            Map<String, String> paramMap = new HashMap<String, String>();
            paramMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
            paramMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
            paramMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
            paramMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, tgId);
            paramMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TG_TYPE);
            paramMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
            paramMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

            // 取得具体资源的权限
            Map<String, String> authMap = authorityCtrlService.doResourceAuthority(paramMap);
            String createAuth = authMap.get(ReliaFlowConstants.AUTHORITY_CREATE);

            // 权限发生变化-操作不能进行
            if (StringUtils.isNotEmpty(createAuth)) {
                throw new OptimisticLockException(MessageId.W009805);
            }

            // 时间整合性检查
            pmTaskService.checkTimeValid(input.getPlanStartTime(), input.getPlanEndTime(), projectId, tgId, null, ReliaFlowConstants.TASK_FIRST_PARENT_ID);

            // 创建新任务
            createTask(input, tgId);
        } else {
            PmTaskTree pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(input.getParentTaskId());
            if (pmTaskTree == null) {
                throw new OptimisticLockException(MessageId.E300130);
            }
            // 时间整合性检查
            pmTaskService.checkTimeValid(input.getPlanStartTime(), input.getPlanEndTime(), projectId, tgId, null, input.getParentTaskId());

            // 根据任务子节点ID查找跟节点ID
            output.setParentTaskId(input.getParentTaskId());
            String firstParentId = Constants.BLANK;
            if (pmTaskTree != null && ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
                firstParentId = pmTaskTree.getTaskId();
            } else if (pmTaskTree != null && !ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
                firstParentId = findFirstParentId(pmTaskTree.getParentTaskId(), pmTaskTree);
            }
            if (StringUtils.isEmpty(firstParentId)) {
                // 数据已经被删除的异常
                throw new OptimisticLockException(MessageId.W009803);
            }

            // 页面权限控制
            Map<String, String> argMap = new HashMap<String, String>();
            argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
            argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
            argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
            argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, input.getParentTaskId());
            argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
            argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
            argMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

            // 取得具体资源的权限
            Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
            String addTaskAuth = authMap.get(ReliaFlowConstants.AUTHORITY_EXT01);

            // 权限发生变化-操作不能进行
            if (StringUtils.isNotEmpty(addTaskAuth)) {
                throw new OptimisticLockException(MessageId.W009805);
            }

            //================================
            // 任务删除排他处理
            //================================
            @SuppressWarnings("unchecked")
            Map<String, Integer> lockTask = (Map<String, Integer>) SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION);
            String lockId = ReliaFlowConstants.PJ_TASK_PAGE_NUMBER + firstParentId;
            Integer oldVersionNo = lockTask.get(lockId);
            // 追加新子任务
            addChildTask(input, tgId);
        }

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**

     * @param input
     * @param tgId
     * @Title: addChildTask
     * @Description: 追加新子任务
     */
    private void addChildTask(Rb0300311SaveInput input,
                              String tgId) {

        // TASK信息表
        String taskId = insertTask(input);

        //=====================
        // 采用左右键值维护树型关系-[当前节点]增加[子节点]时的操作
        // 1、[当前节点]如果是叶子节点、修改为非叶子节点;
        // 2、[子节点]的左键值等于[当前节点]的右键值;
        //     [子节点]的是叶子节点;
        //     [子节点]的深度等于所有父节点的个数;select count(*) from tree where leftNo <= [当前节点].leftNo and rightNo >= [当前节点].rightNo
        // 3、[子节点]追加后所有其它节点的左右键值修正;
        //=====================
        // TASK关系表
        PmTaskTree parentEntity = pmTaskTreeMapper.selectByPrimaryKey(input.getParentTaskId());

        // 1、[当前节点]如果是叶子节点、修改为非叶子节点;
        if (ReliaFlowConstants.TREE_LEAF.equals(parentEntity.getType())) {
            parentEntity.setType(ReliaFlowConstants.TREE_NOT_LEAF);
            pmTaskTreeMapper.updateByPrimaryKey(parentEntity);
        }

        // 3、[子节点]追加后所有其它节点的左右键值修正;
        pmTaskTreeMapper.updateTaskTreeLeftNo(parentEntity.getRightNo(), parentEntity.getOrdinal(), tgId);
        pmTaskTreeMapper.updateTaskTreeRightNo(parentEntity.getRightNo(), parentEntity.getOrdinal(), tgId);

        // 2、[子节点]的左键值等于[当前节点]的右键值;
        //     [子节点]的是叶子节点;
        //     [子节点]的深度等于所有父节点的个数;select count(*) from tree where leftNo <= [当前节点].leftNo and rightNo >= [当前节点].rightNo
        Integer nodeDeep = pmTaskTreeMapper.selectTaskNodeDeep(parentEntity.getLeftNo(), parentEntity.getRightNo(), parentEntity.getOrdinal(), tgId);
        // 基本信息
        PmTaskTree pmTaskTree = new PmTaskTree();
        pmTaskTree.setTaskId(taskId);
        pmTaskTree.setParentTaskId(input.getParentTaskId());
        pmTaskTree.setTgId(tgId);
        pmTaskTree.setDeep(nodeDeep);
        pmTaskTree.setLeftNo(parentEntity.getRightNo());
        pmTaskTree.setRightNo(parentEntity.getRightNo() + Constants.C_NUM_1);
        pmTaskTree.setType(ReliaFlowConstants.TREE_LEAF);
        pmTaskTree.setOrdinal(parentEntity.getOrdinal());
        pmTaskTreeMapper.insert(pmTaskTree);
    }

    /**

     * @param input
     * @param tgId
     * @Title: createTask
     * @Description: 创建新任务(STEP级)
     */
    private void createTask(Rb0300311SaveInput input, String tgId) {

        // TASK信息表
        String taskId = insertTask(input);

        // TASK关系表
        // 基本信息
        PmTaskTree pmTaskTree = new PmTaskTree();
        pmTaskTree.setTaskId(taskId);
        pmTaskTree.setParentTaskId(ReliaFlowConstants.TASK_FIRST_PARENT_ID);
        pmTaskTree.setTgId(tgId);
        pmTaskTree.setDeep(Constants.C_NUM_1); // 根节点 层次为1
        pmTaskTree.setLeftNo(Constants.C_NUM_1); // 左序号为1
        pmTaskTree.setRightNo(Constants.C_NUM_2); // 右序号为2
        pmTaskTree.setType(ReliaFlowConstants.TREE_LEAF); //叶子节点
        pmTaskTree.setOrdinal(pmTaskTreeMapper.selectMaxId(tgId)); // STEP下的并列树
        pmTaskTreeMapper.insert(pmTaskTree);
    }

    /**

     * @param input
     * @param staffCount void
     * @Title: insertTask
     * @Description: 任务的创建
     */
    private String insertTask(Rb0300311SaveInput input) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 基本信息
        PmTask pmTask = new PmTask();
        pmTask.setTaskName(input.getTaskName());
        pmTask.setPlanStartTime(input.getPlanStartTime());
        pmTask.setPlanEndTime(input.getPlanEndTime());
        pmTask.setOrigin(ReliaFlowConstants.TASK_ORIGIN_MANUAL);
        pmTask.setLifeState(ReliaFlowConstants.LIFECYCLE_RELIA_TASK_CREATE);
        pmTask.setShareScale(new BigDecimal(input.getShareScale()));
        pmTask.setDescription(input.getDescription());
        // 系统信息
        pmTask.setInsertUserId(sysUser.getUsername());
        pmTask.setInsertPgmId(sysUser.getPgmId());
        pmTaskMapper.insert(pmTask);
        return pmTask.getTaskId();
    }

    /**
     * @param messageSupport
     * @param tgId
     * @param taskName
     * @return void
     * @Title: checkInputTaskName
     * @Description: 检查任务名称在同一个TG中是否已经存在
     */
    @Override
    public void checkInputTaskName(MessageSupport messageSupport, String tgId, String taskName) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 根据输入值查询表中是否已存在
        if (StringUtils.isNotEmpty(taskName)) {
            int repCount = pmTaskMapper.countTaskNameByTgId(tgId, taskName, null);
            // 判断当前项目名称是否已经存在
            if (repCount > 0) {
                String[] args =
                        new String[]{messageSupport.getText(ReliaFlowCheckPageParamConstants.RELIAFLOW_LABEL_TASKNAME)};
                messageSupport.addErrorMessage(MessageId.E300129, args);
            }
        }
    }

    /**
     * @param taskId
     * @param pmTaskTree
     * @param firstParentId
     * @return firstParentId
     * @Title: findFirstParentId
     * @Description 递归查找任务根节点Id
     */
    private String findFirstParentId(String taskId, PmTaskTree pmTaskTree) {
        String firstParentId = Constants.BLANK;
        pmTaskTree = pmTaskTreeMapper.selectByPrimaryKey(taskId);
        if (pmTaskTree != null && ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            firstParentId = pmTaskTree.getTaskId();
            return firstParentId;
        } else if (pmTaskTree != null && !ReliaFlowConstants.TASK_FIRST_PARENT_ID.equals(pmTaskTree.getParentTaskId())) {
            return findFirstParentId(pmTaskTree.getParentTaskId(), pmTaskTree);
        } else {
            return firstParentId;
        }
    }
}
