package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300312InitInput;
import com.nationrel.modules.project.dto.output.Rb0300312InitOutput;
import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300312InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300312InitServiceImpl
 * @Description 任务编辑初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300312InitServiceImpl implements Rb0300312InitService {

    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务编辑初始化的业务方法
     */
    @Override
    public String perform(Rb0300312InitInput input, Rb0300312InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得project标识
        String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(input.getTaskId());

        //======================
        // 任务信息编辑页面设定
        //======================
        // 当前任务取得
        PmTask pmTask = pmTaskMapper.selectByPrimaryKey(input.getTaskId());
        // 设置版本号
        SessionUtil.setAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION, pmTask.getVersionNo());
        output.setTaskId(pmTask.getTaskId());
        output.setTaskName(pmTask.getTaskName());
        output.setPlanStartTime(pmTask.getPlanStartTime());
        output.setPlanEndTime(pmTask.getPlanEndTime());
        output.setStartTime(pmTask.getStartTime());
        output.setEndTime(pmTask.getEndTime());
        output.setPercent(pmTask.getPercent());
        output.setShareScale(pmTask.getShareScale());
        output.setDescription(pmTask.getDescription());
        if (input.getParentTaskId() != null) {
            output.setParentTaskId(input.getParentTaskId());
        }

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, pmTask.getTaskId());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.BASE_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doAuthorityByPage(argMap);
        output.setAuthMap(authMap);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
