package com.nationrel.modules.project.service.impl;


import com.nationrel.modules.project.dto.input.Rb0300312SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300312SaveOutput;
import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.entity.PpPlan;
import com.nationrel.modules.project.entity.PpPlanTask;
import com.nationrel.modules.project.entity.PtTrace;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300312SaveService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300312SaveServiceImpl
 * @Description 任务编辑初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300312SaveServiceImpl implements Rb0300312SaveService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PtTraceMapper ptTraceMapper;
    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PmTgMapper pmTgMapper;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务编辑初始化的业务方法
     */
    @Override
    public String perform(Rb0300312SaveInput input, Rb0300312SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(input.getTaskId());

        // 根据输入值查询表中是否已存在
        String taskName = input.getTaskName();
        int repCount = pmTaskMapper.countTaskNameByTgId(tgId, taskName, input.getTaskId());

        // 判断当前任务名称是否已经存在
        if (repCount > 0) {
            throw new ApplicationRuntimeException(MessageId.E300129,
                    new String[]{taskName, pmTgMapper.selectByPrimaryKey(tgId).getTgName()});
        }

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, input.getTaskId());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.CHILD_TYPE);
        argMap.put(PmMapKeySet.KEY_S_SEARCH_ROLE, ReliaFlowConstants.IS_SEARCH_YES);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        String editAuth = authMap.get(ReliaFlowConstants.AUTHORITY_MODIFY);

        // 权限发生变化-操作不能进行
        if (StringUtils.isNotEmpty(editAuth)) {
            throw new OptimisticLockException(MessageId.W009805);
        }

        // TASK信息表
        PmTask pmTask = pmTaskMapper.selectByPrimaryKey(input.getTaskId());

        // 任务基本信息编辑排它处理
        Integer versionLock = Integer.parseInt(StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_BASIC_EDIT_VERSION)));
        if (pmTask == null || !versionLock.equals(pmTask.getVersionNo())) {
            // 抛出版本不一致的异常
            throw new OptimisticLockException(MessageId.W009801);
        }

        // 时间有变动，时间整合性检查
        if (DateUtils.isNotDateEquals(pmTask.getPlanStartTime(), input.getPlanStartTime()) ||
                DateUtils.isNotDateEquals(pmTask.getPlanEndTime(), input.getPlanEndTime())) {
            pmTaskService.checkTimeValid(input.getPlanStartTime(), input.getPlanEndTime(), projectId, tgId, pmTask.getTaskId(), input.getParentTaskId());
        }

        BigDecimal percentFromDB = pmTask.getPercent() == null ? new BigDecimal(0) : pmTask.getPercent();
        // 基本信息
        pmTask.setTaskName(input.getTaskName());
        pmTask.setPlanStartTime(input.getPlanStartTime());
        pmTask.setPlanEndTime(input.getPlanEndTime());
        pmTask.setStartTime(input.getStartTime());
        pmTask.setEndTime(input.getEndTime());
        pmTask.setPercent(input.getPercent());
        pmTask.setShareScale(input.getShareScale());
        pmTask.setDescription(input.getDescription());
        // 系统信息
        pmTask.setUpdateUserId(sysUser.getUsername());
        pmTask.setUpdatePgmId(sysUser.getPgmId());

        pmTaskMapper.updateByPrimaryKey(pmTask);

        // 如果完成度有变动&&任务来源不为空,则生成风险系数追踪数据
        BigDecimal percentFromClient = input.getPercent() == null ? new BigDecimal(0) : input.getPercent();
        if (percentFromClient.compareTo(percentFromDB) != 0 && pmTask.getOriginId() != null) {
            generateRiskFactorsTrackData(projectId, pmTask.getOriginId());
        }

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**

     * @param input
     * @param output
     * @return
     * @throws Exception String
     * @Title: findAuthMap
     * @Description: 取得任务页面级权限
     */
    @Override
    public void findAuthMap(Rb0300312SaveInput input, Rb0300312SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, input.getTaskId());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.BASE_TYPE);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doAuthorityByPage(argMap);
        input.setAuthMap(authMap);
    }

    /**
     * @param originId void
     * @Title: generateRiskFactorsTrackData
     * @Description: 生成风险系数追踪数据
     */
    private void generateRiskFactorsTrackData(String projectId, String originId) throws Exception {

        // 1.获取预防性措施规划任务数据
        PpPlanTask ppPlanTask = ppPlanTaskMapper.selectByPrimaryKey(originId);
        if (ppPlanTask == null) {
            return;
        }

        // 取得最初期风险值
        PpPlan ppPlan = ppPlanMapper.selectByPrimaryKey(ppPlanTask.getPpPlanId());
        BigDecimal riskBeforeValue = ppPlan.getRiskBeforeValue();


        // 2.取得当前时间下-所有规划任务累计风险降低值(按完成度累计)
        Map<String, String> map = ppPlanTaskMapper.statisticsPlanTaskRiskValueByPlanId(projectId, ppPlanTask.getPpPlanId());
        Object obj = map.get("RISK_VALUE");
        BigDecimal riskValue = new BigDecimal(obj.toString());
        riskBeforeValue = riskBeforeValue.subtract(riskValue);

        // 完成度更新日期
        String ppTime = DateUtils.convertDatetoString(new Date());

        // 同一组规划任务单天完成度有变化。追踪记录作成
        PtTrace ptTrace = new PtTrace();
        ptTrace.setPpPlanId(ppPlanTask.getPpPlanId());
        ptTrace.setPpTime(ppTime);
        ptTrace.setRiskValue(riskBeforeValue);

        // 3.判断是否存在，如果存在，则更新。否则插入
        PtTrace ptObj = ptTraceMapper.selectByPrimaryKey(ppPlanTask.getPpPlanId(), ppTime);
        if (ptObj == null) {
            ptTraceMapper.insert(ptTrace);
        } else {
            ptTraceMapper.updateByPrimaryKey(ptTrace);
        }
    }
}
