package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300330DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300330DeleteOutput;
import com.nationrel.modules.project.entity.PmTaskCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskCheckMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300330DeleteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300330DeleteServiceImpl
 * @Description TASK检查项删除的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300330DeleteServiceImpl implements Rb0300330DeleteService {

    @Autowired
    private PmTaskCheckMapper pmTaskCheckMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TASK检查项删除的业务方法
     */
    @Override
    public String perform(Rb0300330DeleteInput input, Rb0300330DeleteOutput output) throws Exception {

        // 取得Task标识
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 取得Task检查标识
        Integer taskCheckId = input.getTaskCheckId();

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 取出当前数据
        PmTaskCheck taskCheck = pmTaskCheckMapper.selectByPrimaryKey(taskCheckId, taskId);
        if (taskCheck == null) {
            // 抛出版本不一致的异常
            throw new OptimisticLockException(MessageId.W009803);
        }

        // 删除当前Task检查项
        pmTaskCheckMapper.deleteByPrimaryKey(taskCheckId, taskId);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
