package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300330SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300330SaveOutput;
import com.nationrel.modules.project.entity.PmTaskCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskCheckMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300330SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @version V1.0
 * @ClassName Rb0300330SaveServiceImpl
 * @Description TASK检查列表保存业务类
 * @Author ted.liu
 * @CreateDate 2013-12-19
 */
@Service
public class Rb0300330SaveServiceImpl implements Rb0300330SaveService {

    @Autowired
    private PmTaskCheckMapper pmTaskCheckMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description TASK检查列表保存业务方法
     */
    @Override
    public String perform(Rb0300330SaveInput input, Rb0300330SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 取得TASK标识
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 取得页面TASK检查列表
        List<PmTaskCheck> pmTaskChecks = input.getPmTaskCheck();

        //================================
        // 任务检查列表保存排他处理
        //================================
        /*String lockId = ReliaFlowConstants.PJ_TASK_CHECK_PAGE_NUMBER + taskId;
        Integer oldVersionNo = (Integer) SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_CHECK_SAVE_VERSION);
        dataLockService.checkDataLock(lockId, oldVersionNo);*/

        // TASK检查列表保存
        PmTaskCheck updateEntity = null;
        PmTaskCheck entity = null;

        if (pmTaskChecks != null && pmTaskChecks.size() > 0) {
            for (int i = 0; i < pmTaskChecks.size(); i++) {
                // 取得页面检查项实体
                entity = pmTaskChecks.get(i);
                // 根据主键从DB取得检查项目
                updateEntity = pmTaskCheckMapper.selectByPrimaryKey(entity.getTaskCheckId(), taskId);

                // 系统信息
                updateEntity.setInsertUserId(sysUser.getUsername());
                updateEntity.setInsertPgmId(sysUser.getPgmId());

                // 页面与DB检查结果一致
                if (ReliaFlowConstants.CHECK_RESULT_PASS.equals(entity.getResult())) {
                    if (ReliaFlowConstants.CHECK_RESULT_PASS.equals(updateEntity.getResult())) {
                        continue;
                    } else {
                        updateEntity.setResult(ReliaFlowConstants.CHECK_RESULT_PASS);
                        // 检查项保存数据库
                        pmTaskCheckMapper.updateByPrimaryKey(updateEntity);
                    }
                } else {
                    if (ReliaFlowConstants.CHECK_RESULT_INIT.equals(updateEntity.getResult())) {
                        continue;
                    } else {
                        updateEntity.setResult(ReliaFlowConstants.CHECK_RESULT_INIT);
                        // 检查项保存数据库
                        pmTaskCheckMapper.updateByPrimaryKey(updateEntity);
                    }
                }
            }
        }

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }
}
