package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300331SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300331SaveOutput;
import com.nationrel.modules.project.entity.PmTaskCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskCheckMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300331SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300231SaveServiceImpl
 * @Description 任务检查项创建业务类
 * @Author ted.liu
 * @CreateDate 2013-12-19
 */
@Service
public class Rb0300331SaveServiceImpl implements Rb0300331SaveService {

    @Autowired
    private PmTaskCheckMapper pmTaskCheckMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务检查项保存业务方法
     */
    @Override
    public String perform(Rb0300331SaveInput input, Rb0300331SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 取得任务标识
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 构建检查项实体
        PmTaskCheck entity = new PmTaskCheck();

        // 基本信息
        entity.setTaskCheckId(pmTaskCheckMapper.selectMaxId(taskId));
        entity.setTaskId(taskId);
        entity.setObject(input.getObject());
        entity.setContent(input.getContent());
        entity.setDescription(input.getDescription());
        entity.setResult(ReliaFlowConstants.CHECK_RESULT_INIT);

        // 系统信息
        entity.setInsertUserId(sysUser.getUsername());
        entity.setInsertPgmId(sysUser.getPgmId());

        // 检查项保存数据库
        pmTaskCheckMapper.insert(entity);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }
}
