package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300332InitInput;
import com.nationrel.modules.project.dto.output.Rb0300332InitOutput;
import com.nationrel.modules.project.entity.PmTaskCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskCheckMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300332InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300232InitServiceImpl
 * @Description 任务检查项编辑页面初期化的业务类
 * @Author ted
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300332InitServiceImpl implements Rb0300332InitService {

    @Autowired
    private PmTaskCheckMapper pmTaskCheckMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务检查项编辑页面初期化的业务方法
     */
    @Override
    public String perform(Rb0300332InitInput input, Rb0300332InitOutput output) throws Exception {
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 任务标识取得
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 取得任务检查标识
        Integer taskCheckId = input.getTaskCheckId();

        // 根据主键取得当前任务检查项
        PmTaskCheck pmTaskCheck = pmTaskCheckMapper.selectByPrimaryKey(taskCheckId, taskId);

        // 编辑页面元素值设定
        output.setTaskCheckId(pmTaskCheck.getTaskCheckId());
        output.setObject(pmTaskCheck.getObject());
        output.setContent(pmTaskCheck.getContent());
        output.setDescription(pmTaskCheck.getDescription());

        // 设置版本号
        SessionUtil.setAttribute(SessionKeys.RB_PJ_TASK_CHECK_VERSION, pmTaskCheck.getVersionNo());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
