package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300332SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300332SaveOutput;
import com.nationrel.modules.project.entity.PmTaskCheck;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskCheckMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300332SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300332SaveServiceImpl
 * @Description 任务检查项保存业务类
 * @Author ted
 * @CreateDate 2013-12-19
 */
@Service
public class Rb0300332SaveServiceImpl implements Rb0300332SaveService {

    @Autowired
    private PmTaskCheckMapper pmTaskCheckMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务检查项保存业务方法
     */
    @Override
    public String perform(Rb0300332SaveInput input, Rb0300332SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 任务标识取得
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 取得任务标识
        Integer taskCheckId = input.getTaskCheckId();

        // 构建检查项实体
        PmTaskCheck entity = pmTaskCheckMapper.selectByPrimaryKey(taskCheckId, taskId);

        // 基本信息
        entity.setObject(input.getObject());
        entity.setContent(input.getContent());
        entity.setDescription(input.getDescription());

        // 系统信息
        entity.setUpdateUserId(sysUser.getUsername());
        entity.setUpdatePgmId(sysUser.getPgmId());

        // 任务检查排它处理
        Integer versionLock = Integer.parseInt(StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_CHECK_VERSION)));
        if (entity == null || !versionLock.equals(entity.getVersionNo())) {
            // 抛出版本不一致的异常
            throw new OptimisticLockException(MessageId.W009801);
        }

        // 检查项保存数据库
        pmTaskCheckMapper.updateByPrimaryKey(entity);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }
}
