package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300340InitInput;
import com.nationrel.modules.project.dto.output.Rb0300340InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskStaffMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300340InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300340InitServiceImpl
 * @Description 任务成员列表页面初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300340InitServiceImpl implements Rb0300340InitService {

    @Autowired
    private PmTaskStaffMapper pmTaskStaffMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务列表初始化的业务方法
     */
    @Override
    public String perform(Rb0300340InitInput input, Rb0300340InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取数据
        String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
        String taskId = input.getTaskId();
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
        if (taskId == null || Constants.BLANK.equals(taskId)) {
            taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03003_TASK_ID, taskId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, taskId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.MEMBER_TYPE);
        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        output.setAuthMap(authMap);

        //===========================
        // 取得任务成员列表
        //===========================
        List<Map<String, Object>> pmTaskStaffs = pmTaskStaffMapper.selectPmTaskStaffsByTaskId(taskId, null);

        // 控制删除是否可见,符合当前login userId==staffId && 角色是Task负责人的成员不可以删除.
        int disableOperateCnt = 0;
        if (pmTaskStaffs != null && pmTaskStaffs.size() > 0) {
            for (Map<String, Object> pmTaskStaffMap : pmTaskStaffs) {
                String staffId = StringUtil.parseString(pmTaskStaffMap.get(PmMapKeySet.KEY_R_STAFF_ID));
                if (staffId.equals(sysUser.getUsername()) && ReliaFlowConstants.ROLE_TASK_MANAGER.compareTo(authMap.get(PmMapKeySet.KEY_S_ROLE_TYPE)) <= 0) {
                    pmTaskStaffMap.put(ReliaFlowConstants.AUTHORITY_MODIFY, ReliaFlowConstants.AUTH_CTRL_VIEW);
                    pmTaskStaffMap.put(ReliaFlowConstants.AUTHORITY_DELETE, ReliaFlowConstants.AUTH_CTRL_VIEW);
                    disableOperateCnt++;
                    break;
                }
            }
        }
        output.setPmTaskStaffs(pmTaskStaffs);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;

        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_MODIFY))) {
            ctrOptCount++;
        }
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }
        if (disableOperateCnt == pmTaskStaffs.size() && ctrOptCount > 0) {
            ctrOptCount = 0;
        }

        // 统计显示操作按钮的数量
        output.setCtrOptCount(ctrOptCount);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }
}
