package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300350DeleteInput;
import com.nationrel.modules.project.dto.output.Rb0300350DeleteOutput;
import com.nationrel.modules.project.entity.PmTaskDelivery;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskDeliveryMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300350DeleteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300350DeleteServiceImpl
 * @Description 任务交付删除业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300350DeleteServiceImpl implements Rb0300350DeleteService {

    @Autowired
    private PmTaskDeliveryMapper pmTaskDeliveryMapperMapper;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务交付删除业务方法
     */
    @Override
    public String perform(Rb0300350DeleteInput input, Rb0300350DeleteOutput output) throws Exception {
        // 取得TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));

        // 取得交付Id
        String deliveryId = input.getDeliveryId();
        String deliveryType = input.getDeliveryType();
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
        if (deliveryId == null || deliveryId.equals(Constants.BLANK) || taskId == null || taskId.equals(Constants.BLANK)) {
            return Constants.SERVICE_FAIL;
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 取出当前数据
        PmTaskDelivery pmTaskDelivery = pmTaskDeliveryMapperMapper.selectByPrimaryKey(deliveryId, taskId, deliveryType);
        if (pmTaskDelivery == null) {
            // 抛出版本不一致的异常
            throw new OptimisticLockException(MessageId.W009803);
        }

        // 删除Task交付物
        pmTaskDeliveryMapperMapper.deleteDeliveryByTaskIdAndDeliveryId(taskId, deliveryId, deliveryType);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }
}
