package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300350InitInput;
import com.nationrel.modules.project.dto.output.Rb0300350InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskDeliveryMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300350InitService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300350InitServiceImpl
 * @Description 任务交付列表页面初始化业务类
 * @Author andy
 * @CreateDate 2013-12-24
 */
@Service
public class Rb0300350InitServiceImpl implements Rb0300350InitService {

    @Autowired
    private PmTaskDeliveryMapper pmTaskDeliveryMapper;
    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTaskService pmTaskService;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 任务交付列表页面初始化业务方法
     */
    @Override
    public String perform(Rb0300350InitInput input, Rb0300350InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取参数
        String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));
        String taskId = input.getTaskId();
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
        if (taskId == null) {
            taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03003_TASK_ID, taskId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 根据taskId获取交付列表
        List<Map<String, Object>> deliveryList = pmTaskDeliveryMapper.searchDeliveryListByTaskId(taskId);

        // 页面权限控制
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, taskId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.DOCUMENT_TYPE);
        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);

        // 统计显示操作按钮的数量
        int ctrOptCount = 0;

        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_VIEW))) {
            ctrOptCount++;
        }
        if (StringUtils.isEmpty(authMap.get(ReliaFlowConstants.AUTHORITY_DELETE))) {
            ctrOptCount++;
        }

        // 统计显示操作按钮的数量
        output.setCtrOptCount(ctrOptCount);

        // 设置输出
        output.setDeliveryList(deliveryList);
        output.setAuthMap(authMap);

        // 处理结果
        return Constants.SERVICE_SUCCESS;
    }

}
