package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300360CheckInput;
import com.nationrel.modules.project.dto.output.Rb0300360CheckOutput;
import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.service.PmStageCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300360CheckService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0300360CheckServiceImpl
 * @Description Task阶段检查的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300360CheckServiceImpl implements Rb0300360CheckService {

    @Autowired
    private PmStageCtrlService pmStageCtrlService;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTgService pmTgService;

    /**
     
     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description Task阶段检查的业务方法
     */
    @Override
    public String perform(Rb0300360CheckInput input, Rb0300360CheckOutput output) throws Exception {
        // TG标识
        String tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
        // Task标识
        String taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
        // Task生命周期
        String prevLifeState = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB_PJ_TASK_LIFE_STATE_VERSION));
        // 页面设定
        input.setTaskId(taskId);

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        PmTask pmTask = pmTaskMapper.selectByPrimaryKey(taskId);

        if (pmTask == null) {
            throw new OptimisticLockException(MessageId.W300104);
        }
        if (!prevLifeState.equals(pmTask.getLifeState())) {
            throw new OptimisticLockException(MessageId.W009804);
        }

        // 生命周期推移检查
        pmStageCtrlService.ctrlTaskState(tgId, taskId);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
