package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300360InitInput;
import com.nationrel.modules.project.dto.output.Rb0300360InitOutput;
import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.jar.rb_common.bizcommon.constant.BizModuleCode;
import com.nationrel.modules.project.jar.rb_common.common.constant.*;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.service.AuthorityCtrlService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300360InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300360InitServiceImpl
 * @Description Task步骤阶段初始化的业务类
 * @Author ted.liu
 * @CreateDate 2013-12-18
 */
@Service
public class Rb0300360InitServiceImpl implements Rb0300360InitService {

    @Autowired
    private AuthorityCtrlService authorityCtrlService;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmTgService pmTgService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description Task步骤阶段初始化的业务方法
     */
    @Override
    public String perform(Rb0300360InitInput input, Rb0300360InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 取得项目标识
        String projectId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID));

        // 取得Task标识
        String taskId = input.getTaskId();
        if (taskId == null || Constants.BLANK.equals(taskId)) {
            taskId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03003_TASK_ID));
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03003_TASK_ID, taskId);
        }

        // 取得TG标识
        String tgId = input.getTgId();
        if (tgId == null || Constants.BLANK.equals(tgId)) {
            tgId = StringUtil.parseString(SessionUtil.getAttribute(SessionKeys.RB03002_TG_ID));
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03002_TG_ID, tgId);
        }

        // 检查TG是否存在
        pmTgService.checkTgExist(tgId);

        // 检查任务是否存在
        PmTask pmTask = pmTaskMapper.selectByPrimaryKey(taskId);

        if (pmTask == null) {
            SessionUtil.removeAttribute(SessionKeys.RB_PJ_TASK_LIFE_STATE_VERSION);
            throw new OptimisticLockException(MessageId.W300104);
        }

        // 页面项目值设定
        SessionUtil.setAttribute(SessionKeys.RB_PJ_TASK_LIFE_STATE_VERSION, pmTask.getLifeState());
        output.setTaskId(taskId);
        output.setModuleId(BizModuleCode.MODULE_ID_TASK);

        //========================================
        // 页面权限控制
        // key_LifeState&_key_ResourceType&key_ChildResourceType-StateAuthority
        // key_RoleType&_key_ResourceType&key_ChildResourceType-RoleAuthority
        //========================================
        Map<String, String> argMap = new HashMap<String, String>();
        argMap.put(PmMapKeySet.KEY_S_PROJECT_ID, projectId);
        argMap.put(PmMapKeySet.KEY_S_USER_ID, sysUser.getUsername());
        argMap.put(PmMapKeySet.KEY_S_TG_ID, tgId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_ID, taskId);
        argMap.put(PmMapKeySet.KEY_S_RESOURCE_TYPE, ReliaFlowConstants.RESOURCE_TASK_TYPE);
        argMap.put(PmMapKeySet.KEY_S_CHILD_RESOURCE_TYPE, ReliaFlowConstants.STAGE_TYPE);
        argMap.put(PmMapKeySet.KEY_S_AUTHORITY_SWITCH, ReliaFlowConstants.IS_AUTHORITY_SWITCH_NO);

        // 取得具体资源的权限
        Map<String, String> authMap = authorityCtrlService.doResourceAuthority(argMap);
        output.setAuthMap(authMap);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
