package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300370InitInput;
import com.nationrel.modules.project.dto.output.Rb0300370InitOutput;
import com.nationrel.modules.project.entity.TaskToolModel;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_platform.entity.SysTool;
import com.nationrel.modules.project.mapper.PmTaskToolMapper;
import com.nationrel.modules.project.mapper.SysToolMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300370InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0300370InitServiceImpl
 * @Description 工具添加
 * @Author 孔徐文
 * @CreateDate 2014-4-28
 */
@Service
public class Rb0300370InitServiceImpl implements Rb0300370InitService {

    @Autowired
    private PmTaskToolMapper pmTaskToolMapper;
    @Autowired
    private SysToolMapper sysToolMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 工具添加初始化的业务方法
     */
    @Override
    public String perform(Rb0300370InitInput input, Rb0300370InitOutput output) throws Exception {

        String taskId = input.getTaskId();

        // 检查任务是否存在
        pmTaskService.checkTaskExist(taskId);

        // 可靠性工具
        List<SysTool> sysToolList = sysToolMapper.selectSysTools();

        // 根据任务标识取得任务使用工具
        List<TaskToolModel> pmTaskToolList = pmTaskToolMapper.selectTaskToolModelByTaskId(taskId);

        //======================
        // 工具列表
        //======================
        List<SysTool> taskToolsCreate = new ArrayList<SysTool>();
        // 构建查找任务工具Map
        Map<String, String> taskToolMap = new HashMap<String, String>();
        for (TaskToolModel taskTool : pmTaskToolList) {
            taskToolMap.put(taskTool.getToolId(), taskTool.getToolId());
        }
        // 编辑页面工具列表和任务工具列表
        for (SysTool tool : sysToolList) {
            if (taskToolMap.containsKey(tool.getToolId())) {
                taskToolsCreate.add(tool);
            }
        }
        sysToolList.removeAll(taskToolsCreate);

        // 输出
        output.setSysToolList(sysToolList);
        output.setPmTaskToolList(pmTaskToolList);
        output.setTaskId(taskId);
        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
