package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0300370SaveInput;
import com.nationrel.modules.project.dto.output.Rb0300370SaveOutput;
import com.nationrel.modules.project.entity.PmTaskTool;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.mapper.PmTaskToolMapper;
import com.nationrel.modules.project.mapper.SysToolMapper;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import com.nationrel.modules.project.service.Rb0300370SaveService;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


/**
 * @version V1.0
 * @ClassName Rb0300370SaveServiceImpl
 * @Description 工具添加
 * @Author 孔徐文
 * @CreateDate 2014-4-28
 */
@Service
public class Rb0300370SaveServiceImpl implements Rb0300370SaveService {

    @Autowired
    private PmTaskToolMapper pmTaskToolMapper;
    @Autowired
    private SysToolMapper sysToolMapper;
    @Autowired
    private PmTgService pmTgService;
    @Autowired
    private PmTaskService pmTaskService;

    /**

     * @param input    输入参数
     * @param output   输出参数
     * @return 操作结果
     * @Title: perform
     * @Description 工具添加初始化的业务方法
     */
    @Override
    public String perform(Rb0300370SaveInput input, Rb0300370SaveOutput output) throws Exception {
        String taskId = input.getTaskId();

        // 检查任务是否存在
        pmTaskService.checkTaskExist(input.getTaskId());

        // 更新工具关联信息
        insertTaskTool(input);

        // 输出
        output.setTaskId(taskId);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

    /**

     * @param input
     * @return void
     * @Title: insertTaskTool
     * @Description: Task关联工具追加
     */
    private void insertTaskTool(Rb0300370SaveInput input) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 原关联工具的清除
        pmTaskToolMapper.deleteTaskToolByTaskId(input.getTaskId());

        // 页面新关联工具的追加
        if (StringUtils.isNotEmpty(input.getTools())) {

            String[] tools = input.getTools().split(Constants.COMMA);
            for (int i = 0; i < tools.length; i++) {
                PmTaskTool pmTaskTool = new PmTaskTool();

                // 基本信息
                pmTaskTool.setTaskId(input.getTaskId());
                pmTaskTool.setToolId(tools[i]);

                // 系统信息
                pmTaskTool.setInsertUserId(sysUser.getUsername());
                pmTaskTool.setInsertPgmId(sysUser.getPgmId());

                pmTaskToolMapper.insert(pmTaskTool);
            }
        }
    }
}
