package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405201InitInput;
import com.nationrel.modules.project.dto.output.Rb0405201InitOutput;
import com.nationrel.modules.project.entity.PpPlanPageModel;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.service.Rb0405201InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @version V1.0
 * @ClassName Rb0405201InitServiceImpl
 * @Description 预防性措施管理初始化业务接口
 * @Author andy
 * @CreateDate 2014-04-22
 */
@Service
public class Rb0405201InitServiceImpl implements Rb0405201InitService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405201InitInput input, Rb0405201InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String projectId = input.getProjectId();
        if (projectId == null || projectId.equals(Constants.BLANK)) {
            projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        }

        // 根据可靠性项目标识获取预防性措施管理数据
        List<PpPlanPageModel> ppPlanList = ppPlanMapper.findPpPlanListByProjectId(projectId);

        //============================================================
        // 预防性措施管理页面,权限控制如下:
        //    1.所有人（当前项目成员）都可以创建预防性措施规划.
        //    2.对于规划中的数据,所有者或者项目经理或者责任人可编辑所有PP内容;除此之外,只允许查看.
        //    3.规划完成审批中&规划完成,所有人都只能查看.
        //=============================================================
        if (ppPlanList != null && ppPlanList.size() > 0) {
            for (PpPlanPageModel item : ppPlanList) {
                // 当前预防性措施规划对象处于工作中状态时
                if (item.getLifeState() != null && item.getLifeState().equals(ReliaFlowConstants.LIFECYCLE_RELIA_PLAN_CREATE)) {

                    // 当前用户是当前project的所有者或者项目经理
                    if (Constants.C_NUM_1 == pmProjectStaffMapper.checkCurrUserIsProjectManager(sysUser.getUsername(), projectId, ReliaFlowConstants.PROJECT_OWNER_ID, ReliaFlowConstants.PROJECT_MANAGER_ID)) {
                        item.setModifyPlanAuth(true);
                        continue;
                    }

                    // 当前用户是当前预防性措施规划的责任人
                    if (item.getResponsibleId().equals(sysUser.getUsername())) {
                        item.setModifyPlanAuth(true);
                        continue;
                    }
                }
            }
        }

        // 设置输出
        output.setProactivePlanList(ppPlanList);

        output.setProjectId(input.getProjectId());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
