package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405202InitInput;
import com.nationrel.modules.project.dto.output.Rb0405202InitOutput;
import com.nationrel.modules.project.entity.PaAnalysis;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PaAnalysisMapper;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.service.Rb0405202InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @version V1.0
 * @ClassName Rb0405202InitServiceImpl
 * @Description 预防性措施规划创建初始化业务实现类
 * @Author andy
 * @CreateDate 2014-04-22
 */
@Service
public class Rb0405202InitServiceImpl implements Rb0405202InitService {

    @Autowired
    private PpPlanMapper ppProactivePlanMapper;
    @Autowired
    private PaAnalysisMapper paAnalysisMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405202InitInput input, Rb0405202InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 根据proactiveId获取记录
        PaAnalysis paProactiveAnalysispa = paAnalysisMapper.selectByPrimaryKey(input.getProactiveId());

        //=============================
        // 判断当前登录用户在是否有更改责任人的权限
        // 即：当前用户是当前项目的所有者或者项目经理
        //=============================
        String projectId = input.getProjectId();
        if (projectId == null || Constants.BLANK.equals(projectId)) {
            projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03001_PROJECT_ID, projectId);
        }
        int checkResult = pmProjectStaffMapper.checkCurrUserIsProjectManager(sysUser.getUsername(), projectId, ReliaFlowConstants.PROJECT_OWNER_ID, ReliaFlowConstants.PROJECT_MANAGER_ID);
        if (checkResult == Constants.C_NUM_1) {
            output.setModifyResponsibleAuth(true);
        }

        // 设置输出
        output.setResponsible(sysUser.getUsername());
        output.setProactiveName(paProactiveAnalysispa.getPaAnalysisName());
        output.setResponsibleId(sysUser.getUsername());
        output.setDeliverySrc(input.getDeliverySrc());
        output.setProjectId(input.getProjectId());
        output.setProactiveId(input.getProactiveId());
        output.setSrcId(input.getSrcId());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
