package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405202SaveInput;
import com.nationrel.modules.project.dto.output.Rb0405202SaveOutput;
import com.nationrel.modules.project.entity.PaAnalysis;
import com.nationrel.modules.project.entity.PpPlan;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PaAnalysisMapper;
import com.nationrel.modules.project.mapper.PaPartMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMapper;
import com.nationrel.modules.project.service.Rb0405202SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0405202SaveServiceImpl
 * @Description 预防性措施规划创建初始化业务实现类
 * @Author andy
 * @CreateDate 2014-04-22
 */
@Service
public class Rb0405202SaveServiceImpl implements Rb0405202SaveService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PaAnalysisMapper paAnalysisMapper;
    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PaPartMapper paPartMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405202SaveInput input, Rb0405202SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 获取预防性规划评估数据
        String paAnalysisId = input.getProactiveId();
        PaAnalysis paProactiveAnalysis = paAnalysisMapper.selectByPrimaryKey(paAnalysisId);

        // 设置预防性规划措施对象属性
        PpPlan ppPlanModel = new PpPlan();
        ppPlanModel.setPpPlanName(input.getProactivePlanName());
        ppPlanModel.setProjectId(projectId);
        ppPlanModel.setPaAnalysisId(paAnalysisId);
        ppPlanModel.setPaAnalysisName(input.getProactiveName());
        ppPlanModel.setPaAnalysisType(paProactiveAnalysis.getPaAnalysisType());
        ppPlanModel.setObjectCode(paProactiveAnalysis.getObjectCode());
        ppPlanModel.setObjectName(paProactiveAnalysis.getObjectName());
        ppPlanModel.setResponsible(input.getResponsibleId());
        ppPlanModel.setDescription(input.getDescription());
        ppPlanModel.setDeliverySourceType(input.getDeliverySrc());
        ppPlanModel.setDeliverySourceId(input.getSrcId());
        ppPlanModel.setInsertPgmId(sysUser.getPgmId());
        ppPlanModel.setInsertUserId(sysUser.getUsername());
        ppPlanModel.setDelFlag(Constants.DEFAULT_DEL_FLAG);
        ppPlanModel.setLifeState(ReliaFlowConstants.LIFECYCLE_RELIA_PLAN_CREATE);

        //===================
        // 执行前风险系数-执行后风险系数计算
        //===================
        Map<String, BigDecimal> paPartMaps = paPartMapper.selectRiskValue(paAnalysisId);
        ppPlanModel.setRiskBeforeValue(paPartMaps.get("RELIA_RISK_VALUE"));
        ppPlanModel.setRiskAfterValue(paPartMaps.get("RE_RISK_AFTER_VALUE"));

        // 1.插入预防性规划措施对象到[T_PP_PROACTIVE_PLAN]
        ppPlanMapper.insert(ppPlanModel);

        // 2.插入数据到[T_PP_PROACTIVE_PLAN_TASK]
        ppPlanTaskMapper.generateTasksAfterPpPlanCreated(paAnalysisId, ppPlanModel.getPpPlanId(), sysUser.getUsername(), sysUser.getPgmId());
        output.setPpPlanId(ppPlanModel.getPpPlanId());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
