package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405203InitInput;
import com.nationrel.modules.project.dto.output.Rb0405203InitOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PmProjectStaffMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.service.Rb0405203InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0405203InitServiceImpl
 * @Description 预防性措施规划编辑初始化业务实现类
 * @Author andy
 * @CreateDate 2014-04-24
 */
@Service
public class Rb0405203InitServiceImpl implements Rb0405203InitService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405203InitInput input, Rb0405203InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取预防性措施规划记录
        Map<String, Object> ppProactivePlan = ppPlanMapper.selectPlanInfo(input.getProactivePlanId());

        //=============================
        // 判断当前登录用户在是否有更改责任人的权限
        // 即：当前用户是当前项目的所有者或者项目经理
        //=============================
        String projectId = input.getProjectId();
        if (projectId == null || Constants.BLANK.equals(projectId)) {
            projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);
        } else {
            SessionUtil.setAttribute(SessionKeys.RB03001_PROJECT_ID, projectId);
        }
        int checkResult = pmProjectStaffMapper.checkCurrUserIsProjectManager(sysUser.getUsername(), projectId, ReliaFlowConstants.PROJECT_OWNER_ID, ReliaFlowConstants.PROJECT_MANAGER_ID);
        if (checkResult == Constants.C_NUM_1) {
            output.setModifyResponsibleAuth(true);
        }

        // 设置输出
        output.setResponsibleId(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_USER_ID)));
        output.setDeliverySrc(input.getDeliverySrc());
        output.setResponsible(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_USER_NAME)));
        output.setProactiveName(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PA_ANALYSIS_NAME)));
        output.setDescription(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_DESCRIPTION)));
        output.setProactivePlanId(input.getProactivePlanId());
        output.setProactivePlanName(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PP_PLAN_NAME)));

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
