package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405211CheckInput;
import com.nationrel.modules.project.dto.output.Rb0405211CheckOutput;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PaAnalysisMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMapper;
import com.nationrel.modules.project.service.Rb0405211CheckService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0405211InitServiceImpl
 * @Description 预防性措施规划状态迁移检查业务实现类
 * @Author andy
 * @CreateDate 2014-05-04
 */
@Service
public class Rb0405211CheckServiceImpl implements Rb0405211CheckService {

    @Autowired
    private PpPlanMapper ppProactivePlanMapper;
    @Autowired
    private PaAnalysisMapper paAnalysisMapper;
    @Autowired
    private PpPlanTaskMapper planTaskMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:预防性措施规划状态迁移检查业务方法
     */
    @Override
    public String perform(Rb0405211CheckInput input, Rb0405211CheckOutput output) throws Exception {

        String ppPlanId = null;
        if (input.getProactivePlanId() == null || input.getProactivePlanId().equals(Constants.BLANK)) {
            ppPlanId = (String) SessionUtil.getAttribute(SessionKeys.RB04052_PLAN_ID);
        } else {
            ppPlanId = input.getProactivePlanId();
        }

        // 当前结点向 完成状态迁移时-下级规划是否已全部完成
        MessageId retMsg = this.ctrlProactivePlanState(ppPlanId);
        if (retMsg != null) {
            // 项目阶段检查不通过
            return Constants.SERVICE_FAIL;
        }

        //返回结果
        return Constants.SERVICE_APPROVAL;
    }

    /**
     * @param ppPlanId
     * @return MessageId
     * @Title: ctrlProactivePlanState
     * @Description:
     */
    private MessageId ctrlProactivePlanState(String ppPlanId) {
        MessageId result = MessageId.E300128;
        Map<?, ?> map = planTaskMapper.countPpPlanIdAndPlanTGIdByPpPlanId(ppPlanId);
        if (map.get("SUM(DECODE(PLAN_TG_ID,NULL,0,1))").equals(map.get("COUNT(PP_PLAN_ID)"))) {
            return null;
        }
        return result;
    }
}
