package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405211InitInput;
import com.nationrel.modules.project.dto.output.Rb0405211InitOutput;
import com.nationrel.modules.project.entity.PaAnalysis;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.rb_common.common.dto.Pagination;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PaAnalysisMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMapper;
import com.nationrel.modules.project.service.Rb0405211InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0405211InitServiceImpl
 * @Description 预防性措施规划编辑(列表)业务实现类
 * @Author andy
 * @CreateDate 2014-04-24
 */
@Service
public class Rb0405211InitServiceImpl implements Rb0405211InitService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PaAnalysisMapper paAnalysisMapper;
    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
//    @Autowired
//    private SysUserMapper sysUserMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405211InitInput input, Rb0405211InitOutput output) throws Exception {

        String ppPlanId = null;
        if (input.getPpPlanId() == null || input.getPpPlanId().equals(Constants.BLANK)) {
            ppPlanId = (String) SessionUtil.getAttribute(SessionKeys.RB04052_PLAN_ID);
        } else {
            ppPlanId = input.getPpPlanId();
            SessionUtil.setAttribute(SessionKeys.RB04052_PLAN_ID, ppPlanId);
        }

        // 任务分页信息
        Pagination pagination = input.getPagination();
        if (pagination == null) {
            pagination = new Pagination();
        }

        // 获取预防性措施规划记录
        Map<String, Object> ppProactivePlan = ppPlanMapper.selectPlanInfo(ppPlanId);

        // 获取预防性评估记录
        String proactiveId = null;
        if (input.getProactiveId() == null || input.getProactiveId().equals(Constants.BLANK)) {
            proactiveId = StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PA_ANALYSIS_ID));
        } else {
            proactiveId = input.getProactiveId();
        }
        PaAnalysis paProactiveAnalysis = paAnalysisMapper.selectByPrimaryKey(proactiveId);

        // 取得记录总数
        int rowTotal = ppPlanTaskMapper.countTasksByPageCondition(pagination, ppPlanId);

        // 设置记录总数
        pagination.setRowTotal(rowTotal);

        // 获取预防性措施规划任务集合
        List<Map<String, Object>> ppProactivePlanTasks = ppPlanTaskMapper.getPpPlanTasksByPpPlanId(pagination, ppPlanId);

        // 设置输出
        output.setPagination(pagination);
        output.setResponsible(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_USER_NAME)));
        output.setPpProactivePlanTasks(ppProactivePlanTasks);
        output.setProactiveName(paProactiveAnalysis.getPaAnalysisName());
        output.setDescription(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_DESCRIPTION)));
        output.setProactivePlanId(ppPlanId);
        output.setProactivePlanName(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PP_PLAN_NAME)));

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
