package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405212InitInput;
import com.nationrel.modules.project.dto.output.Rb0405212InitOutput;
import com.nationrel.modules.project.entity.PaAnalysis;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.PaAnalysisMapper;
import com.nationrel.modules.project.mapper.PpPlanMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMapper;
import com.nationrel.modules.project.mapper.PpPlanTraceMapper;
import com.nationrel.modules.project.service.PpPlanService;
import com.nationrel.modules.project.service.Rb0405212InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0405212InitServiceImpl
 * @Description 预防性措施规划编辑(图形)业务实现类
 * @Author andy
 * @CreateDate 2014-04-30
 */
@Service
public class Rb0405212InitServiceImpl implements Rb0405212InitService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PaAnalysisMapper paAnalysisMapper;
    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PpPlanTraceMapper ppPlanTraceMapper;
    @Autowired
    private PpPlanService ppPlanServive;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405212InitInput input, Rb0405212InitOutput output) throws Exception {

        String ppPlanId = null;
        if (input.getProactivePlanId() == null || input.getProactivePlanId().equals(Constants.BLANK)) {
            ppPlanId = (String) SessionUtil.getAttribute(SessionKeys.RB04052_PLAN_ID);
        } else {
            ppPlanId = input.getProactivePlanId();
            SessionUtil.setAttribute(SessionKeys.RB04052_PLAN_ID, ppPlanId);
        }

        // 获取预防性措施规划记录
        Map<String, Object> ppProactivePlan = ppPlanMapper.selectPlanInfo(ppPlanId);

        // 获取预防性评估记录
        String proactiveId = null;
        if (input.getProactiveId() == null || input.getProactiveId().equals(Constants.BLANK)) {
            proactiveId = StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PA_ANALYSIS_ID));
        } else {
            proactiveId = input.getProactiveId();
        }
        PaAnalysis paProactiveAnalysis = paAnalysisMapper.selectByPrimaryKey(proactiveId);

        // 获取预防性措施规划作图数据并设置输出
        String[] planChartData = ppPlanServive.generateChatDataForPlan(ppPlanId);
        if (planChartData != null && planChartData.length > 0) {
            output.setDateRangeStr(planChartData[0]);
            output.setRiskDownValueRangeStr(planChartData[1]);
        }

        // 设置输出
        output.setResponsible(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_USER_NAME)));
        output.setProactiveName(paProactiveAnalysis.getPaAnalysisName());
        output.setDescription(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_DESCRIPTION)));
        output.setProactivePlanId(ppPlanId);
        output.setProactivePlanName(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PP_PLAN_NAME)));

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
