package com.nationrel.modules.project.service.impl;


import com.nationrel.modules.project.dto.input.Rb0405214InitInput;
import com.nationrel.modules.project.dto.output.Rb0405214InitOutput;
import com.nationrel.modules.project.entity.PaAnalysis;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.jar.util.StringUtil;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.PpPlanService;
import com.nationrel.modules.project.service.Rb0405214InitService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @version V1.0
 * @ClassName Rb0405214InitServiceImpl
 * @Description 预防性措施规划查看(图形)业务实现类
 * @Author andy
 * @CreateDate 2014-04-30
 */
@Service
public class Rb0405214InitServiceImpl implements Rb0405214InitService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PaAnalysisMapper paAnalysisMapper;
    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PtTraceMapper ptTraceTaskMapper;
    @Autowired
    private PpPlanService ppPlanServive;
    @Autowired
    private PmProjectStaffMapper pmProjectStaffMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405214InitInput input, Rb0405214InitOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String ppPlanId = null;
        if (input.getProactivePlanId() == null || input.getProactivePlanId().equals(Constants.BLANK)) {
            ppPlanId = (String) SessionUtil.getAttribute(SessionKeys.RB04052_PLAN_ID);
        } else {
            ppPlanId = input.getProactivePlanId();
            SessionUtil.setAttribute(SessionKeys.RB04052_PLAN_ID, ppPlanId);
        }

        // 获取预防性措施规划记录
        Map<String, Object> ppProactivePlan = ppPlanMapper.selectPlanInfo(ppPlanId);

        // 获取预防性评估记录
        String proactiveId = null;
        if (input.getProactiveId() == null || input.getProactiveId().equals(Constants.BLANK)) {
            proactiveId = StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PA_ANALYSIS_ID));
        } else {
            proactiveId = input.getProactiveId();
        }
        PaAnalysis paProactiveAnalysis = paAnalysisMapper.selectByPrimaryKey(proactiveId);

        // 获取图形所需数据并设置到输出
        String[] planChartData = ppPlanServive.generateChatDataForPlan(ppPlanId);
        if (planChartData != null && planChartData.length > 0) {
            output.setDateRangeStr(planChartData[0]);
            output.setRiskDownValueRangeStr(planChartData[1]);
        }

        //==================================
        //控制生命周期控件显示与否
        //  当且仅当满足:当前数据处于规划中&&(当前用户是项目所有者||项目经理||责任人)以外的人
        //  这个条件时,生命周期控件不显示
        //==================================
        if (StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_R_LIFE_STATE)).equals(ReliaFlowConstants.LIFECYCLE_RELIA_PLAN_CREATE)) {
            int checkResult = pmProjectStaffMapper.checkCurrUserIsProjectManager(sysUser.getUsername(), (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID), ReliaFlowConstants.PROJECT_OWNER_ID, ReliaFlowConstants.PROJECT_MANAGER_ID);
            if (checkResult != Constants.C_NUM_1 || !StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_RESPONSIBLE)).equals(sysUser.getUsername())) {
                output.setModifyResponsibleAuth(true);
            }
        }

        // 设置输出
        output.setResponsible(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_USER_NAME)));
        output.setProactiveName(paProactiveAnalysis.getPaAnalysisName());
        output.setDescription(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_DESCRIPTION)));
        output.setProactivePlanId(ppPlanId);
        output.setProactivePlanName(StringUtil.parseString(ppProactivePlan.get(PmMapKeySet.KEY_PP_PLAN_NAME)));

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
