package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.dto.input.Rb0405231InitInput;
import com.nationrel.modules.project.dto.output.Rb0405231InitOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.entity.PpPlanTask;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.common.constant.SessionKeys;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.jar.util.SessionUtil;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMapper;
import com.nationrel.modules.project.mapper.PpPlanTaskMemberMapper;
import com.nationrel.modules.project.service.Rb0405231InitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @version V1.0
 * @ClassName Rb0405231InitServiceImpl
 * @Description 预防性措施规划任务编辑初始化业务实现类
 * @Author andy
 * @CreateDate 2014-04-24
 */
@Service
public class Rb0405231InitServiceImpl implements Rb0405231InitService {

    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PpPlanTaskMemberMapper ppPlanTaskMemberMapper;
    @Autowired
    private PmTgMapper pmTgMapper;
//    @Autowired
//    private SysUserMapper sysUserMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405231InitInput input, Rb0405231InitOutput output) throws Exception {

        // 获取projectId
        String projectId = (String) SessionUtil.getAttribute(SessionKeys.RB03001_PROJECT_ID);

        // 根据PlanTask标识获取预防性规划任务信息
        PpPlanTask ppPlanTask = ppPlanTaskMapper.selectByPrimaryKey(input.getPlanTaskId());

        // 根据ProjectId获取TG集合
        List<PmTg> pmTgList = pmTgMapper.selectPmTgByProjectId(projectId);

        // 根据预防性措施规划类型,构造出最终的TG集合
        List<PmTg> finalPmTgList = new ArrayList<PmTg>();
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(ppPlanTask.getPlanTgId());
        if (pmTg != null) {
            output.setTgId(pmTg.getTgId());
        }

        // 当前任务所属的预防性规划来源于项目交付
        if (pmTgList != null && pmTgList.size() > 0) {
            for (PmTg obj : pmTgList) {
                if (obj.getLifeState().compareTo(ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK) <= 0) {
                    finalPmTgList.add(obj);
                }
            }
        }

        // 获取责任人
        /*PpPlanTaskMember ppPlanTaskMember = ppPlanTaskMemberMapper.selectByPpPlanTaskId(ppPlanTask.getPpPlanTaskId());
        if (ppPlanTaskMember != null) {
            SysUser sysUser = sysUserMapper.selectByPrimaryKey(ppPlanTaskMember.getUserId());
            output.setResponsible(sysUser.getUsername());
            output.setResponsibleId(sysUser.getUserId());
        }*/

        // 保存TG集合到Session
        SessionUtil.setAttribute(SessionKeys.RB_PJ_PROACTIVE_PLAN_TGLIST, finalPmTgList);

        // 设置输出
        output.setPartName(ppPlanTask.getPartName());
        output.setPaName(ppPlanTask.getPaName());
        output.setPlanStartTime(DateUtils.convertDatetoString(ppPlanTask.getPlanStartTime()));
        output.setPlanEndTime(DateUtils.convertDatetoString(ppPlanTask.getPlanEndTime()));
        output.setDescription(ppPlanTask.getDescription());
        output.setPlanTaskId(input.getPlanTaskId());
        output.setPlanTaskName(ppPlanTask.getPlanTaskName());
        output.setDescription(ppPlanTask.getDescription());
        output.setProactivePlanId(input.getProactivePlanId());

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }

}
