package com.nationrel.modules.project.service.impl;


import com.nationrel.modules.project.dto.input.Rb0405231SaveInput;
import com.nationrel.modules.project.dto.output.Rb0405231SaveOutput;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.entity.PpPlanTask;
import com.nationrel.modules.project.entity.PpPlanTaskMember;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.Rb0405231SaveService;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @version V1.0
 * @ClassName Rb0405231SaveServiceImpl
 * @Description 预防性措施规划任务编辑保存业务实现类
 * @Author andy
 * @CreateDate 2014-04-24
 */
@Service
public class Rb0405231SaveServiceImpl implements Rb0405231SaveService {

    @Autowired
    private PpPlanMapper ppPlanMapper;
    @Autowired
    private PpPlanTaskMemberMapper ppPlanTaskMemberMapper;
    @Autowired
    private PpPlanTaskMapper ppPlanTaskMapper;
    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PpPlanTraceMapper ppPlanTraceMapper;

    /**

     * @param input:输入参数
     * @param output:输出参数
     * @return String:返回结果
     * @Title: perform
     * @Description:
     */
    @Override
    public String perform(Rb0405231SaveInput input, Rb0405231SaveOutput output) throws Exception {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        // 获取预防性规划任务记录
        PpPlanTask ppPlanTask = ppPlanTaskMapper.selectByPrimaryKey(input.getPlanTaskId());
        Date planStartTime = DateUtils.convertDate(input.getPlanStartTime());
        Date planEndTime = DateUtils.convertDate(input.getPlanEndTime());

        // 获取TG信息
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(input.getTgId());

        // 检查其在当前选择的TG范围内是否重复
        int checkResult = ppPlanTaskMapper.checkTaskNameExistInTG(input.getPlanTaskName(), input.getTgId(), input.getPlanTaskId());
        if (checkResult != Constants.C_NUM_0) {
            throw new ApplicationRuntimeException(MessageId.E300129, new String[]{input.getPlanTaskName(), pmTg.getTgName()});
        }

        // 任务编辑的规划时间范围必在TG时间范围之内（TG时间有输入的场合）
        if (planStartTime.before(pmTg.getPlanStartTime()) || planStartTime.after(pmTg.getPlanEndTime())) {
            throw new ApplicationRuntimeException(MessageId.E300126);
        }
        if (planEndTime.after(pmTg.getPlanEndTime()) || planEndTime.before(pmTg.getPlanStartTime())) {
            throw new ApplicationRuntimeException(MessageId.E300127);
        }

        // 首次编辑
        if (ppPlanTask.getPlanStartTime() == null && ppPlanTask.getPlanEndTime() == null) {
            /*
             * 1、做成时间范围不存在的所有记录；
             * 2、更新对应记录；
             */
            ppPlanTraceMapper.insertTraceCaseAccordCondition(input.getProactivePlanId(), input.getPlanStartTime(), input.getPlanEndTime());
            double rickDownValue = ppPlanTraceMapper.getRickDownValueByDateRange(input.getPlanTaskId(), input.getPlanEndTime(), input.getPlanStartTime());
            ppPlanTraceMapper.updateRiskDownValueByPlanIdAndDateRange(ppPlanTask.getPpPlanId(), input.getPlanTaskId(), input.getPlanStartTime(), input.getPlanEndTime(), rickDownValue);
        } else {
            // 页面传入的规划开始&结束时间与规划任务修改之前的时间不一致时
            if (DateUtils.isNotDateEquals(ppPlanTask.getPlanStartTime(), planStartTime) ||
                    DateUtils.isNotDateEquals(ppPlanTask.getPlanEndTime(), planEndTime)) {
                /*
                 * 1.查找规划标识=input.getProactivePlanId(),时间维度在修正后预计开始时间至预计结束时间之间的记录.
                 *   如果这个时间范围的记录都存在,则什么都不做;
                 *   如果这个时间范围的记录部分不存在,则做成不存在的这个范围的记录,且风险值都置为0
                 * 2.更新修正前的时间范围下每天风险降低值对时间范围下各记录的影响;
                 * 3.根据新输入的时间范围，算出每一天风险降低值.
                 * 4.删除规划标识为1，规划的风险降低值为0的记录.
                 */
                ppPlanTraceMapper.insertTraceCaseAccordCondition(input.getProactivePlanId(), input.getPlanStartTime(), input.getPlanEndTime());
                double oldRickDownValue = ppPlanTraceMapper.getRickDownValueByDateRange(input.getPlanTaskId(), DateUtils.convertDatetoString(ppPlanTask.getPlanEndTime()), DateUtils.convertDatetoString(ppPlanTask.getPlanStartTime()));
                ppPlanTraceMapper.updateRiskDownValueByPlanIdAndDateRange(ppPlanTask.getPpPlanId(), input.getPlanTaskId(), DateUtils.convertDatetoString(ppPlanTask.getPlanStartTime()), DateUtils.convertDatetoString(ppPlanTask.getPlanEndTime()), -oldRickDownValue);
                double newRickDownValue = ppPlanTraceMapper.getRickDownValueByDateRange(input.getPlanTaskId(), input.getPlanEndTime(), input.getPlanStartTime());
                ppPlanTraceMapper.updateRiskDownValueByPlanIdAndDateRange(ppPlanTask.getPpPlanId(), input.getPlanTaskId(), input.getPlanStartTime(), input.getPlanEndTime(), newRickDownValue);
                ppPlanTraceMapper.deleteByPlanIdCaseAccordCondition(ppPlanTask.getPpPlanId());
            }
        }

        // 设置规划任务属性
        ppPlanTask.setPlanTaskName(input.getPlanTaskName());
        ppPlanTask.setPlanTgId(input.getTgId());
        ppPlanTask.setPlanStartTime(planStartTime);
        ppPlanTask.setPlanEndTime(planEndTime);
        ppPlanTask.setDescription(input.getDescription());

        // 更新规划任务
        ppPlanTaskMapper.updateByPrimaryKey(ppPlanTask);

        // 获取预防性规划任务责任人信息
        PpPlanTaskMember ppPlanTaskMember = ppPlanTaskMemberMapper.selectByPpPlanTaskId(input.getPlanTaskId());
        if (ppPlanTaskMember != null) {
            ppPlanTaskMemberMapper.deleteByPpPlanTaskId(input.getPlanTaskId());
        }

        // 构造预防性规划任务责任人对象
        ppPlanTaskMember = new PpPlanTaskMember();
        ppPlanTaskMember.setPpPlanTaskId(ppPlanTask.getPpPlanTaskId());
        ppPlanTaskMember.setUserId(input.getResponsibleId());
        ppPlanTaskMember.setProjectRoleId(ReliaFlowConstants.PROJECT_ROLE_PRINCIPAL_MEMBER_ID);
        ppPlanTaskMember.setInsertUserId(sysUser.getUsername());
        ppPlanTaskMember.setInsertPgmId(sysUser.getPgmId());

        // 插入责任人记录到表[T_PP_PLAN_TASK_MEMBER]
        ppPlanTaskMemberMapper.insert(ppPlanTaskMember);

        // 返回结果
        return Constants.SERVICE_SUCCESS;
    }
}
